import express, { application, NextFunction, Request, Response } from 'express';
const app = express();
import dotenv from 'dotenv';
dotenv.config({ path: './.env' });
import { config } from './config'
import webRoutes from './web/routes';
import { v4 } from "uuid";
import getErrorObject from './utills/errorStructure';
import dbPreLoad from './web/commonServices/dbPreLoad';
import { alertSystemCron, storyCron, sexOffenderScreeningCheckCron, datePageViewCountCron } from './web/commonServices/cronJobs';
import cors from 'cors';
import path from 'path';
import multer from 'multer'
const upload = multer();

app.use((req, res: any, next) => {
    let oldjson = res.json;
    res.json = function (data: any) {
        let responseHeader:
            {
                "error"?: any,
                "message"?: string,
                "requestId": any,
                "responseId": string,
                "status"?: string,
            } = {
            "requestId": req.body?.requestHeader?.requestId || v4(),
            "responseId": v4(),
            "error": {},
            "message": data.message ? data.message : '',
            "status": data.status ? data.status : 'ERROR',
        }
        if (data.error) {
            responseHeader['error'] = data.error;
        } else {
            responseHeader['error'] = getErrorObject();
        }
        delete data.error;
        delete data.message;
        delete data.status;
        let extendedData = { responseHeader, ...data };
        oldjson.call(res, extendedData);
    }
    next();
});

app.use(cors());
app.use(upload.any());
app.use(express.json());
app.use(express.urlencoded({ extended: false }));
app.use('/static', express.static(path.join(__dirname, '/public')));

app.use((req: Request, res: Response, next: NextFunction) => {
    res.header('Access-Control-Allow-Origin', '*');
    res.header('Access-Control-Allow-Headers', 'Origin, X-Requested-With, Content-Type, Accept, Authorization');

    if (req.method == 'OPTIONS') {
        res.header('Access-Control-Allow-Methods', 'PUT, POST, PATCH, DELETE, GET');
        return res.status(200).json({});
    } else {

    }
    next();
});

app.use(config.dev.api.prefix, webRoutes);

app.get('/ping', (req, res) => {
    res.status(200).json({ message: "Welcome to Date-Safe", status: 200 });
});

// import db from './models';
// db.sequelize.sync({
//     "force": false// Dangerous, force: true will truncate all tables
// }).then(() => {
//     app.listen(config.dev.server.port, () => {
//         console.log(`server is started at ${config.dev.server.port}`);
//     });
// }).catch(er => {
//     console.log(er)
// });
app.listen(config.dev.server.port, () => {
    console.log(`server is started at ${config.dev.server.port}`);
    dbPreLoad.loadData();
    storyCron.start();
    datePageViewCountCron.start();
    alertSystemCron.start();
    // dvsVerificationCheckCron.start();
    // idenfyVerificationCheckCron.start();
    sexOffenderScreeningCheckCron.start();
});
