'use strict';
import { Model } from 'sequelize';
export interface IQuestion {
  id?: number;
  name: string;
  isSearchable: number;
}
module.exports = (sequelize: any, DataTypes: any) => {
  class Question extends Model<IQuestion> implements IQuestion {
    id!: number;
    name!: string;
    isSearchable: number;
    /**
     * Helper method for defining associations.
     * This method is not a part of Sequelize lifecycle.
     * The `models/index` file will call this method automatically.
     */
    static associate(models: any) {
      // define association here
      Question.belongsTo(models.QuestionCategory, {
        foreignKey: {
          allowNull: false,
          name: 'quesCategoryId',
          field: 'quesCategoryId'
        }
      });
      Question.hasMany(models.UserCharacteristics, {
        foreignKey: 'questionId'
      });
      Question.hasMany(models.UserPreference, {
        foreignKey: 'questionId'
      });
    }
  }
  Question.init({
    id: {
      type: DataTypes.INTEGER,
      allowNull: false,
      primaryKey: true,
      autoIncrement: true
    },
    name: {
      type: DataTypes.STRING,
      allowNull: false
    },
    isSearchable: {
      type: DataTypes.INTEGER,
      defaultValue: false,
      allowNull: false
    }
  }, {
    sequelize,
    createdAt: false,
    updatedAt: false,
    modelName: 'Question'
  });
  return Question;
};
