'use strict';
import { Model } from "sequelize";
export interface IStory {
  id?: number;
  type: number;
  typeText: string;
  thumbnail: string;
  mediaPath: string;
  timestamp: number;
  expire: number;
  totalViewsCount: number;
  caption: string;
}
module.exports = (sequelize: any, DataTypes: any) => {
  class Story extends Model<IStory> implements IStory {
    id?: number;
    type!: number;
    typeText!: string;
    thumbnail!: string;
    mediaPath!: string;
    timestamp!: number;
    expire!: number;
    totalViewsCount!: number;
    caption!: string;
    /**
     * Helper method for defining associations.
     * This method is not a part of Sequelize lifecycle.
     * The `models/index` file will call this method automatically.
     */
    static associate(models: any) {
      // define association here
      Story.belongsTo(models.User);
      Story.hasMany(models.Reaction);
      Story.hasMany(models.Notification);
      Story.hasMany(models.StoryViews, {
        sourceKey: 'id',
        foreignKey: 'StoryId'
      });
      Story.hasMany(models.Comment);
    }
  }
  Story.init({
    id: {
      type: DataTypes.INTEGER,
      primaryKey: true,
      autoIncrement: true
    },
    type: {
      type: DataTypes.INTEGER,
      allowNull: false
    },
    typeText: {
      type: DataTypes.ENUM,
      values: ["IMAGE", "VIDEO", "TEXT"],
      allowNull: false
    },
    thumbnail: {
      type: DataTypes.STRING,
      allowNull: false
    },
    mediaPath: {
      type: DataTypes.STRING,
      allowNull: false
    },
    timestamp: {
      type: DataTypes.INTEGER,
      allowNull: false
    },
    expire: {
      type: DataTypes.INTEGER,
      allowNull: false
    },
    totalViewsCount: {
      type: DataTypes.INTEGER,
      defaultValue: 0
    },
    caption: {
      type: DataTypes.STRING,
      allowNull: true,
      defaultValue: ''
    }
  }, {
    sequelize,
    modelName: 'Story'
  });
  return Story;
};