import { Model } from 'sequelize';
export interface IUser {
  id?: string;
  userName: string;
  firstName: string;
  lastName: string;
  countryCode: string;
  phone: string;
  gender: string;
  email: string;
  password: string;
  DOB: string;
  birthYear: number;
  profilePicture: string;
  profileThumbnail: string;
  defaultNotification: string;
  profileBio: string;
  emailNotification: number;
  smsNotification: number;
  loginType: string;
  facebookId: string;
  googleId: string;
  appleId: string;
  isActive: number;
  isBlocked: number;
  isVerified: number;
  isDocumentVerified: number;
  isVideoVerified: number;
  termOfServicesConsent: number;
  alertSystemConsent: number;
  pgProfileId: string;
  fcmToken: string;
  themeColor: string;
  isLastSeenEnabled: number;
  isOnDate: number;
  onDateTimeStamp: number;
  onDateLong: string;
  onDateLat: string;
  datePageViewCount: number;
  userType: string;
  height: number;
  heightVerified: number;
  instagramToken: string;
  tiktokToken: string;
  spotifyToken: string;
  appleMusicToken: string;
  distanceUnit: string;
  heightUnit: string;
}
module.exports = (sequelize: any, DataTypes: any) => {
  class User extends Model<IUser> implements IUser {
    id?: string;
    userName!: string;
    firstName!: string;
    lastName!: string;
    countryCode!: string;
    phone!: string;
    gender!: string;
    email!: string;
    password!: string;
    DOB!: string;
    birthYear!: number;
    profilePicture!: string;
    profileThumbnail!: string;
    defaultNotification!: string;
    profileBio: string;
    emailNotification: number;
    smsNotification: number;
    loginType: string;
    facebookId: string;
    googleId: string;
    appleId: string;
    isActive!: number;
    isBlocked!: number;
    isVerified!: number;
    isDocumentVerified: number;
    isVideoVerified!: number;
    termOfServicesConsent!: number;
    alertSystemConsent!: number;
    isLastSeenEnabled: number;
    isOnDate!: number;
    onDateTimeStamp!: number;
    onDateLong!: string;
    onDateLat!: string;
    datePageViewCount!: number;
    pgProfileId: string;
    fcmToken!: string;
    themeColor!: string;
    userType!: string;
    height!: number;
    heightVerified!: number;
    instagramToken!: string;
    tiktokToken!: string;
    spotifyToken!: string;
    appleMusicToken!: string;
    distanceUnit!: string;
    heightUnit!: string;
    /**
     * Helper method for defining associations.
     * This method is not a part of Sequelize lifecycle.
     * The `models/index` file will call this method automatically.
     */
    static associate(models: any) {
      User.hasMany(models.Tag);
      User.hasMany(models.Follower, {
        foreignKey: 'toUser'
      });
      User.hasMany(models.Follower, {
        foreignKey: 'fromUser'
      });
      User.hasOne(models.Otp);
      User.hasOne(models.Location);
      User.hasMany(models.Post, {
        sourceKey: 'id',
        foreignKey: 'UserId'
      });
      User.hasMany(models.Reaction, {
        sourceKey: 'id',
        foreignKey: 'UserId'
      });
      User.hasMany(models.Story, {
        sourceKey: 'id',
        foreignKey: 'UserId'
      });
      User.hasMany(models.Reaction);
      User.hasMany(models.UserAnswer);
      User.hasMany(models.Comment);
      User.hasMany(models.EmergencyContact, {
        foreignKey: 'userId'
      });
      User.hasMany(models.UserTimeLogs, {
        foreignKey: 'userId'
      });
      User.hasMany(models.DatingPageMedia, {
        foreignKey: 'userId'
      });
      User.hasMany(models.UserCharacteristics, {
        foreignKey: 'userId'
      });
      User.hasMany(models.UserPreference, {
        foreignKey: 'userId'
      });
      User.hasMany(models.DatePageAction, {
        foreignKey: 'userId'
      });
      User.hasMany(models.DatePageAction, {
        foreignKey: 'actionUserId'
      });
      User.hasMany(models.UserHearAboutUsFrom, {
        foreignKey: 'userId'
      });
      User.belongsToMany(models.Interest, {
        through: "userInterest"
      });
      User.hasMany(models.ReportedUser, {
        foreignKey: 'userId'
      });
      User.hasMany(models.ReportedUser, {
        foreignKey: 'toUserId'
      });
      User.hasMany(models.StoryPrivacy, {
        foreignKey: 'hideByUserId'
      });
      User.hasMany(models.StoryPrivacy, {
        foreignKey: 'hideForUserId'
      });
      User.hasMany(models.SubscribedUser);
      User.hasMany(models.BlockUser, {
        foreignKey: 'blockByUserId'
      });
      User.hasMany(models.BlockUser, {
        foreignKey: 'blockForUserId'
      });
      User.hasOne(models.DistanceMatch, {
        foreignKey: 'userId'
      });
      User.hasMany(models.DocumentVerificationLogs, {
        foreignKey: 'userId'
      });
      User.hasMany(models.IdenfyLogs, {
        foreignKey: 'userId'
      });
      User.hasMany(models.PaymentGatewayLogs, {
        foreignKey: 'userId'
      });
      User.hasMany(models.UserToken, {
        foreignKey: 'userId'
      });
      User.hasMany(models.UserPurchase, {
        foreignKey: 'userId'
      });
      User.hasMany(models.UserPrompts, {
        foreignKey: 'userId'
      });
      User.hasMany(models.SignUpCheckList, {
        foreignKey: 'userId'
      });
      User.hasMany(models.NotificationSettings, {
        foreignKey: 'userId'
      });
      User.hasOne(models.AgeRanges, {
        foreignKey: 'userId'
      });
      User.hasMany(models.UserOnlineLogs, {
        foreignKey: 'userId'
      });
      User.hasMany(models.SpotifyPlaylists, {
        foreignKey: 'userId'
      });
    }
  }
  User.init({
    id: {
      type: DataTypes.INTEGER,
      allowNull: false,
      primaryKey: true,
      autoIncrement: true
    },
    userName: {
      type: DataTypes.STRING,
      allowNull: false
    },
    firstName: {
      type: DataTypes.STRING,
      allowNull: true
    },
    lastName: {
      type: DataTypes.STRING,
      allowNull: true
    },
    countryCode: {
      type: DataTypes.STRING,
      defaultValue: "",
      allowNull: false
    },
    phone: {
      type: DataTypes.STRING,
      allowNull: true,
      defaultValue: null
    },
    gender: {
      type: DataTypes.STRING,
      allowNull: true
    },
    email: {
      type: DataTypes.STRING,
      allowNull: false
    },
    password: {
      type: DataTypes.STRING,
      allowNull: false
    },
    DOB: {
      type: DataTypes.STRING,
      allowNull: true
    },
    birthYear: {
      type: DataTypes.INTEGER,
      allowNull: true
    },
    profilePicture: {
      type: DataTypes.STRING,
      allowNull: true
    },
    profileThumbnail: {
      type: DataTypes.STRING,
      allowNull: true
    },
    defaultNotification: {
      type: DataTypes.ENUM('SMS', 'EMAIL'),
      allowNull: true
    },
    profileBio: {
      type: DataTypes.STRING,
      allowNull: true
    },
    emailNotification: {
      type: DataTypes.INTEGER,
      defaultValue: 1,
      allowNull: false
    },
    smsNotification: {
      type: DataTypes.INTEGER,
      defaultValue: 0,
      allowNull: false
    },
    loginType: {
      type: DataTypes.ENUM("NORMAL", "FACEBOOK", "GOOGLE", "APPLE"),
      allowNull: false,
      defaultValue: "NORMAL"
    },
    googleId: {
      type: DataTypes.STRING,
      allowNull: false,
      defaultValue: ""
    },
    facebookId: {
      type: DataTypes.STRING,
      allowNull: false,
      defaultValue: ""
    },
    appleId: {
      type: DataTypes.STRING,
      allowNull: false,
      defaultValue: ""
    },
    isActive: {
      type: DataTypes.INTEGER,
      defaultValue: 1,
      allowNull: false
    },
    isBlocked: {
      type: DataTypes.INTEGER,
      defaultValue: 0,
      allowNull: false
    },
    isVerified: {
      type: DataTypes.INTEGER,
      defaultValue: 0,
      allowNull: false
    },
    isDocumentVerified: {
      type: DataTypes.INTEGER,
      defaultValue: 0,
      allowNull: false
    },
    isVideoVerified: {
      type: DataTypes.INTEGER,
      defaultValue: 0,
      allowNull: false
    },
    termOfServicesConsent: {
      type: DataTypes.INTEGER,
      defaultValue: 0,
      allowNull: false
    },
    alertSystemConsent: {
      type: DataTypes.INTEGER,
      defaultValue: 0,
      allowNull: false
    },
    pgProfileId: {
      type: DataTypes.STRING,
      allowNull: false,
      defaultValue: ''
    },
    fcmToken: {
      type: DataTypes.STRING,
      allowNull: true
    },
    themeColor: {
      type: DataTypes.STRING,
      defaultValue: '',
      allowNull: true
    },
    isLastSeenEnabled: {
      type: DataTypes.INTEGER,
      defaultValue: 0,
      allowNull: false
    },
    isOnDate: {
      type: DataTypes.INTEGER,
      defaultValue: 0,
      allowNull: false
    },
    onDateTimeStamp: {
      type: DataTypes.INTEGER,
      allowNull: false,
      defaultValue: 0
    },
    onDateLat: {
      type: DataTypes.STRING,
      allowNull: false,
      defaultValue: "0"
    },
    onDateLong: {
      type: DataTypes.STRING,
      allowNull: false,
      defaultValue: "0"
    },
    datePageViewCount: {
      type: DataTypes.INTEGER,
      allowNull: false,
      defaultValue: 0
    },
    userType: {
      type: DataTypes.ENUM("USER", "ADMIN"),
      defaultValue: "USER",
      allowNull: false
    },
    height: {
      type: DataTypes.INTEGER,
      allowNull: true,
      defaultValue: null
    },
    heightVerified: {
      type: DataTypes.INTEGER,
      allowNull: false,
      defaultValue: 0
    },
    instagramToken: {
      type: DataTypes.STRING,
      allowNull: true
    },
    spotifyToken: {
      type: DataTypes.STRING,
      allowNull: true
    },
    tiktokToken: {
      type: DataTypes.STRING,
      allowNull: true
    },
    appleMusicToken: {
      type: DataTypes.STRING,
      allowNull: true
    },
    distanceUnit: {
      type: DataTypes.ENUM("MI", "KM"),
      allowNull: false,
      defaultValue: "MI"
    },
    heightUnit: {
      type: DataTypes.ENUM("FT", "CM"),
      allowNull: false,
      defaultValue: "FT"
    },
  }, {
    sequelize,
    modelName: 'User'
  });
  return User;
};
