import { Logging } from "../web/commonServices/logging.service";
import FCM from 'fcm-node';
import { config } from "../config";
import userService from "../web/routes/users/user.service";
import NotificationService from "../web/routes/notifications/notification.service";
import moment from "moment";
const CryptoJS = require("cryp-to-js").default;

export class Utils {
    public static base64Decode = (data: string) => {
        // return Buffer.from(data, 'base64').toString('ascii');
        return data;
    };
    public static base64Encode = (data: string) => {
        return Buffer.from(data).toString('base64');
    };
    public static randomDigits = function (length: number) {
        var text = "";
        var possible = "123456789";
        for (var i = 0; i < length; i++) {
            var sup = Math.floor(Math.random() * possible.length);
            text += i > 0 && sup == i ? "0" : possible.charAt(sup);
        }
        return Number(text);
    };

    public static getOptMailHtml = (otp: number) => {
        return `<p>Please Confirm your Account.</p><p>OTP: ${otp}</p>`;
    };

    public static getOtpSMSText = (otp: number) => {
        return `${otp} is your Datesafe verification code.`;
    };

    public static getLoginInfoHtml = (email: string, password: string) => {
        return `<p>Here is your Login Information.</p><p>Email: ${email}</p><p>Password: ${password}</p>`;
    };

    public static getDateEmegencyAlertHtml = (toName: string, fromName: string, dateLatLong: any, currentLatLong: any) => {
        return `<p> Hello ${toName}, </p>` +
            `<p> This is ${fromName} here, </p>` +
            `<p>I was gone to date at Latitude: ${dateLatLong.latitude} and Logitude: ${dateLatLong.longitude}, ` +
            ` and Currently at Latitude: ${currentLatLong.latitude} and Logitude: ${currentLatLong.longitude}. </p>`;
    };

    public static maskEmail = (email: string) => {
        let splittedEmail = email.split('@');
        return email[0] + email[1] + Array(splittedEmail[0].length - 2).join('*') + splittedEmail[0][splittedEmail[0].length - 1] + '@' + splittedEmail[1];
    };

    public static parseJwt = (token: string) => {
        var base64Payload = token.split('.')[1];
        var payload = Buffer.from(base64Payload, 'base64');
        return JSON.parse(payload.toString());
    };

    public static sendNotification = async (configData: any) => {
        const fcm = new FCM(config.dev.fcm.server_key);
        let toUserData = await userService.getUserById(configData.toUserId, ['fcmToken', 'userName', 'profileThumbnail']);
        let fromUserData = await userService.getUserById(configData.fromUserId, ['userName', 'profileThumbnail']);

        await NotificationService.addNotifications({
            fromUser: configData.fromUserId,
            toUser: configData.toUserId,
            timeStamp: moment().unix(),
            expirationTimeStamp: moment().add(10, 'days').unix(),
            type: configData.type
        });

        const message = {
            to: toUserData.fcmToken,
            notification: {
                title: 'Datesafe',
                body: `${fromUserData.userName} ${configData.message}`,
                sound: "ping.aiff",
                imageUrl: fromUserData.profileThumbnail,
                delivery_receipt_requested: true,
                ...configData.notificationData
            },
            data: {
                message: `${fromUserData.userName} ${configData.message}`
            }
        };
        fcm.send(message, (d) => {
            Logging.logMessage(configData.message);
        });
    };

    public static findDistanceAmongUsersInKms = (latLongFrom, latLongTo) => {
        var p = Math.PI / 180;    // Math.PI / 180
        var c = Math.cos;
        const R = 6371;   // total radius of earth
        var a = 0.5 - c((parseFloat(latLongTo.latitude) - parseFloat(latLongFrom.latitude)) * p) / 2 +
            c(parseFloat(latLongFrom.latitude) * p) * c(parseFloat(latLongTo.latitude) * p) *
            (1 - c((parseFloat(latLongTo.longitude) - parseFloat(latLongFrom.longitude)) * p)) / 2;
        return (2 * R * Math.asin(Math.sqrt(a))).toFixed(2); // 2 * R; R = 6371 km
    };

    public static findDistanceAmongUsersInMiles = (latLongFrom, latLongTo) => {
        const distanceInKms = this.findDistanceAmongUsersInKms(latLongFrom, latLongTo);
        return (parseFloat(distanceInKms) / 1.609).toFixed(2);
    };

    
    public static decryptedAES(encrypted: string, keyType?: number) {
        
        let ivKey = process.env.EN_DC_IVKEY;
        let aeskey = process.env.EN_DC_AESKEY;
   
        const iv = CryptoJS.enc.Utf8.parse(ivKey);
        const newkey = CryptoJS.enc.Utf8.parse(aeskey);
        const decrypted = CryptoJS.AES.decrypt(encrypted, newkey,
            {
                iv,
                mode: CryptoJS.mode.CBC,
                padding: CryptoJS.pad.Pkcs7,
            });
            
        return decrypted.toString(CryptoJS.enc.Utf8);
    }

    public static encryptedAES(data, keyType?: number) {
        let ivKey = process.env.EN_DC_IVKEY;
        let aeskey = process.env.EN_DC_AESKEY;

        let iv = CryptoJS.enc.Utf8.parse(ivKey);
        let newkey = CryptoJS.enc.Utf8.parse(aeskey);
        
        const encrypted = CryptoJS.AES.encrypt(data, newkey,
            {
                iv,
                mode: CryptoJS.mode.CBC,
                padding: CryptoJS.pad.Pkcs7,
            });
        return encrypted.ciphertext.toString(CryptoJS.enc.Base64);
    }
}