import { config } from "../config";
import { EmailService } from "./enums";
import nodemailer from 'nodemailer';
import sendgridMail from '@sendgrid/mail';
import { Constants } from "./constants";
import { Logging } from "../web/commonServices/logging.service";

const emailService = process.env.EMAIL_SERVICE;

export const sendMail = function (from: string, to: string, subject: string, html: string) {
	switch (emailService) {
		case EmailService.SMTP:
			// create reusable transporter object using the default SMTP transport
			let transporter = nodemailer.createTransport({
				host: config.dev.smtp.email_smtp_host,
				port: config.dev.smtp.email_smtp_port,
				//secure: process.env.EMAIL_SMTP_SECURE, // lack of ssl commented this. You can uncomment it.
				auth: {
					user: config.dev.smtp.from_email,
					pass: config.dev.smtp.email_password
				}
			});
			return transporter.sendMail({
				from: from,
				to: to,
				subject: subject,
				html: html
			});
			break;
		case EmailService.SENDGRID:

			sendgridMail.setApiKey(process.env.SENDGRID_API_KEY);
			return sendgridMail.send({
				to: to,
				from: `${Constants.APP_NAME} <${process.env.SENDGRID_MAIL_ID}>`,
				subject: subject,
				html: html,
			}).then(() => { }).catch((error) => {
				Logging.logMessage(`SENDGRID mail error..${error}`)
			});
			break;

		default:
			break;
	}
};