import Twilio from 'twilio';
import { SmsService } from './enums';
import { Logging } from '../web/commonServices/logging.service';
import request from 'request-promise';

const smsService = process.env.SMS_SERVICE;
const smsEnabled = process.env.SMS_ENABLED;

export const sendSMS = async function (sendToCountryCode: string, sendToNumber: string, smsText: string) {
    switch (smsService) {
        case SmsService.TWILIO:
            const accountSid = process.env.TWILIO_ACCOUNT_SID;
            const authToken = process.env.TWILIO_AUTH_TOKEN;
            const messagingSid = process.env.TWILIO_MSG_SERVICE_ID;
            const fromNumber = process.env.TWILIO_FROM_NUMBER;

            const smsClient = Twilio(accountSid, authToken)

            if (smsEnabled) {
                return smsClient.messages.create({
                    body: smsText,
                    from: fromNumber,
                    messagingServiceSid: messagingSid,
                    to: sendToCountryCode + sendToNumber
                }).then((message) => { return message.sid })
                    .catch(err => Logging.logMessage(`TWILIO SMS error.. ${err} `));
            } else {
                return;
            }
            break;
        case SmsService.SINCH:
            const SERVICE_PLAN_ID = process.env.SINCH_SERVICE_PLAN_ID;
            const API_TOKEN = process.env.SINCH_API_TOKEN;
            const SINCH_NUMBER = process.env.SINCH_PHONE;
            const SINCH_URL = process.env.SINCH_API_URL;
            const toNumber = sendToCountryCode + sendToNumber;

            if (smsEnabled) {
                try {
                    const options = {
                        uri: `${SINCH_URL}/${SERVICE_PLAN_ID}/batches`,
                        headers: {
                            'Content-Type': 'application/json',
                            Authorization: "Bearer " + API_TOKEN
                        },
                        json: true,
                        body: {
                            "from": SINCH_NUMBER,
                            "to": [toNumber],
                            "body": smsText
                        }
                    }
                    const response = await request.post(options);
                } catch (err) {
                    Logging.logMessage(`SINCH SMS error.. ${err} `);
                }
            } else {
                return;
            }
            break;
    
        default:
            break;
    }
}