import jwt from 'jsonwebtoken';
import { config } from '../config';
import { Logging } from '../web/commonServices/logging.service';
import LoginSignupService from '../web/routes/loginSignup/loginSignup.service';
import UserService from '../web/routes/users/user.service';
import { Utils } from './helperMethods';
interface IUserInfo {
    name: String,
    id: String,
    email: String,
    userType: String,
    issueTimeStamp: Number;
}

export const jwtSign = (userData: IUserInfo) => {
    try {
        return jwt.sign(userData, config.dev.jwt.secret);
    } catch (error: any) {
        Logging.logGeneralError(error);
    }
}

export const getParsedToken = async (token: string) => {
    try {
        if (token) {
            if (jwt.verify(token, config.dev.jwt.secret)) {
                let user = Utils.parseJwt(token);
                if (
                    (await UserService.exists({
                        'email': user.email,
                        'id': user.id
                    })) && (await LoginSignupService.isValidToken(user.id, user.userType, token))
                ) {
                    return user;
                }
            }
        }
        return null;
    } catch (error: any) {
        Logging.logGeneralError(error);
        return null;
    }
}