import { FolderNames, ImageExentions, MediaSources } from "../../utills/enums";
import fs from 'fs';
import path from 'path';
import ffmpeg from 'ffmpeg-static';
import genThumbnail from 'simple-thumbnail';
import { v4 } from "uuid";
import { Logging } from "./logging.service";
interface fileUploadType {
    type: number;
    userId: string;
    itemId: string | null;
    files: any[] | any;
}

interface fileDeleteType {
    type: number;
    userId: string;
    itemId: string | null;
    filePath: string;
}
export const handleFileUpload = (data: fileUploadType) => {
    return new Promise((resolve, reject) => {
        let destination = '';
        if (data.type === MediaSources.POST) {
            destination = path.join(__dirname, '../../public/users/' + data.userId + '/' + FolderNames.POST + '/' + data.itemId);
        } else if (data.type === MediaSources.STORY) {
            destination = path.join(__dirname, '../../public/users/' + data.userId + '/' + FolderNames.STORIES + '/');
        } else if (data.type === MediaSources.DATING_PAGE_MEDIA) {
            destination = path.join(__dirname, '../../public/users/' + data.userId + '/' + FolderNames.DATING_PAGE_MEDIA + '/');
        } else if (data.type === MediaSources.PROFILE_PICTURE) {
            destination = path.join(__dirname, '../../public/users/' + data.userId + '/' + FolderNames.PROFILE_PICTURE + '/');
        }
        if (!fs.existsSync(destination)) {
            fs.mkdirSync(destination, { recursive: true })
        }
        let filePaths = [];
        if (Array.isArray(data.files)) {
            let fileName;
            for (let file of data.files) {
                fileName = v4();
                let fileExt = path.extname(file.originalname);
                filePaths.push(`${fileName}${fileExt}`);
                fs.writeFileSync(`${destination}/${fileName}${fileExt}`, file.buffer);
            }
        }
        resolve(filePaths);
    });
};

export const handleFileDelete = (data: fileDeleteType) => {
    return new Promise<void>((resolve, reject) => {
        let destination = '';

        if (data.type === MediaSources.POST) {
            destination = path.join(__dirname, '../../public/users/' + data.userId + '/' + FolderNames.POST + '/' + data.itemId + '/');
        } else if (data.type === MediaSources.STORY) {
            destination = path.join(__dirname, '../../public/users/' + data.userId + '/' + FolderNames.STORIES + '/');
        } else if (data.type === MediaSources.DATING_PAGE_MEDIA) {
            destination = path.join(__dirname, '../../public/users/' + data.userId + '/' + FolderNames.DATING_PAGE_MEDIA + '/');
        } else if (data.type === MediaSources.PROFILE_PICTURE) {
            destination = path.join(__dirname, '../../public/users/' + data.userId + '/' + FolderNames.PROFILE_PICTURE + '/');
        }
        try {
            fs.unlinkSync(destination + data.filePath);
        } catch (error: any) {
        }

        return resolve();
    });
};

export const handleImageVideoThumb = (data: fileUploadType) => {
    return new Promise(async (resolve, reject) => {
        let destination = '';
        if (data.type === MediaSources.POST) {
            destination = path.join(__dirname, '../../public/users/' + data.userId + '/' + FolderNames.POST + '/' + data.itemId);
        } else if (data.type === MediaSources.STORY) {
            destination = path.join(__dirname, '../../public/users/' + data.userId + '/' + FolderNames.STORIES);
        } else if (data.type === MediaSources.DATING_PAGE_MEDIA) {
            destination = path.join(__dirname, '../../public/users/' + data.userId + '/' + FolderNames.DATING_PAGE_MEDIA);
        } else if (data.type === MediaSources.PROFILE_PICTURE) {
            destination = path.join(__dirname, '../../public/users/' + data.userId + '/' + FolderNames.PROFILE_PICTURE);
        }
        if (!fs.existsSync(destination)) {
            fs.mkdirSync(destination, { recursive: true })
        }
        let filePaths = [];
        if (Array.isArray(data.files)) {
            let fileName;
            for (let file of data.files) {
                fileName = 'thumb_' + path.parse(file).name;
                let fileExt = ImageExentions.PNG;

                try {
                    await genThumbnail(`${destination}/${file}`,
                        `${destination}/${fileName}${fileExt}`, '250x250',
                        {
                            path: ffmpeg
                        });
                    filePaths.push(`${fileName}${fileExt}`);
                } catch (err) {
                    Logging.logMessage(err);
                }

            }
        }
        resolve(filePaths);
    });
};