import request from 'request-promise';
import { Logging } from './logging.service';

const getIdenfyToken = async () => new Promise((resolve, reject) => {
    const options = {
        uri: `${process.env.IDENFY_VERIFY_URL}/token`,
        headers: {
            'Content-Type': 'application/json',
            Authorization: "Basic " + Buffer.from((process.env.IDENFY_USERNAME + ":" + process.env.IDENFY_PASS)).toString("base64")
        },
        json: true,
        body: {
            "clientId": Math.floor(Math.random() * 1000000000).toString(),
            "documents": JSON.parse(process.env.IDENFY_VALID_DOCUMENTS),
            "maxAttemptCount": process.env.IDENFY_MAX_ATTEMPTS,
            "checkDuplicateFaces": true,
            "checkDuplicateDocFaces": true,
        }
    }
    return request.get(options)
        .then((data) => resolve(data))
        .catch((err) => {
            Logging.logMessage(`IDENFY TOKEN Err... ${err.message}`);
            reject(err.message);
        });
});

const getIdenfyStatus = async (scanRef: string) => new Promise((resolve, reject) => {
    const options = {
        uri: `${process.env.IDENFY_VERIFY_URL}/status`,
        headers: {
            'Content-Type': 'application/json',
            Authorization: "Basic " + Buffer.from((process.env.IDENFY_USERNAME + ":" + process.env.IDENFY_PASS)).toString("base64")
        },
        json: true,
        body: {
            "scanRef": scanRef
        }
    }
    return request.post(options)
        .then((data) => resolve(data))
        .catch((err) => {
            Logging.logMessage(`IDENFY STATUS Err... ${err.message}`);
            reject(err.message);
        });
});

const getIdenfyFiles = async (scanRef: string) => new Promise((resolve, reject) => {
    const options = {
        uri: `${process.env.IDENFY_VERIFY_URL}/files`,
        headers: {
            'Content-Type': 'application/json',
            Authorization: "Basic " + Buffer.from((process.env.IDENFY_USERNAME + ":" + process.env.IDENFY_PASS)).toString("base64")
        },
        json: true,
        body: {
            "scanRef": scanRef
        }
    }
    return request.post(options)
        .then((data) => resolve(data))
        .catch((err) => {
            Logging.logMessage(`IDENFY FILES Err... ${err.message}`);
            reject(err.message);
        });
});

const getIdenfyData = async (scanRef: string) => new Promise((resolve, reject) => {
    const options = {
        uri: `${process.env.IDENFY_VERIFY_URL}/data`,
        headers: {
            'Content-Type': 'application/json',
            Authorization: "Basic " + Buffer.from((process.env.IDENFY_USERNAME + ":" + process.env.IDENFY_PASS)).toString("base64")
        },
        json: true,
        body: {
            "scanRef": scanRef
        }
    }
    return request.post(options)
        .then((data) => resolve(data))
        .catch((err) => {
            Logging.logMessage(`IDENFY DATA Err... ${err.message}`);
            reject(err.message);
        });
});

const deleteIdenfyData = async (scanRef: string) => new Promise((resolve, reject) => {
    const options = {
        uri: `${process.env.IDENFY_VERIFY_URL}/delete`,
        headers: {
            'Content-Type': 'application/json',
            Authorization: "Basic " + Buffer.from((process.env.IDENFY_USERNAME + ":" + process.env.IDENFY_PASS)).toString("base64")
        },
        json: true,
        body: {
            "scanRef": scanRef
        }
    }
    return request.post(options)
        .then((data) => resolve(data))
        .catch((err) => {
            Logging.logMessage(`IDENFY DATA Err... ${err.message}`);
            reject(err.message);
        });
});


export {
    getIdenfyToken,
    getIdenfyStatus,
    getIdenfyFiles,
    getIdenfyData,
    deleteIdenfyData
}