import { NextFunction, Request, Response } from "express";
import { HTTPStatusCode } from "../../utills/enums";
import { StaticMessages } from "../../utills/staticMessage";
import { getParsedToken } from "../../utills/token";

interface CustomRequest extends Request {
    user?: any; // Replace 'any' with the actual type of 'user' if possible
}

export const tokenValidation = async (req: CustomRequest, res: Response, next: NextFunction) => {
    let parsedToken = await getParsedToken(<string>req.header('Access-Token'));
    if (parsedToken) {
        req['user'] = parsedToken;
        next();
    } else {
        res.status(HTTPStatusCode.SUCCESS).json({ 
            message: StaticMessages.INVALID_ACCESS_TOKEN, 
            error: {
                statusCode: HTTPStatusCode.BAD_REQUEST,
                message: StaticMessages.INVALID_ACCESS_TOKEN
            } 
        });
    }
};