import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import AgeRangesService from "./ageRanges.service";
import getErrorObject from "../../../utills/errorStructure";
import { CustomError } from "../../../utills/types";

const getUserAgeRamge = async (req: Request, res: Response) => {
    try {
        const authUser = req['user'];

        const ageRangeData = await AgeRangesService.getAgeRangeByUserId(authUser.id)
        if (!ageRangeData) throw new CustomError(StaticMessages.getNotExistMessage('Age Range'), HTTPStatusCode.NOTFOUND)

        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getListReturnedMessage('Age Range'),
            ageRangeData: ageRangeData
        });
    } catch (err) {
        if (err instanceof Error) {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: StaticMessages.INTERNAL_SERVER_ERROR
            });
        } else {
            err = getErrorObject(err)
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: err.message
            });
        }
    }
};

const updateUserAgeRange = async (req: Request, res: Response) => {
    try {
        const authUser = req['user'];

        await AgeRangesService.updateAgeRangeByUserId(authUser.id, req.body);

        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getUpdatedMessage('Age Range')
        });
    } catch (err) {
        if (err instanceof Error) {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: StaticMessages.INTERNAL_SERVER_ERROR
            });
        } else {
            err = getErrorObject(err)
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: err.message
            });
        }
    }
};

export default {
    getUserAgeRamge,
    updateUserAgeRange
}