import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const updateAgeRageValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("User Age Rage -> Update Age Rage")
    try {
        const schema = Joi.object().keys({
            minAge: Joi.number().required().description('minAge: 15').error(new Error('minAge is missing or incoreect')),
            maxAge: Joi.number().required().description('maxAge: 25').error(new Error('maxAge is missing or incoreect')),
            isEnabled: Joi.bool().required().description('isEnabled: true or false').error(new Error('isEnabled is missing or incoreect')),
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    updateAgeRageValidator
}