import db from "../../../models";
const BlockUser = db.BlockUser;
const DatePageAction = db.DatePageAction;
import { Op, Sequelize } from "sequelize";

const createBlockUser = async (data: any) => {
    try {
        await DatePageAction.destroy({
            where: {
                [Op.or] : [{
                    userId: data.blockByUserId,
                    actionUserId: data.blockForUserId
                }, {
                    actionUserId: data.blockByUserId,
                    userId: data.blockForUserId
                }]
            }
        });
        return await BlockUser.create(data);
    } catch (err: any) {
        throw err
    }
};

const deleteBlockUserByUserId = async (id ,userId: number) => {
    try {
        return await BlockUser.destroy({
            where: {
                blockByUserId: id,
                blockForUserId: userId
            }
        });
    } catch (err: any) {
        throw err
    }
};

const getBlockUserByUserId = async (userId: number) => {
    try {
        const blockedUsers = await BlockUser.findAll({
            where: {
                blockByUserId: userId
            },
            include: [{
                model: db.User,
                as: 'User',
                attributes: [],
                on: Sequelize.literal('User.id = BlockUser.blockForUserId')
            }],
            attributes: [
                'id',
                'blockByUserId',
                'blockForUserId',
                [Sequelize.col('User.userName'), 'blockedUserName'],
                [Sequelize.col('User.firstName'), 'blockedUserFirstName'],
                [Sequelize.col('User.lastName'), 'blockedUserLastName'],
                [Sequelize.col('User.countryCode'), 'blockedUserCountryCode'],
                [Sequelize.col('User.phone'), 'blockedUserPhone'],
                [Sequelize.col('User.gender'), 'blockedUserGender'],
                [Sequelize.col('User.DOB'), 'blockedUserDOB'],
                [Sequelize.col('User.userType'), 'blockedUserType'],
                [Sequelize.col('User.profilePicture'), 'blockedUserProfilePicture'],
            ],
            raw: true
        });

        return blockedUsers;
    } catch (err: any) {
        throw err
    }
};

const getBlocksForUserId = async (userId: number) => {
    try {
        const blockedUsers = await BlockUser.findAll({
            where: {
                blockForUserId: userId
            },
            // include: [{
            //     model: db.User,
            //     as: 'User',
            //     attributes: [],
            //     on: Sequelize.literal('User.id = BlockUser.blockByUserId')
            // }],
            // attributes: [
            //     'id',
            //     'blockByUserId',
            //     'blockForUserId',
            //     [Sequelize.col('User.userName'), 'blockedByUserName'],
            //     [Sequelize.col('User.firstName'), 'blockedByUserFirstName'],
            //     [Sequelize.col('User.lastName'), 'blockedByUserLastName'],
            //     [Sequelize.col('User.countryCode'), 'blockedByUserCountryCode'],
            //     [Sequelize.col('User.phone'), 'blockedByUserPhone'],
            //     [Sequelize.col('User.gender'), 'blockedByUserGender'],
            //     [Sequelize.col('User.DOB'), 'blockedByUserDOB'],
            //     [Sequelize.col('User.userType'), 'blockedByUserType'],
            //     [Sequelize.col('User.profilePicture'), 'blockedByUserProfilePicture'],
            // ],
            raw: true
        });

        return blockedUsers;
    } catch (err: any) {
        throw err
    }
};

const getBlockedUserByUserIdAndForUserId = async (blockByUserId: number, blockForUserId: number) => {
    try {
        return await BlockUser.findOne({
            where: {
                blockByUserId: blockByUserId,
                blockForUserId: blockForUserId
            },
            raw: true
        });
    } catch (err: any) {
        throw err
    }
};

export default {
    createBlockUser,
    deleteBlockUserByUserId,
    getBlockUserByUserId,
    getBlockedUserByUserIdAndForUserId,
    getBlocksForUserId
};