import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addOrDeleteBlockUserValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Block User Validator  -> Add UserBlock");
    try {
        const schema = Joi.object().keys({
            userId: Joi.number().required().description('userIds: integer id of user').error(new Error('userId is missing or incorrect')),
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addOrDeleteBlockUserValidator
}