import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import ChoicesService from "./choices.service";
import QuestionCategoryService from "../questionCategory/questionCategory.service";
import { Logging } from "../../commonServices/logging.service";

const addNewChoices = (req: Request, res: Response) => {
    const checkQuesCategoryExistOrNot = () => new Promise<void>((resolve, reject) => {
        return QuestionCategoryService.getOneQuestionCategoryById(req.body.questionCategoryId)
            .then((data) => {
                if (!data) reject({ message: StaticMessages.getNotExistMessage('Question Category'), status: HTTPStatusCode.NOTFOUND });
                resolve();
            })
            .catch(err => reject(err))
    });

    const addChoices = () => new Promise((resolve, reject) => {
        const insertData = {
            name: req.body.name,
            quesCategoryId: req.body.questionCategoryId
        };
        return ChoicesService.addChoices(insertData)
            .then((data) => resolve(data))
            .catch(err => reject(err))
    });

    return checkQuesCategoryExistOrNot()
        .then(addChoices)
        .then(() => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getCreatedMessage('Choice')
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

const getChoicesList = (req: Request, res: Response) => {
    const getChoices = () => new Promise((resolve, reject) => {
        return ChoicesService.getChoiceWithOutCond()
            .then((data) => {
                if (data.length > 0) {
                    data.map((dt)=> {
                        delete dt.QuestionCategory
                        delete dt['QuestionCategory.name'];
                        return dt;
                    })
                    resolve(data);
                }
                reject({ message: StaticMessages.getNotExistMessage('Choices'), status: HTTPStatusCode.NOTFOUND });
            })
            .catch(err => reject(err))
    })

    return getChoices()
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Choices'),
                choices: data
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
}

const getChoicesListByQuesCategoryId = (req: Request, res: Response) => {
    const getChoices = () => new Promise((resolve, reject) => {
        return ChoicesService.getChoicesByQuestionCategoryId(String(req.query.quesCategoryId))
            .then((data) => {
                if (data.length > 0) {
                    data.map((dt)=> {
                        delete dt.QuestionCategory
                        delete dt['QuestionCategory.name'];
                        return dt;
                    })
                    resolve(data);
                }
                reject({ message: StaticMessages.getNotExistMessage('Choices'), status: HTTPStatusCode.NOTFOUND });
            })
            .catch(err => reject(err))
    })

    return getChoices()
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Choices'),
                choices: data
            });
        }).catch((err) => {
            Logging.logMessage(err);
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
}

export default {
    addNewChoices,
    getChoicesList,
    getChoicesListByQuesCategoryId
}