import db from "../../../models";
import sequelize from 'sequelize';

const Choice = db.Choice;

const addChoices = async (data: any) => {
    try {
        return await Choice.create(data);
    } catch (err: any) {
        throw err;
    }
};

const getChoicesByChoiceId = async (id: string) => {
    try {
        return await Choice.findOne({
            where: {
                id: id
            }
        });
    } catch (err: any) {
        throw err;
    }
};

const getChoiceWithOutCond = async () => {
    try {
        return await Choice.findAll({
            include: [
                {
                    model: db.QuestionCategory,
                    as: 'QuestionCategory',
                    attributes: ["name"]
                }
            ],
            attributes: [
                "id",
                "name",
                "quesCategoryId",
                [sequelize.col('QuestionCategory.name'), 'quesCategoryName'],
            ],
            order:[['name', 'ASC']],
            raw: true
        });
    } catch (err: any) {
        throw err;
    }
};

const getChoicesByQuestionCategoryId = async (id: string) => {
    try {
        return await Choice.findAll({
            where: {
                quesCategoryId: id
            },
            include: [
                {
                    model: db.QuestionCategory,
                    as: 'QuestionCategory',
                    attributes: ["name"]
                }
            ],
            attributes: [
                "id",
                "name",
                "quesCategoryId",
                [sequelize.col('QuestionCategory.name'), 'quesCategoryName'],
            ],
            raw: true
        });
    } catch (err: any) {
        throw err;
    }
};

export default {
    addChoices,
    getChoicesByChoiceId,
    getChoiceWithOutCond,
    getChoicesByQuestionCategoryId
}