import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import getErrorObject from "../../../utills/errorStructure";
import CountryCodesService from "./countryCodes.service";

const getCountryCodes = async (req: Request, res: Response) => {
    try {
        const countryData = await CountryCodesService.getCountries()
        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getListReturnedMessage('Country List'),
            countryList: countryData
        });
    } catch (err) {
        if (err instanceof Error) {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: StaticMessages.INTERNAL_SERVER_ERROR
            });
        } else {
            err = getErrorObject(err)
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: err.message
            });
        }
    }
};

export default {
    getCountryCodes
}