import db from "../../../models";
import { Utils } from "../../../utills/helperMethods";
import { DatePageActions, FolderNames, HTTPStatusCode, NotificationType } from "../../../utills/enums";
import { Sequelize } from "sequelize";
import { CustomError } from "../../../utills/types";
import { StaticMessages } from "../../../utills/staticMessage";
const DatePageAction = db.DatePageAction;

const createAction = async (data: any) => {
    try {
        if(data.userId === data.actionUserId) {
            throw new CustomError(StaticMessages.REACT_TO_OWN_PROFILE, HTTPStatusCode.CONFLICT);
        }
        const datePageAction = await DatePageAction.create(data);
        if (data.actionType == DatePageActions.LIKE) {
            await Utils.sendNotification({
                "fromUserId": data.userId,
                "toUserId": data.actionUserId,
                "type": NotificationType.PROFILE_LIKE,
                "message": 'has liked your profile'
            });
        }
        return datePageAction;
    } catch (err: any) {
        throw err;
    }
};

const getAllActionByUserIdActionUserId = async (actionUserId: string, userId: string) => {
    try {
        return await DatePageAction.findAll({
            where: {
                actionUserId: actionUserId,
                userId: userId
            },
            raw: true
        });
    } catch (err: any) {
        throw err;
    }
};

const getAllActionByUserIdActionUserIdActionType = async (actionUserId: string, userId: string, actionType: string) => {
    try {
        return await DatePageAction.findAll({
            where: {
                actionUserId: actionUserId,
                userId: userId,
                actionType: actionType
            },
            raw: true
        });
    } catch (err: any) {
        throw err;
    }
};

const getActionsByUserId = async (userId: string) => {
    try {
        return await DatePageAction.findAll({
            where: {
                userId: userId
            },
            attributes: [
                "actionUserId"
            ],
            raw: true
        });
    } catch (err: any) {
        throw err;
    }
};

const getActionsByUserIdWithUserInfo = async (userId: string) => {
    try {
        const users = await DatePageAction.findAll({
            where: {
                actionUserId: userId,
                actionType: DatePageActions.LIKE
            },
            include: [
                {
                    model: db.User,
                    as: 'User',
                    attributes: [],
                    on: Sequelize.literal('User.id = DatePageAction.userId')
                }
            ],
            attributes: [
                "id",
                "actionType",
                "timestamp",
                [Sequelize.col('User.id'), 'userId'],
                [Sequelize.col('User.userName'), 'userName'],
                [Sequelize.col('User.firstName'), 'firstName'],
                [Sequelize.col('User.lastName'), 'lastName'],
                [Sequelize.col('User.profilePicture'), 'profilePicture'],
            ],
            raw: true
        });

        users.map((user) => {
            user.profilePicture = user.profilePicture ? '/static/users/' + user.userId + '/' + FolderNames.PROFILE_PICTURE + '/' + user.profilePicture : "";
        })
        return users;
    } catch (err: any) {
        throw err;
    }
};

const deleteActionByUserIdActionUserId = async (actionUserId: string, userId: string) => {
    try {
        return await DatePageAction.destroy({
            where: {
                actionUserId: actionUserId,
                userId: userId
            }
        });
    } catch (err: any) {
        throw err;
    }
};

export default {
    createAction,
    getAllActionByUserIdActionUserId,
    getAllActionByUserIdActionUserIdActionType,
    getActionsByUserId,
    deleteActionByUserIdActionUserId,
    getActionsByUserIdWithUserInfo
}