import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addDatingActionValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Dating Page Action Validator  -> Add Action Validator")
    try {
        const schema = Joi.object({
            actionUserId: Joi.number().description('actionUserId: 123').required().error(new Error('actionUserId is missing/invalid')),
            actionType: Joi.string().valid('LIKE', 'IGNORE').description('actionType: LIKE or IGNORE').required().error(new Error('actionType is missing/invalid that must be LIKE or IGNORE'))
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        };
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const deleteDatingActionValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Dating Page Action Validator  -> delete Action Validator")
    try {
        const schema = Joi.object({
            actionUserId: Joi.number().description('actionUserId: 123').required().error(new Error('actionUserId is missing/invalid'))
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        };
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addDatingActionValidator,
    deleteDatingActionValidator
}