import db from "../../../models";
import sequelize from 'sequelize';
import { DatePageActions } from "../../../utills/enums";
const DatingPageMedia = db.DatingPageMedia;
const User = db.User;

const createMedia = async (data: any) => {
    try {
        return await DatingPageMedia.create(data);
    } catch (err: any) {
        throw err
    }
};

const getMediaByUserId = async (id: string) => {
    try {
        return await DatingPageMedia.findAll({
            where: {
                userId: id
            },
            order: [['isPrimaryImage', 'DESC'], ['isDeletable', 'DESC']],
            raw: true
        });
    } catch (err: any) {
        throw err
    }
};

const getSingleMediaById = async (id: string) => {
    try {
        return await DatingPageMedia.find({
            where: {
                id: id
            }
        });
    } catch (err: any) {
        throw err
    }
};

const getSingleMediaByIdAndUserId = async (id: string, userId: string) => {
    try {
        return await DatingPageMedia.findOne({
            where: {
                id: id,
                userId: userId
            }
        });
    } catch (err: any) {
        throw err
    }
};

const getMediaByUserIdWithUserDetails = async (id: string) => {
    try {
        return await DatingPageMedia.findAll({
            where: {
                userId: id
            },
            include: [
                {
                    model: db.User,
                    as: 'User',
                    attributes: [
                        "userName",
                        "firstName",
                        "lastName"
                    ]
                }
            ],
            attributes: [
                "id", "mediaUrl", "mediaType", "mediaTypeText", "userId", "isDeletable", "isPrimaryImage",
                [sequelize.col('User.userName'), 'userName'],
                [sequelize.col('User.firstName'), 'firstName'],
                [sequelize.col('User.lastName'), 'lastName']
            ],
            order: [["isDeletable", "ASC"]],
            raw: true
        });
    } catch (err: any) {
        throw err
    }
};

const deleteMediaByMediaId = async (id: string) => {
    try {
        const mediaToDelete = await DatingPageMedia.findOne({
            where: {
                id: id
            },
            raw: true
        });
        
        const deletedMedia =  await DatingPageMedia.destroy({
            where: {
                id: id
            }
        });

        if(mediaToDelete.isPrimaryImage === '1' || mediaToDelete.isPrimaryImage === 1) {
            let imageTobeSetAsProfilePic = '';
            const primaryMedias = await DatingPageMedia.findAll({
                where: {
                    userId: mediaToDelete.userId,
                    isPrimaryImage: 1
                },
                order: [['id','DESC']]
            });

            if(primaryMedias.length) {
                imageTobeSetAsProfilePic = primaryMedias[0].mediaUrl;
            } else {
                const faceImage = await DatingPageMedia.findOne({
                    where: {
                        isDeletable: 0,
                        userId: mediaToDelete.userId
                    }
                });
                if(faceImage) {
                    imageTobeSetAsProfilePic = faceImage.mediaUrl
                }
            }
    
            await User.update({
                profilePicture: imageTobeSetAsProfilePic
            }, {
                where: {
                    id: mediaToDelete.userId
                }
            });
        }

        return deletedMedia;
    } catch (err: any) {
        throw err
    }
};

const deleteAllMediaByUserId = async (id: string) => {
    try {
        return await DatingPageMedia.destroy({
            where: {
                userId: id
            }
        });
    } catch (err: any) {
        throw err
    }
};

export default {
    createMedia,
    getMediaByUserId,
    getSingleMediaById,
    getSingleMediaByIdAndUserId,
    getMediaByUserIdWithUserDetails,
    deleteMediaByMediaId,
    deleteAllMediaByUserId
}
