import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addMediaPayloadValidate = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Dating Page Media Validator  -> Add media Validator")
    try {
        const schema = Joi.object().keys({
            mediaPath: Joi.any().allow('').description('mediaPath').error(new Error('mediaPath is missing')),
            isPrimaryImage: Joi.any().allow('').description('isPrimaryImage').error(new Error('isPrimaryImage field is missing'))
        });
        req.body.mediaPath = req['files'];
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        };
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const deleteMediaPayloadValidate = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Dating Page Media Validator  -> Delete media Validator")
    try {
        const schema = Joi.object().keys({
            id: Joi.string().required().description('id : media id').error(new Error('id is missing or incorrect'))
        });
        // req.body = Object.keys(req.body).length > 0 ? req.body : req['fields'];
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        };
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addMediaPayloadValidate,
    deleteMediaPayloadValidate
}
