import { NextFunction, Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import { Logging } from "../../commonServices/logging.service";
import getErrorMessage from "../../../utills/errorStructure";
import DistanceMatchService from "./distanceMatch.service";
import UserService from "../users/user.service";
import { CustomError } from "../../../utills/types";

const addDistanceMatch = async (req: Request, res: Response) => {
    try {
        const user = req['user'];
        let userData = await UserService.getUserById(user.id);
        if (!userData) {
            return res.status(HTTPStatusCode.NOTFOUND).json({
                message: StaticMessages.getNotExistMessage('User'),
                status: Constants.ERROR
            });
        }
        const insertData = {
            distanceInKms: parseInt(req.body.distanceInKms),
            isEnabled: parseInt(req.body.isEnabled),
            userId: user.id
        }
        await DistanceMatchService.createOrUpdateDistanceMatch(insertData);
        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getCreatedMessage('Distance Match')
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        return res.status(HTTPStatusCode.SUCCESS).json({
            error: getErrorMessage(error),
            status: Constants.ERROR,
            message: StaticMessages.STORY_CREATE_FAILED
        });
    }
};

const getDistanceMatch = async (req: Request, res: Response) => {
    try {
        const user = req['user'];
        let userData = await UserService.getUserById(user.id);
        if (!userData) {
            throw new CustomError(StaticMessages.getNotExistMessage('User'), HTTPStatusCode.NOTFOUND);
        }
        const distanceMatch = await DistanceMatchService.getDistanceMatchByUserId(user.id);
        if (!distanceMatch) {
            throw new CustomError(StaticMessages.getNotExistMessage('Distance Match'), HTTPStatusCode.NOTFOUND);
        }
        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getListReturnedMessage('Distance Match'),
            distanceMatchData: distanceMatch
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        return res.status(HTTPStatusCode.SUCCESS).json({
            error: getErrorMessage(error),
            status: Constants.ERROR,
            message: getErrorMessage(error).message
        });
    }
};

export default {
    addDistanceMatch,
    getDistanceMatch
}