import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addDistanceMatchValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Distance Match Validator  -> Add DistanceMatch");
    try {
        const schema = Joi.object().keys({
            distanceInKms: Joi.number().optional().description('distanceInKms: 0').error(new Error('distanceInKms is missing or incorrect')),
            isEnabled: Joi.number().valid(0, 1).optional().description('isEnabled: 0').error(new Error('isEnabled is missing or incorrect'))
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addDistanceMatchValidator
}