import db from "../../../models";
import path from 'path';
import request from 'request';
import fs from 'fs';
import { v4 } from "uuid";
const DocumentVerificationLogs = db.DocumentVerificationLogs;
const IdenfyLogs = db.IdenfyLogs;
const User = db.User;
const DatingPageMedia = db.DatingPageMedia;
import { IIdenfyLogs } from "../../../models/IdenfyLogs";
import { FolderNames, ImageExentions, MediaTypes } from "../../../utills/enums";

const createDocumentLogs = async (data: any) => {
    try {
        const log = await DocumentVerificationLogs.findOne({
            where: {
                userId : data.userId
            }
        });
        // await User.update({
        //     isDocumentVerified: data.succeeded
        // }, {
        //     where: {
        //         id: data.userId
        //     }
        // });
        if(log) {
            if( data.succeeded) {
                await DocumentVerificationLogs.update( data, {
                    where: {
                        userId: data.userId
                    }
                });
            } else {
                await DocumentVerificationLogs.update( {
                    ...data,
                    verificationFailed:  log.verificationFailed + 1
                },{
                    where: {
                        userId: data.userId
                    }
                });
            }
        } else {
            if (data.succeeded) {
                await DocumentVerificationLogs.create(data);
            } else {
                await DocumentVerificationLogs.create({
                    ...data,
                    verificationFailed: 1
                });
            }
        }
        return true;
    } catch (err: any) {
        throw err
    }
};

const getSuccededDocumentLogs = async (data: any) => {
    try {
        return await DocumentVerificationLogs.findAll({
            where: {
                succeded: 1
            },
            raw: true
        });
    } catch (err: any) {
        throw err
    }
};

const getLogsByRequestId = async (requestId: string) => {
    try {
        return await DocumentVerificationLogs.findOne({
            where: {
                verificationId: requestId
            },
            raw: true
        });
    } catch (err: any) {
        throw err
    }
};

const updateLogsByRequestId = async (data: any, requestId: string) => {
    try {
        return await DocumentVerificationLogs.update(data, {
            where: { verificationId: requestId }
        });
    } catch (err: any) {
        throw err
    }
};

const createIdenfyLogs = async (data: IIdenfyLogs) => {
    try {
        return await IdenfyLogs.create(data);
    } catch (err: any) {
        throw err
    }
}

const getIdenfyLogs = async (condition: any) => {
    try {
        return await IdenfyLogs.findAll({
            where: condition,
            raw: true
        });
    } catch (err: any) {
        throw err
    }
}

const getIdenfyLogsByUserIdWithIdSort = async (userId: string) => {
    try {
        return await IdenfyLogs.findAll({
            where: {
                userId: userId
            },
            order: [
                ['id', 'DESC']
            ],
            raw: true
        });
    } catch (err: any) {
        throw err
    }
}

const deleteIdenfyLogsByScanRef = async (scanRef: any) => {
    try {
        return await IdenfyLogs.destroy({
            where: {
                scanRef: scanRef
            },
            raw: true
        });
    } catch (err: any) {
        throw err
    }
}

const createDatingMedia = async (face, userId, saveDatingMediaInDB = true) => {
    const destination = path.join(__dirname, '../../../public/users/' + userId + '/' + FolderNames.DATING_PAGE_MEDIA + '/');
    const profilePicDestination = path.join(__dirname, '../../../public/users/' + userId + '/' + FolderNames.PROFILE_PICTURE + '/');
    if (!fs.existsSync(destination)) {
        fs.mkdirSync(destination, {recursive: true})
    }

    if (!fs.existsSync(profilePicDestination)) {
        fs.mkdirSync(profilePicDestination, {recursive: true})
    }

    const fileName = `${v4()}${ImageExentions.PNG}`;
    const profilePicName = `${v4()}${ImageExentions.PNG}`;

    // await request(face).pipe(fs.createWriteStream(`${destination}${fileName}`));
    // await request(face).pipe(fs.createWriteStream(`${profilePicDestination}${profilePicName}`));

    fs.writeFileSync(`${destination}/${fileName}`, face.buffer);
    fs.writeFileSync(`${profilePicDestination}/${profilePicName}`, face.buffer);

    await User.update({
        profilePicture: profilePicName
    }, {
        where: {
            id: userId
        }
    });
    if (saveDatingMediaInDB) {
        await DatingPageMedia.create({
            userId: userId,
            mediaUrl: fileName,
            mediaType: 1,
            isDeletable: 0,
            mediaTypeText: MediaTypes.IMAGE
        });
    }

}

const getVerificationStatus = async (userId) => {
    try {
        const user = await User.findOne({
            where: {
                id: userId
            }
        });
        if(!user) return user;
        return user.isDocumentVerified;
    } catch (err) {
        throw err;
    }
}


const setVideoVerificationStatus = async (userId) => {
    try {
        const videoVerified = await User.update({
            isVideoVerified: 1
        },{
            where: {
                id: userId
            }
        });
        return videoVerified;
    } catch (err) {
        throw err;
    }
}

export default {
    createDocumentLogs,
    createIdenfyLogs,
    getSuccededDocumentLogs,
    getLogsByRequestId,
    updateLogsByRequestId,
    getIdenfyLogsByUserIdWithIdSort,
    getIdenfyLogs,
    deleteIdenfyLogsByScanRef,
    createDatingMedia,
    getVerificationStatus,
    setVideoVerificationStatus
};
