import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addEmergencyContactValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Emargency Contact Validator  -> Add New Contacts")
    try {
        const schema = Joi.object().keys({
            contacts: Joi.array().items(Joi.object({
                name: Joi.string().allow('').required(),
                email: Joi.string().allow('').required(),
                phone: Joi.string().allow('').required()
            })).required().error(new Error('Contacts should be Array of Object')),
            alertSystemConsent: Joi.number().allow(0, 1).required().description('alertSystemConsent: 0-False, 1-True').error(new Error('alertSystemConsent is missing/invalid')),
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const setOnDateValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Emargency Contact Validator  -> Set Is On Date")
    try {
        const schema = Joi.object().keys({
            longitude: Joi.string().allow('').required().error(new Error('longitude is incorrect/invalid')),
            latitude: Joi.string().allow('').required().error(new Error('latitude is incorrect/invalid')),
            isOnDate: Joi.number().allow(0, 1).required().error(new Error('isOnDate is incorrect/invalid'))
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const addTimeLogsValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Emargency Contact Validator  -> Add Time Logs")
    try {
        const schema = Joi.object().keys({
            longitude: Joi.string().allow('').required().error(new Error('longitude is incorrect/invalid')),
            latitude: Joi.string().allow('').required().error(new Error('latitude is incorrect/invalid'))
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addEmergencyContactValidator,
    setOnDateValidator,
    addTimeLogsValidator
}