import { Request, Response } from 'express';
import { Constants } from '../../../utills/constants';
import { HTTPStatusCode } from '../../../utills/enums';
import getErrorObject from '../../../utills/errorStructure';
import { StaticMessages } from '../../../utills/staticMessage';
import { Logging } from '../../commonServices/logging.service';
import FindFriendsService from './findFriends.service';

const getNearByFriends = async (req: Request, res: Response) => {
    try {
        let user = req['user'];
        let createNotificationData = await FindFriendsService.getNearByFriends(user.id, req.query.limit, req.query.offset);
        res.status(HTTPStatusCode.SUCCESS).json({
            friendsLocations: createNotificationData,
            status: Constants.SUCCESS
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('Notification'),
            status: Constants.ERROR
        });
    }
};

export default {
    getNearByFriends
}