import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
const addPost = (req: Request, res: Response, next: NextFunction) => {
    try {
        const schema = Joi.object().keys({
            media: Joi.binary(),
            location: Joi.string(),
            files: Joi.array(),
            mediaType: Joi.number(),
            description: Joi.string(),
            tags: Joi.array().items(Joi.number()),
            // tags: Joi.string(),
            requestHeader: Joi.string().required().description('requestHeader').error(new Error('Request Header is missing'))
        });
        if (req['files']) {
            req.body['files'] = req['files'];
        }
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addPost
}