import db from "../../../models";
import { CustomError } from "../../../utills/types";
import { CommonAnswers, FolderNames, FollowStatus, Questions, UserLastStatus } from "../../../utills/enums";
import { Sequelize } from "sequelize";
import { Constants } from "../../../utills/constants";
import moment from "moment";
import UserOnlineLogsService from "../userOnlineLogs/userOnlineLogs.service";
import DatingPageMediaService from "../datingPageMedia/datingPageMedia.service";
import UserCharacteristicService from "../userCharacteristic/userCharacteristic.service";
import LocationService from "../locations/location.service";
import { Utils } from "../../../utills/helperMethods";
import UserPromptsService from "../userPrompts/userPrompts.service";
import {Op} from 'sequelize';
const Follower = db.Follower;
const User = db.User;
const Location = db.Location;

const followUser = async (toUserId: string, fromUserId: string) => {
    try {
        const requestExists = await getUserFollowFollowingData(fromUserId, toUserId)
        if (requestExists) {
            return requestExists;
        }
        let follower = await Follower.create({
            toUser: parseInt(toUserId),
            fromUser: parseInt(fromUserId),
            status: FollowStatus.FOLLOW_REQUEST_SENT,
            followedDt: moment().unix()
        });
        if (follower) {
            return follower;
        } else {
            throw new CustomError('Following Request Could not be sent.');
        }
    } catch (error: any) {
        return error;
    }
};

const followUserWithStatus = async (toUserId: string, fromUserId: string, status: any) => {
    try {
        const requestExists = await getUserFollowFollowingData(fromUserId, toUserId)
        if (requestExists && requestExists.status && requestExists.status == status) {
            return requestExists;
        }

        return await Follower.create({
            toUser: parseInt(toUserId),
            fromUser: parseInt(fromUserId),
            status: FollowStatus.FOLLOW_REQUEST_ACCEPTED,
            followedDt: moment().unix()
        });;
    } catch (error: any) {
        return error;
    }
};

const getUserFollowFollowingData = async (fromUser: string, toUser: string) => {
    return await Follower.findOne({
        where: {
            "fromUser": parseInt(fromUser),
            "toUser": parseInt(toUser)
        },
        raw: true
    });
}

const getUserFollowingIds = async (id: string) => {
    try {
        let followings = await Follower.findAll({
            where: {
                "fromUser": id,
                "status": FollowStatus.FOLLOW_REQUEST_ACCEPTED //2
            },
            raw: true
        });
        let followingsIds = followings.map(followings => followings.toUser);

        return followingsIds;
    } catch (error: any) {
        return error;
    };
};

const getUserFollowingLatLongs = async (id: string, limit, offset) => {
    try {
        let followingIds = await getUserFollowingIds(id);
        let followings = await Location.findAll({
            where: {
                "UserId": followingIds
            },
            include: [
                {
                    model: db.User,
                    attributes: [
                        // 'profileThumbnail',
                        // 'userName',
                        // 'id',
                    ],
                }
            ],
            attributes: [
                'latitude',
                'longitude',
                'id',
                [Sequelize.col('User.profileThumbnail'), 'profileThumbnail'],
                [Sequelize.col('User.id'), 'userId'],
                [Sequelize.col('User.userName'), 'userName'],
            ],
            offset: offset ? parseInt(offset) : 0,
            limit: limit ? parseInt(limit) : Constants.LIMIT_ALL,
            raw: true
        });

        followings.map((follow) => {
            follow.profileThumbnail = follow.profileThumbnail ? '/static/users/' + follow.userId + '/' + FolderNames.PROFILE_PICTURE + '/' + follow.profileThumbnail : "";
        })
        return followings;
    } catch (error: any) {
        return error;
    };
};

const getUserFollowings = async (id: string, limit = 20, offset = 0) => {
    try {
        let followingsIds = await getUserFollowingIds(id);
        let userFollowings = await User.findAll({
            where: {
                "id": followingsIds
            },
            offset: offset,
            limit: limit,
            raw: true
        });

        userFollowings.map((users) => {
            users.profileThumbnail = users.profileThumbnail ? '/static/users/' + users.id + '/' + FolderNames.PROFILE_PICTURE + '/' + users.profileThumbnail : "";
        });
        return userFollowings;
    } catch (error: any) {
        return error;
    };
};

const getUserFollowers = async (id: string, limit: any, offset: any) => {
    try {
        if(limit) {
            limit = parseInt(limit);
        } else {
            limit = 20;
        }
        if(offset) {
            offset = parseInt(offset);
        } else {
            offset = 0;
        }
        const threeDaysAgoUnixTimestamp = moment().subtract(3, 'days').unix();
        let followers = await Follower.findAll({
            where: {
                "toUser": id,
                "status": FollowStatus.FOLLOW_REQUEST_ACCEPTED, //2
                "followedDt": {
                    [Op.gte] : threeDaysAgoUnixTimestamp
                }
            }
        });
        let followersIds = followers.map(followers => followers.fromUser);
        let userFollowers = await User.findAll({
            where: {
                "id": followersIds
            },
            offset: offset,
            limit: limit,
        });

        const finalResp = [];
        let isCurrentUserLatLongFound = false;
        let sourceUserLatLong = {
            latitude: 0,
            longitude: 0
        };
        const authUserLatLong = await LocationService.getLocationByUserId(String(id))
        if (authUserLatLong && (authUserLatLong.latitude && authUserLatLong.latitude !== '') && (authUserLatLong.longitude && authUserLatLong.longitude !== '')) {
            isCurrentUserLatLongFound = true;
            sourceUserLatLong.latitude = parseFloat(authUserLatLong.latitude);
            sourceUserLatLong.longitude = parseFloat(authUserLatLong.longitude);
        }

        const userFollowerMap = await userFollowers.map(async (users) => {
            const userData: any = {};
            userData.age = users.DOB == null ? Constants.DEFAULT_AGE : moment().diff(users.DOB, 'years');
            userData.profilePicture = users.profilePicture ? '/static/users/' + users.id + '/' + FolderNames.PROFILE_PICTURE + '/' + users.profilePicture : "";
            userData.profileThumbnail = users.profileThumbnail ? '/static/users/' + users.id + '/' + FolderNames.PROFILE_PICTURE + '/' + users.profileThumbnail : "";
            userData.firstName = users.firstName || "";
            userData.lastName = users.lastName || "";
            const userOnlineLog = await UserOnlineLogsService.getOnlineLogByUserId(users.id);
            if ((!userOnlineLog) || (userOnlineLog && userOnlineLog.offTimestamp && userOnlineLog.offTimestamp !== 0)) {
                userData.onlineStatus = UserLastStatus.OFFLINE;
                userData.lastSeenTimestamp = userOnlineLog && userOnlineLog.offTimestamp || moment().subtract(5, 'minutes').unix();
            } else if (userOnlineLog && userOnlineLog.offTimestamp == 0) {
                userData.onlineStatus = UserLastStatus.ONLINE;
                userData.lastSeenTimestamp = null;
            }

            let datingMedia = await DatingPageMediaService.getMediaByUserId(users.id);
            datingMedia = datingMedia.map((media) => {
                media.mediaUrl = '/static/users/' + media.userId + '/' + FolderNames.DATING_PAGE_MEDIA + '/' + media.mediaUrl;
                delete media.userId;
                return media;
            });

            const workQuestion = await UserCharacteristicService.getCharacteristicsByUserIdQuestionId(users.id, Questions.WORKS);
            userData.works = workQuestion[0] ? workQuestion[0].customAns : null;

            const eductaionLevelQuestion = await UserCharacteristicService.getCharacteristicsByUserIdQuestionId(users.id, Questions.EDUCATION_LEVEL);
            const universityQuestion = await UserCharacteristicService.getCharacteristicsByUserIdQuestionId(users.id, Questions.COLLEGE_UNIVERSITY);
            if (eductaionLevelQuestion.length > 0 && universityQuestion.length > 0) {
                if (eductaionLevelQuestion[0].answers[0].answerText != CommonAnswers.PREFER_NOT_TO_SAY) {
                    userData.educationLevel = eductaionLevelQuestion[0].answers[0].answerText;
                    userData.university = universityQuestion[0].customAns;
                }
            } else {
                userData.educationLevel = null;
                userData.university = null;
            }

            const hometownQuestion = await UserCharacteristicService.getCharacteristicsByUserIdQuestionId(users.id, Questions.HOMETOWN);
            userData.hometown = hometownQuestion[0] ? hometownQuestion[0].customAns : null;

            // subscription detail pending as we don't need at this time.

            const prefUserLocation = await LocationService.getLocationByUserId(String(users.id));
            const location: any = {}
            if (isCurrentUserLatLongFound && prefUserLocation && (prefUserLocation.latitude && prefUserLocation.latitude !== '') && (prefUserLocation.longitude && prefUserLocation.longitude !== '')) {
                const destUserLatLong = {
                    latitude: parseFloat(prefUserLocation.latitude),
                    longitude: parseFloat(prefUserLocation.longitude)
                }
                location.latitude = parseFloat(prefUserLocation.latitude);
                location.longitude = parseFloat(prefUserLocation.longitude);
                const distanceInKms = Utils.findDistanceAmongUsersInKms(sourceUserLatLong, destUserLatLong);
                location.distanceInKms = distanceInKms + ' kms away';
                location.distanceInMiles = Utils.findDistanceAmongUsersInMiles(sourceUserLatLong, destUserLatLong) + ' miles away';

                // if (authUserDistanceMatchDetails && authUserDistanceMatchDetails.isEnabled) {
                //     // if (!(authUserDistanceMatchDetails.distanceInKms >= distanceInKms)) users.splice(users[i], 1);
                // }
            } else {
                location.distanceInKms = 'Less than a Km away';
                location.distanceInMiles = 'Less than a mile away';
                location.latitude = 0;
                location.longitude = 0;
            }

            const prefUserPrompts = await UserPromptsService.getUserPromptsByUserId(users.id);
            prefUserPrompts.map((prompt) => {
                delete prompt.userId;
                return prompt;
            })

            finalResp.push({
                userId: users.id,
                userBasicInfo: userData,
                datingPageMedia: datingMedia,
                prompts: prefUserPrompts,
                interests: ["Netflix"],
                socialMedia: [
                    {
                        mediaType: "FACEBOOK",
                        mediaId: "example"
                    }
                ],
                location: location
            })
            return users;

        })
        return Promise.all(userFollowerMap).then(() => finalResp);
    } catch (error: any) {
        return error;
    }
};

const updateFollowRequest = async (toUserId: string, fromUserId: string, status: string) => {
    try {
        return await Follower.update({ status: status }, {
            where: {
                "toUser": toUserId,
                "fromUser": fromUserId
            }
        });
    } catch (error: any) {
        return error;
    }
};

const removeFollowRequest = async (toUserId: string, fromUserId: string) => {
    try {
        return await Follower.destroy({
            where: {
                "toUser": toUserId,
                "fromUser": fromUserId
            }
        });
    } catch (error: any) {
        return error;
    }
};

const getFollowRequestListByUserId = async (userId: string) => {
    try {
        const followRequests = await Follower.findAll({
            where: {
                toUser: userId,
                status: FollowStatus.FOLLOW_REQUEST_SENT
            },
            include: [{
                model: db.User,
                attributes: [
                    "userName",
                    "profileThumbnail"
                ],
                on: Sequelize.literal('User.id = Follower.fromUser')
            }],
            raw: true
        });

        followRequests.map((follow) => {
            follow.userName = follow['User.userName'];
            follow.profileThumbnail = follow['User.profileThumbnail'] ? '/static/users/' + follow.fromUser + '/' + FolderNames.PROFILE_PICTURE + '/' + follow['User.profileThumbnail'] : "";

            delete follow['User.profileThumbnail'];
            delete follow['User.userName'];
            return follow;
        });
        return followRequests;
    } catch (error: any) {
        return error;
    }
};

export default {
    followUser,
    followUserWithStatus,
    getUserFollowings,
    getUserFollowers,
    getUserFollowFollowingData,
    updateFollowRequest,
    removeFollowRequest,
    getUserFollowingIds,
    getUserFollowingLatLongs,
    getFollowRequestListByUserId
}
