import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const followUnfollowUserPayload = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Follow Validator  -> Add New Follow Validator")
    try {
        const schema = Joi.object().keys({
            toUser: Joi.number().required().description('toUser : 1').error(new Error('toUser is missing or incoreect'))
        })

        req.body = Object.keys(req.body).length > 0 ? req.body : req['fields'];
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const getFollowersFollowings = (req: Request, res: Response, next: NextFunction) => {
    try {
        const schema = Joi.object().keys({
            limit: Joi.string().allow('').default('20').description('limit').optional(),
            offset: Joi.string().allow('').default('0').description('offset').optional()
        });
        const validation = schema.validate(req.query);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const rejectOrDeleteUser = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Follow Validator  -> Add New Follow Validator")
    try {
        const schema = Joi.object().keys({
            fromUser: Joi.number().required().description('toUser : 1').error(new Error('fromUser is missing or incoreect'))
        })
        req.body = Object.keys(req.body).length > 0 ? req.body : req['fields'];
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const acceptFollowRequest = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Follow Validator  -> Add New Follow Validator")
    try {
        const schema = Joi.object().keys({
            fromUser: Joi.number().required().description('toUser : 1').error(new Error('fromUser is missing or incoreect')),
            status: Joi.number().required()
        })
        req.body = Object.keys(req.body).length > 0 ? req.body : req['fields'];
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};
export default {
    followUnfollowUserPayload,
    getFollowersFollowings,
    rejectOrDeleteUser,
    acceptFollowRequest
}