import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const validateForgetPasswordOTP = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            requestHeader: Joi.object().optional().description('requestHeader').error(new Error('Request Header is missing')),
            countryCode: Joi.string().allow('').description('phone').required(),
            phone: Joi.string().allow('').description('phone').required(),
            otp: Joi.string().allow('').description('otp').required()
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const resetPassword = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            requestHeader: Joi.object().optional().description('requestHeader').error(new Error('Request Header is missing')),
            phone: Joi.string().allow('').description('phone').required(),
            password: Joi.string().allow('').description('password').required()
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const forgotPassword = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            requestHeader: Joi.object().optional().description('requestHeader').error(new Error('Request Header is missing')),
            countryCode: Joi.string().allow('').description('phone').required(),
            phone: Joi.string().allow('').description('phone').required()
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    validateForgetPasswordOTP,
    resetPassword,
    forgotPassword
}