import { Constants } from "../../../utills/constants";
import { Utils } from "../../../utills/helperMethods";
import { sendMail } from "../../../utills/mailer";
import db from "../../../models";
import { HTTPStatusCode } from "../../../utills/enums";
import { StaticMessages } from "../../../utills/staticMessage";
import { OtpType } from "../../../utills/enums";
import { CustomError } from "../../../utills/types";
const User = db.User;
const OTP = db.Otp;

const validateOTP = async (email: string, otp: number, otpType: number) => {
    try {
        let countAndRow: any;
        if (OtpType.SIGNUP === otpType) {
            countAndRow = await OTP.findAndCountAll({ where: { email: email, signUpOTP: otp } });
            if (countAndRow.count) {
                await User.update({ isVerified: 1 }, {
                    where: {
                        email: email
                    }
                });
                return true;
            }
        } else if (OtpType.RESET_PASSWORD === otpType) {
            countAndRow = await OTP.findAndCountAll({ where: { email: email, resetPasswordOTP: otp } });
        } else {
            throw new CustomError(StaticMessages.INVALID_OTP_TYPE, HTTPStatusCode.BAD_REQUEST);;
        }
        if (!countAndRow.count) {
            throw new CustomError(StaticMessages.INVALID_OTP, HTTPStatusCode.BAD_REQUEST);
        }
    } catch (error: any) {
        throw error;
    }
};

const forgotPassword = async (email: string) => {
    try {
        let { count, rows } = await OTP.findAndCountAll({ where: { email: email } });
        if (count) {
            const otp = Utils.randomDigits(4);
            sendMail(Constants.FROM_EMAIL, email, "Reset Password datesafe Account", Utils.getOptMailHtml(otp));
            await OTP.update({ resetPasswordOTP: otp, }, { where: { email: email } });
            return true;
        } else {
            throw new CustomError(StaticMessages.EMAIL_NOT_FOUND, HTTPStatusCode.UNAUTHORIZED);
        }
    } catch (error: any) {
        throw error;
    }
};

const resetPassword = async (countryCode: string, phone: string, password: string) => {
    try {
        let { count, rows } = await User.findAndCountAll({ where: { phone: phone } });
        if (count) {
            password = password ? Utils.encryptedAES(password) : '';
            await User.update({ password }, { where: { phone: phone } });
            return true;
        } else {
            throw new CustomError(StaticMessages.getNotExistMessage('Mobile number'), HTTPStatusCode.UNAUTHORIZED);
        }
    } catch (error: any) {
        throw error;
    }
};

export default {
    validateOTP,
    forgotPassword,
    resetPassword
};
