import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import HearAboutUsFromService from "./hearAboutUsFrom.service";

const getHearAboutUsChoices = (req: Request, res: Response) => {
    const getChoices = () => new Promise((resolve, reject) => {
        return HearAboutUsFromService.getHearLocations()
            .then((data) => {
                if (data.length > 0) {
                    resolve(data);
                }
                reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('Hear About Us') });
            })
            .catch(err => reject(err))
    });
    return getChoices()
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Hear About us'),
                hearAboutUsChoices: data
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

export default {
    getHearAboutUsChoices
}