import { ILocation } from "../../../models/location";
import db from "../../../models";
import SignUpCheckListService from "../signUpCheckList/signUpCheckList.service";
const Location = db.Location;


const createLocation = async (data: ILocation, userId) => {
    try {
        let res = await Location.create({
            ...data,
            UserId: userId
        });

        await SignUpCheckListService.updateCheckListByUserId(userId, { locationAdded: true });
        return res;
    } catch (error: any) {
        throw error;
    }
};

const getLocationByUserId = async (id: string, attrs: string[] = []) => {
    try {
        if (attrs.length === 0) {
            return await Location.findOne({
                where: { UserId: id }
            });
        } else {
            return await Location.findOne({
                where: { UserId: id },
                attributes: attrs
            });
        }
    } catch (error) {
        return error;
    }
};

const getLocationByMultipleUserIds = async (userIds: string[], attrs: string[] = []) => {
    try {
        if (attrs.length === 0) {
            return await Location.findAll({
                where: { UserId: userIds }
            });
        } else {
            return await Location.findAll({
                where: { UserId: userIds },
                attributes: attrs
            });
        }
    } catch (error) {
        return error;
    }
};

const updateLocationByUserId = async (id: string, data: any) => {
    try {
        const user = await Location.update(data, {
            where: { UserId: id }
        });
        return user;
    } catch (error) {
        return error;
    }
};

const deleteLocationByUserId = async (id: string) => {
    try {
        return await Location.destroy({
            where: { UserId: id }
        });
    } catch (error) {
        return error;
    }
};

export default {
    createLocation,
    getLocationByUserId,
    updateLocationByUserId,
    deleteLocationByUserId,
    getLocationByMultipleUserIds
}