import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addLocation = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            longitude: Joi.string().allow('').description('longitude').required(),
            latitude: Joi.string().allow('').description('latitude').required(),
            address: Joi.string().allow('').description('address').required()
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const updateLocationPrivacy = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            locationPrivacy: Joi.number().valid(0, 1).description('locationPrivacy').required().error(new Error('locationPrivacy missing or invalid'))
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};
export default {
    addLocation,
    updateLocationPrivacy
}