import { Constants } from "../../../utills/constants";
import { Utils } from "../../../utills/helperMethods";
import { sendMail } from "../../../utills/mailer";
import { IUser } from "../../../models/user";
import db from "../../../models";
import { HTTPStatusCode, LoginTypes, OtpType, UserType } from "../../../utills/enums";
import { StaticMessages } from "../../../utills/staticMessage";
import { jwtSign } from "../../../utills/token";
import { CustomError } from "../../../utills/types";
import moment from "moment";
import SignUpCheckListService from "../signUpCheckList/signUpCheckList.service";
import { Op } from "sequelize";
const User = db.User;
const OTP = db.Otp;
const UserToken = db.UserToken;

const createUser = async (data: IUser) => {
    try {
        data.password = data.password ? Utils.encryptedAES(data.password) : '';
        let res = await User.create(data);
        if (data.loginType == LoginTypes.NORMAL) {
            // createOtpAndSendEmail({ email: data.email });
        } else {
            // createSuccessMailAndSendEmail({ email: data.email, password: data.password })
        }
        await SignUpCheckListService.addCheckList(res.id);
        return res;
    } catch (err: any) {
        throw err
    }
};

const createSuccessMailAndSendEmail = async (data) => {
    await sendMail(
        Constants.FROM_EMAIL,
        data.email,
        StaticMessages.SUCCESS_SIGNUP,
        Utils.getLoginInfoHtml(data.email, data.password));
};

const createOtpAndSendEmail = async (data) => {
    const otp = Utils.randomDigits(4);
    await OTP.create({
        signUpOTP: otp,
        resetPasswordOTP: null,
        email: data.email
    });
    await sendMail(
        Constants.FROM_EMAIL,
        data.email,
        StaticMessages.CONFIRM_ACCOUNT,
        Utils.getOptMailHtml(otp));
    return otp;
};

const checkUserByEmail = async (email: string) => {
    try {
        return User.findOne({
            where: {
                email: email
            }
        });
    } catch (err: any) {
        throw err;
    }
};

const checkUserByUserName = async (userName: string) => {
    try {
        return User.findOne({
            where: {
                userName: userName
            }
        });
    } catch (err: any) {
        throw err;
    }
};

const checkUserByPhone = async (countryCode: string, phone: string) => {
    try {
        return User.findOne({
            where: {
                countryCode: countryCode,
                phone: phone
            }
        });
    } catch (err: any) {
        throw err;
    }
};

const userLogin = async (userName: string, password: string, email: string, userType: string) => {
    try {
        if (!userName && !email) {
            throw new CustomError('Incorrect Email or Phone Number.', HTTPStatusCode.BAD_REQUEST, 'CustomError');
        }
        if (!password) {
            throw new CustomError(StaticMessages.INCORRECT_PASSWORD, HTTPStatusCode.BAD_REQUEST);
        }
        
        let WHERE = userName ? { userName: userName, userType: userType } : { [Op.or] : [ {email: email} , {phone: email} ], userType: userType};
        console.log(WHERE);
        const user = await User.findOne({ where: WHERE });
        if (user) {
            if(user.password && user.password.length > 22){
                let decryptPassword=Utils.decryptedAES(user.password)
                if(password !== decryptPassword){
                     throw new CustomError(StaticMessages.INCORRECT_PASSWORD, HTTPStatusCode.UNAUTHORIZED);
                }
            }else{
                if(password !== user.password){
                    throw new CustomError(StaticMessages.INCORRECT_PASSWORD, HTTPStatusCode.UNAUTHORIZED);
                }
            }
            if (user.isBlocked) {
                throw new CustomError(StaticMessages.USER_BLOCKED, HTTPStatusCode.UNAUTHORIZED);
            }
            // if(!user.isDocumentVerified) {
            //     throw new CustomError(StaticMessages.DOCUMENTS_NOT_VERIFIED, HTTPStatusCode.ERROR_410);
            // }
            let userJwt = jwtSign({ name: user.userName, email: user.email, id: user.id, userType: user.userType, issueTimeStamp: moment().unix() });
            await logUserToken(user.id, user.userType, userJwt);
            return userJwt;
        } else {
            const user = await User.findOne( { where: { [Op.or] : [ {email: email} , {phone: email} ] } } );
            if(user) {
                throw new CustomError('Incorrect Password.', HTTPStatusCode.UNAUTHORIZED);
            } else {
                throw new CustomError('Incorrect Email or Phone Number.', HTTPStatusCode.UNAUTHORIZED);
            }
        }
    } catch (error: any) {
        throw error;
    }
};

const validateOTP = async (email: string, otp: number, otpType: number) => {
    try {
        let countAndRow: any;
        if (OtpType.SIGNUP === otpType) {
            countAndRow = await OTP.findAndCountAll({ where: { email: email, signUpOTP: otp } });
            if (countAndRow.count) {
                await User.update({ isVerified: 1 }, {
                    where: {
                        email: email
                    }
                });
                return true;
            }
        } else if (OtpType.RESET_PASSWORD === otpType) {
            countAndRow = await OTP.findAndCountAll({ where: { email: email, resetPasswordOTP: otp } });
        } else {
            throw new CustomError(StaticMessages.INVALID_OTP_TYPE, HTTPStatusCode.BAD_REQUEST);;
        }
        if (!countAndRow.count) {
            throw new CustomError(StaticMessages.INVALID_OTP, HTTPStatusCode.BAD_REQUEST);
        }
    } catch (error: any) {
        throw error;
    }
};
const resendOTP = async (email: string) => {
    try {
        const otp = Utils.randomDigits(4);
        let { count, rows } = await OTP.findAndCountAll({ where: { email: email } });
        let response = null;
        if (!count) {
            response = await OTP.create({
                signUpOTP: otp,
                resetPasswordOTP: null,
                email: email
            });
        } else {
            let updateResponse = await OTP.update({
                signUpOTP: otp,
            }, {
                where: { email: email }
            });
            response = updateResponse && updateResponse.length && updateResponse[0] !== '' && updateResponse[0] !== 0;
        }
        await sendMail(
            Constants.FROM_EMAIL,
            email,
            StaticMessages.CONFIRM_ACCOUNT,
            Utils.getOptMailHtml(otp));
        return response;
    } catch (error: any) {
        throw error;
    }
};

const userLogout = async (userId: string, userType: string, userToken: string) => {
    try {
        return UserToken.destroy({
            where: {
                userId: userId,
                userType: userType,
                token: userToken
            }
        });
    } catch (error: any) {
        throw error;
    }
};

const logUserToken = async (userId: string, userType: string, userToken: string) => {
    try {
        return UserToken.create({
            userId: userId,
            userType: userType,
            token: userToken
        });
    } catch (error: any) {
        throw error;
    }
};

const isValidToken = async (userId: string, userType: string, userToken: string) => {
    try {
        const token = await UserToken.findOne({
            where: {
                userId: userId,
                userType: userType,
                token: userToken
            }
        });
        return token !== null;
    } catch (error: any) {
        throw error;
    }
};

export default {
    createUser,
    checkUserByEmail,
    checkUserByPhone,
    checkUserByUserName,
    createOtpAndSendEmail,
    userLogin,
    validateOTP,
    resendOTP,
    userLogout,
    logUserToken,
    isValidToken
};
