import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import NotificationSettingsService from "./notificationSettings.service";
import getErrorObject from "../../../utills/errorStructure";
import { CustomError } from "../../../utills/types";

const getUserNotificaitonSettings = async (req: Request, res: Response) => {
    try {
        const authUser = req['user'];
        const notificationType = String(req.query.notificationType);

        const notificationData = await NotificationSettingsService.getNotificationSettingsByUserIdNotificationType(authUser.id, notificationType)
        if (!notificationData) throw new CustomError(StaticMessages.getNotExistMessage('Notification Settings'), HTTPStatusCode.NOTFOUND)

        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getListReturnedMessage('Notification Settings'),
            notificationSettings: notificationData
        });
    } catch (err) {
        if (err instanceof Error) {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: StaticMessages.INTERNAL_SERVER_ERROR
            });
        } else {
            err = getErrorObject(err)
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: err.message
            });
        }
    }
};

const updateUserNotificaitonSettings = async (req: Request, res: Response) => {
    try {
        const authUser = req['user'];

        await NotificationSettingsService.updateNotificationSettingByCondition(authUser.id, req.body);

        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getUpdatedMessage('Notification Settings')
        });
    } catch (err) {
        if (err instanceof Error) {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: StaticMessages.INTERNAL_SERVER_ERROR
            });
        } else {
            err = getErrorObject(err)
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: err.message
            });
        }
    }
};

export default {
    updateUserNotificaitonSettings,
    getUserNotificaitonSettings
}