import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const updateNotificationSettingsValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("User Notification Settings -> Update Notification Settings")
    try {
        const schema = Joi.object().keys({
            notificationType: Joi.string().valid('EMAIL', 'PUSH').required().description('notificationType: EMAIL or PUSH').error(new Error('notificationType is missing or incoreect')),
            like: Joi.bool().required().description('like: true or false').error(new Error('like is missing or incoreect')),
            match: Joi.bool().required().description('match: true or false').error(new Error('match is missing or incoreect')),
            messages: Joi.bool().required().description('messages: true or false').error(new Error('messages is missing or incoreect')),
            promotions: Joi.bool().required().description('promotions: true or false').error(new Error('promotions is missing or incoreect')),
            annoucements: Joi.bool().required().description('annoucements: true or false').error(new Error('annoucements is missing or incoreect')),

        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const getNotificationSettingsValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("User Notification Settings -> get Notification Settings")
    try {
        const schema = Joi.object().keys({
            notificationType: Joi.string().valid('EMAIL', 'PUSH').required().description('notificationType: EMAIL or PUSH').error(new Error('notificationType is missing or incoreect'))
        });
        const validation = schema.validate(req.query);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }

        req.query = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    updateNotificationSettingsValidator,
    getNotificationSettingsValidator
}