import express, { NextFunction, Request, Response } from 'express';
import { Constants } from '../../../utills/constants';
import { HTTPStatusCode } from '../../../utills/enums';
import getErrorObject from '../../../utills/errorStructure';
import { StaticMessages } from '../../../utills/staticMessage';
import { Logging } from '../../commonServices/logging.service';
import NotificationService from './notification.service';

const getNotifications = async (req: Request, res: Response) => {
    try {
        let user = req['user'];
        let offset = req.query.offset;
        let limit = req.query.limit;
        let notifications = await NotificationService.getNotifications(user.id, offset, limit);
        res.status(HTTPStatusCode.SUCCESS).json({
            notifications: notifications,
            status: Constants.SUCCESS
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('Notification'),
            status: Constants.ERROR
        });
    }
};

const addNotifications = async (req: Request, res: Response) => {
    try {
        let user = req['user'];
        let createNotificationData = await NotificationService.addNotifications(user.id);
        res.status(HTTPStatusCode.SUCCESS).json({
            notification: createNotificationData,
            status: Constants.SUCCESS
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('Notification'),
            status: Constants.ERROR
        });
    }
};

export default {
    getNotifications,
    addNotifications
}