import db from "../../../models";
import moment from "moment";
import { Op } from 'sequelize';
import UserService from "../users/user.service";

const Post = db.Post;
const PostMedia = db.PostMedia;
const Tag = db.Tag;
const Notification = db.Notification;
const Comment = db.Comment;
const Reaction = db.Reaction;

const getNotifications = async (userId, offset, limit) => {
    try {
        let notifications = await Notification.findAll({
            where: {
                toUser: userId,
                expirationTimeStamp: {
                    [Op.gte]: moment().unix()
                }
            },
            offset: parseInt(offset),
            limit: parseInt(limit),
            raw: true,
        });
        for (let i = 0; i < notifications.length; i++) {
            let toUserData = await UserService.getUserById(notifications[i].fromUser, ['fcmToken', 'userName', 'profileThumbnail']);
            notifications[i]['fromUserName'] = toUserData.userName;
            notifications[i]['profileThumbnail'] = toUserData.profileThumbnail;
        }
        return notifications;
    } catch (err: any) {
        throw err;
    }
};

const addNotifications = async (data) => {
    try {
        let notifications = await Notification.create(data);
        return notifications;
    } catch (err: any) {
        throw err;
    }
};

export default {
    getNotifications,
    addNotifications
};