import { Request, Response, NextFunction } from "express";
import { Logging } from "../../commonServices/logging.service";
import { Utils } from "../../../utills/helperMethods";
import { HTTPStatusCode, OtpType } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import getErrorMessage from "../../../utills/errorStructure";
import { StaticMessages } from "../../../utills/staticMessage";
import OtpService from "./otp.service";
import getErrorObject from "../../../utills/errorStructure";

const sendSmsOTP = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let updateResponse = await OtpService.sendOtpOnSms(req.body.countryCode, req.body.phone, OtpType.SIGNUP);
        if (updateResponse) {
            res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.OTP_SENT + `${req.body.countryCode} ${req.body.phone}.`,
            });
        } else {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.OP_FAILED,
                error: {
                    message: StaticMessages.OP_FAILED,
                    statusCode: HTTPStatusCode.BAD_REQUEST
                },
                status: Constants.ERROR
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.OP_FAILED,
            status: Constants.ERROR
        });
    }
};

const validateSmsOTP = async (req: Request, res: Response, next: NextFunction) => {
    try {
        const user = req['user'];
        await OtpService.validateOtpOnSms(req.body.countryCode, req.body.phone, req.body.otp, OtpType.SIGNUP, user.id);
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.OTP_VERIFIED
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: error.messages,
            status: Constants.ERROR
        });
    }
};

export default {
    sendSmsOTP,
    validateSmsOTP
}