import { Utils } from "../../../utills/helperMethods";
import db from "../../../models";
import { HTTPStatusCode, OtpType } from "../../../utills/enums";
import { StaticMessages } from "../../../utills/staticMessage";
import { CustomError } from "../../../utills/types";
import { sendSMS } from "../../../utills/smsService";
import SignUpCheckListService from "../signUpCheckList/signUpCheckList.service";
import userService from "../users/user.service";
const User = db.User;
const OTP = db.Otp;

const sendOtpOnSms = async (countryCode: string, phone: string, type: any) => {
    try {
        const otp = Utils.randomDigits(4);
        let { count, rows } = await OTP.findAndCountAll({ where: { countryCode: countryCode, phone: phone } });
        let response = null;
        if (!count) {
            if (type == OtpType.SIGNUP) {
                response = await OTP.create({
                    signUpOTP: otp,
                    resetPasswordOTP: null,
                    countryCode: countryCode,
                    phone: phone
                });
            } else if (type == OtpType.RESET_PASSWORD) {
                response = await OTP.create({
                    signUpOTP: null,
                    resetPasswordOTP: otp,
                    countryCode: countryCode,
                    phone: phone
                });
            }
        } else {
            let updateOtp = {}
            if (type == OtpType.SIGNUP) {
                let userFound = await userService.exists({
                    phone: phone,
                    isVerified: 1
                });
                if(userFound) {
                    throw new CustomError(StaticMessages.PHONE_UNIQUE, HTTPStatusCode.CONFLICT);
                }
                updateOtp = {
                    signUpOTP: otp
                };
            } else if (type == OtpType.RESET_PASSWORD) {
                updateOtp = {
                    resetPasswordOTP: otp
                };
            }
            let updateResponse = await OTP.update(updateOtp, {
                where: { countryCode: countryCode, phone: phone }
            });
            response = updateResponse && updateResponse.length && updateResponse[0] !== '' && updateResponse[0] !== 0;
        }
        await sendSMS(countryCode, phone, Utils.getOtpSMSText(otp));
        return response;
    } catch (error: any) {
        throw error;
    }
};

const validateOtpOnSms = async (countryCode: string, phone: string, otp: number, otpType: number, userId: any) => {
    try {
        let countAndRow: any;
        if (OtpType.SIGNUP === otpType) {
            countAndRow = await OTP.findAndCountAll({ where: { countryCode: countryCode, phone: phone, signUpOTP: otp } });
            // temporarily not validating for the testing purpose
            // if (countAndRow.count) {
                await User.update({ isVerified: 1, countryCode: countryCode, phone: phone }, {
                    where: {
                        id: userId
                    }
                });   
                await SignUpCheckListService.updateCheckListByUserId(userId, { mobileVerified: true });
                return true;
            // }
        } else if (OtpType.RESET_PASSWORD === otpType) {
            countAndRow = await OTP.findAndCountAll({ where: { countryCode: countryCode, phone: phone, resetPasswordOTP: otp } });
            if (countAndRow.count) {
                return true;
            }
        } else {
            throw new CustomError(StaticMessages.INVALID_OTP_TYPE, HTTPStatusCode.BAD_REQUEST);
        }
        if (!countAndRow.count) {
            throw new CustomError(StaticMessages.INVALID_OTP, HTTPStatusCode.BAD_REQUEST);
        }
    } catch (error: any) {
        throw error;
    }
};

export default {
    sendOtpOnSms,
    validateOtpOnSms
};
