import { NextFunction, Request, Response } from "express";
import Joi, { valid } from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const sendSmsOTP = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("User Validator  -> POST sendSmsOTP Validator");
    try {
        const schema = Joi.object({
            countryCode: Joi.string().allow('').description('countryCode').required(),
            phone: Joi.string().allow('').description('phone').required()
        }).unknown()

        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const validateSmsOTP = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            requestHeader: Joi.object().optional().description('requestHeader').error(new Error('Request Header is missing')),
            countryCode: Joi.string().allow('').description('countryCode').required(),
            phone: Joi.string().allow('').description('phone').required(),
            otp: Joi.string().allow('').description('otp').required()
        })

        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};
export default {
    sendSmsOTP,
    validateSmsOTP
}