import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addCardValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("PaymentGateway  -> addCardValidator")
    try {
        const schema = Joi.object().keys({
            cardNumber: Joi.string().min(16).max(16).required().description('cardNumber').error(new Error('cardNumber is missing or invalid')),
            cardExpiryDate: Joi.string().description('cardExpiryDate').required().error(new Error('cardExpiryDate is missing or invalid')),
            cvv: Joi.string().min(3).max(4).description('cvv').required().error(new Error('cvv is missing or invalid'))
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const deleteCardValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("PaymentGateway  -> deleteCardValidator")
    try {
        const schema = Joi.object().keys({
            paymentProfileId: Joi.string().description('paymentProfileId').required().error(new Error('paymentProfileId is missing or incoreect'))
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addCardValidator,
    deleteCardValidator
}