import express from 'express';
import { tokenValidation } from '../../middlewares/validateToken';
import PostsController from './posts.controller';
import PostsValidator from './posts.validator';
const router = express.Router();

// Add Post
router.post('/posts', tokenValidation, PostsValidator.addPost, PostsController.addPost);

// Delete Post
router.delete('/posts/:id', tokenValidation, PostsController.deletePost);

// Update Post
router.put('/posts/:id', tokenValidation, PostsValidator.updatePost, PostsController.updatePost);

// Get Posts Feed
router.get('/posts', tokenValidation, PostsController.getFriendsPosts);

// Get Post by Id
router.get('/posts/:id', tokenValidation, PostsController.getPostById);

// Get Post Comments
router.get('/posts/:id/comments', tokenValidation, PostsController.getComments);

// Add Post Comment
router.post('/posts/:id/comments', tokenValidation, PostsValidator.addComments, PostsController.addComments);

// reply to a Comment
router.post('/posts/:id/comments/:commentId', tokenValidation, PostsValidator.addComments, PostsController.addComments);

// Remove Reaction on Comment
router.delete('/posts/comments/reactions/', tokenValidation, PostsValidator.removeReactionFromComment, PostsController.removeReactionOnComment);

// Edit Comment
router.put('/posts/:id/comments/:id', tokenValidation, PostsValidator.updateComment, PostsController.updateComment);

// Delete Comment
router.delete('/posts/comments/:commentId', tokenValidation, PostsController.deleteComment);

// Add Reaction On Post
router.post('/posts/reactions', tokenValidation, PostsController.addReaction);

// Remove Post Reaction
router.delete('/posts/reactions/posts', tokenValidation, PostsValidator.removeReactionFromPost, PostsController.removeReaction);

// Add Reaction on Comment
router.post('/posts/comments/reactions/', tokenValidation, PostsValidator.addReactionOnComment, PostsController.addReactionOnComment);


export default router;