import { Utils } from "../../../utills/helperMethods";
import db from "../../../models";
import { HTTPStatusCode, MediaSources, NotificationType } from "../../../utills/enums";
import { StaticMessages } from "../../../utills/staticMessage";
import { CustomError } from "../../../utills/types";
import { handleFileUpload } from "../../commonServices/fsFileUpload";
import FollowingFollowerService from "../followingFollower/followingFollower.service";
import { Sequelize } from "sequelize";

const Post = db.Post;
const User = db.User;
const PostMedia = db.PostMedia;
const Tag = db.Tag;
const Comment = db.Comment;
const Reaction = db.Reaction;
const createPost = async (userId, data) => {
    try {
        let taggedUserIds = data.tags;
        let createPostRes = await Post.create({
            postText: data.postText,
            location: data.location,
            UserId: userId
        });
        createPostRes = JSON.parse(JSON.stringify(createPostRes))
        let filePaths;
        const postMedias = [], taggedUserData = [];
        filePaths = await handleFileUpload({
            type: MediaSources.POST,
            files: data.files,
            itemId: createPostRes.id,
            userId: userId
        });
        await Promise.all(filePaths.map(async (mediaPath, index) => {
            const postMedia = await PostMedia.create({
                mediaPath: mediaPath,
                PostId: createPostRes.id,
                mediaPosition: index + 1
            });
            postMedias.push({
                postMediaId: postMedia.id,
                mediaPath: postMedia.mediaPath,
                mediaPosition: postMedia.mediaPosition,
            });
        }))
        createPostRes.postMedias = postMedias;
        if (taggedUserIds) {
            for (let i = 0; i < taggedUserIds.length; i++) {
                await Utils.sendNotification({
                    "fromUserId": userId,
                    "toUserId": taggedUserIds[i],
                    "message": "has tagged you in your post",
                    "type": NotificationType.TAGGED_IN_POST,
                    "postId": createPostRes.id,
                    notificationData: {
                        postId: createPostRes.id
                    }
                });

                taggedUserData.push(await User.findOne({
                    where: {
                        id: taggedUserIds[i]
                    },
                    attributes: [
                        'id',
                        'userName',
                        'firstName',
                        'lastName',
                        'profilePicture'
                    ],
                    raw: true
                }))

                taggedUserIds[i] = {
                    PostId: createPostRes.id,
                    UserId: taggedUserIds[i]
                };
            }
            // let tagsData = taggedUserIds.map(tagUserId => {
            //     tagUserId = {
            //         PostId: createPostRes.id,
            //         UserId: tagUserId
            //     }
            //     return tagUserId;
            // });
            Tag.bulkCreate(taggedUserIds, { returning: true });
        }
        createPostRes.taggedUserData = taggedUserData;
        return createPostRes;
    } catch (err: any) {
        throw err;
    }
};

const deletePost = async (postId: string, authUserId: string) => {
    try {
        return await Post.destroy({
            where: {
                id: postId,
                UserId: authUserId
            }
        });
    } catch (err: any) {
        throw err;
    }
};

const updatePost = async (postId, userId, data) => {
    try {
        let updatedPosResp = await Post.update(data, {
            where: {
                id: postId,
                UserId: userId
            }
        });
        if (updatedPosResp && updatedPosResp.length && updatedPosResp[0] > 0) {
            let taggedUserIds = data.tags;
            let tagIds = await Tag.findAll({
                where: {
                    PostId: postId,
                    UserId: userId
                },
                attributes: [
                    "id"
                ]
            });
            if (taggedUserIds) {
                taggedUserIds = taggedUserIds.filter(id => tagIds.indexOf(id) === -1);
                for (let i = 0; i < taggedUserIds.length; i++) {
                    await Utils.sendNotification({
                        "fromUserId": userId,
                        "toUserId": taggedUserIds[i],
                        "message": "has tagged you in your post",
                        "type": NotificationType.TAGGED_IN_POST,
                        "postId": postId,
                        notificationData: {
                            postId: postId
                        }
                    });
                    taggedUserIds[i] = {
                        PostId: postId,
                        UserId: taggedUserIds[i]
                    };
                }
                Tag.bulkCreate(taggedUserIds, { returning: true });
            }
            return updatedPosResp;
        } else {
            throw new CustomError(StaticMessages.getUpdateFailedMessage('Post'), HTTPStatusCode.BAD_REQUEST);;
        }
    } catch (err: any) {
        throw err;
    }
};

const addComments = async (data) => {
    try {
        let comment = await Comment.create(data);
        await Utils.sendNotification({
            "fromUserId": data.UserId,
            "toUserId": data.postUserId,
            "type": NotificationType.POST_COMMENT,
            "message": 'has added a comment on your post',
            "commentId": data.CommentId,
            notificationData: {
                commentId: data.CommentId
            }
        });
        return comment;
    } catch (err: any) {
        throw err;
    }
};

const getComments = async (postId, limit = 20, offset = 0) => {
    try {
        let comments = await Comment.findAll({
            where: {
                PostId: postId
            },
            include: [
                {
                    model: db.User,
                    attributes: [
                        "userName",
                        "profileThumbnail",
                    ]
                },
            ],
            attributes: [
                ['id', 'id'],
                ['text', 'text'],
                ['PostId', 'postId'],
                ['UserId', 'userId'],
                ['parentId', 'parentId'],
                ['createdAt', 'createdAt'],
                ['updatedAt', 'updatedAt'],
                [Sequelize.col('User.userName'), 'userName'],
                [Sequelize.col('User.profileThumbnail'), 'userThumbnail'],
            ],
            order: [
                ['parentId', 'DESC']
            ],
            offset: offset,
            limit: limit,
            raw: true
        });

        let commentByIdObjects = comments.reduce((acc, comment) => {
            delete comment["User.userName"];
            delete comment["User.profileThumbnail"];
            acc[comment.id] = comment;
            return acc;
        }, {});

        // Object.keys(commentByIdObjects)
        comments.forEach((comment) => {
            if (comment.parentId) {
                commentByIdObjects[comment.parentId] = { ...commentByIdObjects[comment.parentId], "comments": commentByIdObjects[comment.id] };
                delete commentByIdObjects[comment.id];
            }
        });
        let commentsArray = Object.keys(commentByIdObjects).reduce((acc, commentId) => {
            acc.push(commentByIdObjects[commentId]);
            return acc;
        }, []);
        return commentsArray;
    } catch (err: any) {
        throw err;
    }
};

const updateComment = async (id, userId, data) => {
    try {
        return await Comment.update(data, {
            where: {
                id: id,
                UserId: userId
            }
        });
    } catch (err: any) {
        throw err;
    }
};

const deleteComment = async (commentId: string, userId) => {
    try {
        return await Comment.destroy({
            where: {
                id: commentId,
                UserId: userId
            }
        });

    } catch (err: any) {
        throw err;
    }
};

const getPostById = async (postId) => {
    try {
        let post = await Post.findOne({
            where: {
                id: postId
            },
            include: [{
                model: db.User,
                as: 'User',
                attributes: [
                    'userName',
                    'firstName',
                    'lastName',
                    'profilePicture'
                ]
            }],
            attributes: [
                ['id', 'id'],
                ['postText', 'description'],
                ['location', 'location'],
                ['createdAt', 'createdAt'],
                [Sequelize.col('User.userName'), 'userName'],
                [Sequelize.col('User.firstName'), 'firstName'],
                [Sequelize.col('User.lastName'), 'lastName'],
                [Sequelize.col('User.profilePicture'), 'profileThumb'],
                ['UserId', 'UserId']
            ],
            raw: true
        });
        delete post['User.userName']
        delete post['User.firstName']
        delete post['User.lastName']
        delete post['User.profilePicture']
        post.postMedias = await PostMedia.findAll({
            where: {
                PostId: post.id
            },
            attributes: [
                ['id', 'postMediaId'],
                'mediaPath',
                'mediaPosition'
            ],
            raw: true
        });
        let taggedUsers = await Tag.findAll({
            where: {
                PostId: post.id
            },
            include: [{
                model: db.User,
                as: 'User',
                attributes: [
                    'id',
                    'userName',
                    'firstName',
                    'lastName',
                    'profilePicture'
                ]
            }],
            attributes: [
                [Sequelize.col('User.id'), 'UserId'],
                [Sequelize.col('User.userName'), 'userName'],
                [Sequelize.col('User.firstName'), 'firstName'],
                [Sequelize.col('User.lastName'), 'lastName'],
                [Sequelize.col('User.profilePicture'), 'profilePicture'],
            ],
            raw: true
        })
        post.taggedUserData = taggedUsers.map((tUser) => {
            delete tUser['User.id']
            delete tUser['User.userName']
            delete tUser['User.firstName']
            delete tUser['User.lastName']
            delete tUser['User.profilePicture']
            return tUser;
        })
        post.commentsCount = await Comment.count({
            where: {
                PostId: post.id
            }
        })
        post.reactionsCount = await Reaction.count({
            where: {
                PostId: post.id
            }
        })
        return post;
    } catch (error: any) {
        throw error;
    }
};

const getFriendsPosts = async (userId, otherUserId = "", limit = 20, offset = 0) => {
    try {
        limit = limit ? limit : 20;
        offset = offset ? offset : 0;
        let ids = await FollowingFollowerService.getUserFollowingIds(userId);
        ids = [...ids, userId];

        if (otherUserId !== "") {
            ids = [otherUserId];
        }

        let posts = await Post.findAll({
            where: {
                UserId: ids
            },
            include: [{
                model: db.User,
                as: 'User',
                attributes: [
                    'userName',
                    'firstName',
                    'lastName',
                    'profilePicture'
                ]
            }],
            attributes: [
                ['id', 'id'],
                ['postText', 'description'],
                ['location', 'location'],
                ['createdAt', 'createdAt'],
                [Sequelize.col('User.userName'), 'userName'],
                [Sequelize.col('User.firstName'), 'firstName'],
                [Sequelize.col('User.lastName'), 'lastName'],
                [Sequelize.col('User.profilePicture'), 'profileThumb'],
                ['UserId', 'UserId']
            ],
            offset: offset,
            limit: limit,
            raw: true
        });

        posts = Promise.all(await posts.map(async (post) => {
            delete post['User.userName']
            delete post['User.firstName']
            delete post['User.lastName']
            delete post['User.profilePicture']

            post.postMedias = await PostMedia.findAll({
                where: {
                    PostId: post.id
                },
                attributes: [
                    ['id', 'postMediaId'],
                    'mediaPath',
                    'mediaPosition'
                ],
                raw: true
            });

            let taggedUsers = await Tag.findAll({
                where: {
                    PostId: post.id
                },
                include: [{
                    model: db.User,
                    as: 'User',
                    attributes: [
                        'id',
                        'userName',
                        'firstName',
                        'lastName',
                        'profilePicture'
                    ]
                }],
                attributes: [
                    [Sequelize.col('User.id'), 'UserId'],
                    [Sequelize.col('User.userName'), 'userName'],
                    [Sequelize.col('User.firstName'), 'firstName'],
                    [Sequelize.col('User.lastName'), 'lastName'],
                    [Sequelize.col('User.profilePicture'), 'profilePicture'],
                ],
                raw: true
            })

            post.taggedUserData = taggedUsers.map((tUser) => {
                delete tUser['User.id']
                delete tUser['User.userName']
                delete tUser['User.firstName']
                delete tUser['User.lastName']
                delete tUser['User.profilePicture']
                return tUser;
            });
            post.commentsCount = await Comment.count({
                where: {
                    PostId: post.id
                }
            });
            post.reactionsCount = await Reaction.count({
                where: {
                    PostId: post.id
                }
            });
            return post;
        }));
        return posts;
    } catch (error: any) {
        throw error;
    }
};

const addCommentReaction = async (data: any) => {
    try {
        Reaction.create(data);
        await Utils.sendNotification({
            "fromUserId": data.UserId,
            "toUserId": data.CommentUserId,
            "message": "has Liked your Comment",
            "postId": data.postId,
            "type": NotificationType.TAGGED_IN_POST,
            notificationData: {
                postId: data.postId
            }
        });
    } catch (err: any) {
        throw err;
    }
};

const addReaction = async (data) => {
    try {
        Reaction.create(data);
        await Utils.sendNotification({
            "fromUserId": data.UserId,
            "toUserId": data.PostUserId,
            "message": "has Liked your Post",
            "postId": data.PostId,
            "type": NotificationType.TAGGED_IN_POST,
            notificationData: {
                postId: data.PostId
            }
        });
    } catch (err: any) {
        throw err;
    }
};

const removeReaction = async (data: any) => {
    try {
        Reaction.destroy({
            where: data
        });
    } catch (err: any) {
        throw err;
    }
};

export default {
    createPost,
    deletePost,
    updatePost,
    getFriendsPosts,
    getPostById,
    addComments,
    updateComment,
    getComments,
    deleteComment,
    removeReaction,
    addReaction,
    addCommentReaction
};