import { Request, Response, NextFunction } from "express";
import { Logging } from "../../commonServices/logging.service";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import getErrorMessage from "../../../utills/errorStructure";
import profileMatchesService from "./profileMatches.service";
import UserService from "../users/user.service";
import { StaticMessages } from "../../../utills/staticMessage";
import { FollowStatus } from "../../../utills/enums";

const getUserMatches = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let userId = req['user'].id;
        if (req.params.id) {
            userId = req.params.id;
        }
        console.log('17',userId);
        let matches = await profileMatchesService.getUserMatches(userId, req.query.limit, req.query.offset);
        if (matches) {
            res.status(HTTPStatusCode.SUCCESS).json({
                matches: matches,
                message: StaticMessages.getItemMessage("Matches of user"),
                status: Constants.SUCCESS
            });
        } else {
            res.status(HTTPStatusCode.NOTFOUND).json({
                message: StaticMessages.NO_RECORD_FOUND,
                status: Constants.ERROR
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error)
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

export default {
    getUserMatches,
}