import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import PromptsCategoryService from "./promptsCategory.service";

const addNewPromptsCategory = (req: Request, res: Response) => {
    const addPromptsCategory = () => new Promise((resolve, reject) => {
        const insertData = {
            name: req.body.promptsCategory
        };
        return PromptsCategoryService.addPromptsCategory(insertData)
            .then((data) => resolve(data))
            .catch(err => reject(err))
    });

    return addPromptsCategory()
        .then(() => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getCreatedMessage('Prompts Category')
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

const getPromptsCategories = (req: Request, res: Response) => {
    const getPromptsCategory = () => new Promise((resolve, reject) => {
        return PromptsCategoryService.getPromptsCategoriesWithoutCond()
            .then((data) => {
                if (data.length > 0) {
                    resolve(data);
                }
                reject({ message: StaticMessages.getNotExistMessage('Prompts Category'), status: HTTPStatusCode.NOTFOUND });
            })
            .catch(err => reject(err))
    });

    return getPromptsCategory()
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Prompts Category'),
                promptsCategories: data
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

export default {
    addNewPromptsCategory,
    getPromptsCategories
}