import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addPromptsCategoryPayloadValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Prompts Category Validator  -> Add New Prompts Category Validator")
    try {
        const schema = Joi.object().keys({
            promptsCategory: Joi.string().required().description('promptsCategory: String').error(new Error('promptsCategory is missing or incoreect')),
        })
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addPromptsCategoryPayloadValidator
}