import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import PromptsQuestionsService from "./promptsQuestions.service";
import PromptsCategoryService from "../promptsCategory/promptsCategory.service";
import { Logging } from "../../commonServices/logging.service";

const addNewPromptsQuestions = (req: Request, res: Response) => {
    const checkQuesCategoryExistOrNot = () => new Promise<void>((resolve, reject) => {
        return PromptsCategoryService.getOnePromptsCategoryById(req.body.promptsCategoryId)
            .then((data) => {
                if (!data) reject({ message: StaticMessages.getNotExistMessage('PromptsQuestion Category'), status: HTTPStatusCode.NOTFOUND });
                resolve();
            })
            .catch(err => reject(err))
    });

    const addQuestion = () => new Promise((resolve, reject) => {
        const insertData = {
            name: req.body.name,
            createdAt: Date.now(),
            updatedAt: Date.now(),
            promptsCategoryId: req.body.promptsCategoryId
        };
        return PromptsQuestionsService.addPromptsQuestion(insertData)
            .then((data) => resolve(data))
            .catch(err => reject(err))
    });
    return checkQuesCategoryExistOrNot()
        .then(addQuestion)
        .then(() => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getCreatedMessage('Questions')
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

const getQuestionList = (req: Request, res: Response) => {
    const getQuestions = () => new Promise((resolve, reject) => {
        return PromptsQuestionsService.getQuesionWithOutCond()
            .then((data) => {
                if (data.length > 0) {
                    data.map((dt) => {
                        delete dt.PromptsCategory;
                        delete dt['PromptsCategory.name'];
                        return dt;
                    })
                    resolve(data);
                }
                reject({ message: StaticMessages.getNotExistMessage('Questions'), status: HTTPStatusCode.NOTFOUND });
            })
            .catch(err => reject(err))
    });

    return getQuestions()
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Questions'),
                promptsQuestions: data
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

const getQuestionListByPromptsCategoryId = (req: Request, res: Response) => {
    const getQuestions = () => new Promise((resolve, reject) => {
        return PromptsQuestionsService.getQuesionByPromptsCategoryId(String(req.query.promptsCategoryId))
            .then((data) => {
                if (data.length > 0) {
                    data.map((dt) => {
                        delete dt.PromptsCategory;
                        delete dt['PromptsCategory.name']
                        return dt;
                    })
                    resolve(data);
                }
                reject({ message: StaticMessages.getNotExistMessage('Questions'), status: HTTPStatusCode.NOTFOUND });
            })
            .catch(err => reject(err))
    });

    return getQuestions()
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Questions'),
                promptsQuestions: data
            });
        }).catch((err) => {
            Logging.logMessage(err);
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

export default {
    addNewPromptsQuestions,
    getQuestionList,
    getQuestionListByPromptsCategoryId
}