import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addQuesCategoryPayloadValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("PromptsQuestion Category Validator  -> Add New PromptsQuestion Category Validator")
    try {
        const schema = Joi.object().keys({
            name: Joi.string().required().description('name: String').error(new Error('name is missing or incoreect')),
            promptsCategoryId: Joi.string().required().description('promptsCategoryId: String').error(new Error('promptsCategoryId is missing or incoreect'))
        })
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const getChoicesQueryValidatior = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("PromptsQuestion Category Validator  -> GET choices query Validator")
    try {
        const schema = Joi.object().keys({
            promptsCategoryId: Joi.string().required().description('promptsCategoryId: String').error(new Error('promptsCategoryId is missing or incoreect'))
        })
        const validation = schema.validate(req.query);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addQuesCategoryPayloadValidator,
    getChoicesQueryValidatior
}