import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import QuestionCategoryService from "./questionCategory.service";

const getQuestionCategories = (req: Request, res: Response) => {
    const getQUesCategory = () => new Promise((resolve, reject) => {
        return QuestionCategoryService.getQuestionCategoriesWithoutCond()
            .then((data) => {
                if (data.length > 0) {
                    resolve(data);
                }
                reject({ message: StaticMessages.getNotExistMessage('Question Category'), status: HTTPStatusCode.NOTFOUND });
            })
            .catch(err => reject(err))
    });

    return getQUesCategory()
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Question Category'),
                quesCategories: data
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

export default {
    getQuestionCategories
}
