import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addQuesCategoryPayloadValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Question Category Validator  -> Add New Question Category Validator")
    try {
        const schema = Joi.object().keys({
            questionCategory: Joi.string().required().description('questionCategory: String').error(new Error('questionCategory is missing or incoreect')),
        })
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addQuesCategoryPayloadValidator
}