import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import QuestionsService from "./questions.service";
import QuestionCategoryService from "../questionCategory/questionCategory.service";
import { Logging } from "../../commonServices/logging.service";

const addNewQuestions = (req: Request, res: Response) => {
    const checkQuesCategoryExistOrNot = () => new Promise<void>((resolve, reject) => {
        return QuestionCategoryService.getOneQuestionCategoryById(req.body.questionCategoryId)
            .then((data) => {
                if (!data) reject({ message: StaticMessages.getNotExistMessage('Question Category'), status: HTTPStatusCode.NOTFOUND });
                resolve();
            })
            .catch(err => reject(err))
    });

    const addQuestion = () => new Promise((resolve, reject) => {
        const insertData = {
            name: req.body.name,
            isSearchable: req.body.isSearchable,
            quesCategoryId: req.body.questionCategoryId
        };
        return QuestionsService.addQuesion(insertData)
            .then((data) => resolve(data))
            .catch(err => reject(err))
    });
    return checkQuesCategoryExistOrNot()
        .then(addQuestion)
        .then(() => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getCreatedMessage('Questions')
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

const getQuestionList = (req: Request, res: Response) => {
    const getQuestions = () => new Promise((resolve, reject) => {
        return QuestionsService.getQuesionWithOutCond()
            .then((data) => {
                if (data.length > 0) {
                    if(data.length >= 2) {
                        [ data[0], data[1] ] = [ data[1], data[0] ]
                    }
                    data.map((dt) => {
                        delete dt.QuestionCategory;
                        delete dt['QuestionCategory.name'];
                        return dt;
                    })
                    resolve(data);
                }
                reject({ message: StaticMessages.getNotExistMessage('Questions'), status: HTTPStatusCode.NOTFOUND });
            })
            .catch(err => reject(err))
    });

    return getQuestions()
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Questions'),
                questions: data
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

const getQuestionListByQuesCategoryId = (req: Request, res: Response) => {
    const getQuestions = () => new Promise((resolve, reject) => {
        return QuestionsService.getQuesionByQuestionCategoryId(String(req.query.quesCategoryId))
            .then((data) => {
                if (data.length > 0) {
                    data.map((dt) => {
                        delete dt.QuestionCategory;
                        delete dt['QuestionCategory.name']
                        return dt;
                    })
                    resolve(data);
                }
                reject({ message: StaticMessages.getNotExistMessage('Questions'), status: HTTPStatusCode.NOTFOUND });
            })
            .catch(err => reject(err))
    });

    return getQuestions()
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Questions'),
                questions: data
            });
        }).catch((err) => {
            Logging.logMessage(err);
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

export default {
    addNewQuestions,
    getQuestionList,
    getQuestionListByQuesCategoryId
}