import { Request, Response, NextFunction } from "express";
import { Logging } from "../../commonServices/logging.service";
import { HTTPStatusCode, OtpType } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import getErrorMessage from "../../../utills/errorStructure";
import { StaticMessages } from "../../../utills/staticMessage";
import ReactionService from "./reaction.service";

const addPostReaction = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let authUser = req['user'];
        let data = {};
        data['UserId'] = authUser.id;
        data['PostUserId'] = req.body.userId;
        data['PostId'] = req.body.postId;
        data['reactionType'] = req.body.reactionType? req.body.reactionType : 'like';
        await ReactionService.addPostReaction(data);
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.REACTION_SUCCESS,
        });
    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.REACTION_FAIL,
            status: Constants.ERROR
        });
    }
};

const addStoryReaction = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let authUser = req['user'];
        let data = {};
        data['UserId'] = authUser.id;
        data['StoryUserId'] = req.body.userId;
        data['StoryId'] = req.body.storyId;
        data['reactionType'] = req.body.reactionType? req.body.reactionType : 'like';
        await ReactionService.addStoryReaction(data);
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.REACTION_SUCCESS,
        });
    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.REACTION_FAIL,
            status: Constants.ERROR
        });
    }
};

const removeReaction = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let authUser = req['user'];
        let data = {};
        data['UserId'] = authUser.id;
        if (req.body.postId) {
            data['PostId'] = req.body.postId;
        }
        if (req.body.storyId) {
            data['StoryId'] = req.body.storyId;
        }
        if (req.body.commentId) {
            data['CommentId'] = req.body.commentId;
        }
        await ReactionService.removeReaction(data);
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.REACTION_REMOVE_SUCCESS,
        });
    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.REACTION_REMOVE_FAIL,
            status: Constants.ERROR
        });
    }
};

export default {
    addStoryReaction,
    addPostReaction,
    removeReaction
}