import db from "../../../models";
import FCM from 'fcm-node';
import { config } from "../../../config";
import UserService from "../users/user.service";
import { Logging } from "../../commonServices/logging.service";
const Reaction = db.Reaction;

const addPostReaction = async (data: any) => {
    try {
        Reaction.create(data);
        const fcm = new FCM(config.dev.fcm.server_key);
        let userData = await UserService.getUserById(data.PostUserId, ['fcmToken', 'userName', 'profileThumbnail']);
        const message = {
            to: userData.fcmToken,
            notification: {
                title: 'Datesafe',
                body: `${userData.userName} has Liked your Post`,
                sound: "ping.aiff",
                imageUrl: userData.profileThumbnail,
                delivery_receipt_requested: true,
                postId: data.PostId
            },
            data: {
                message: `${userData.userName} has Liked your Post`
            }
        };
        fcm.send(message, (d) => {
            Logging.logMessage('Post Liked Successfully.')
        });
    } catch (err: any) {
        throw err;
    }
};

const addStoryReaction = async (data: any) => {
    try {
        Reaction.create(data);
        const fcm = new FCM(config.dev.fcm.server_key);
        let userData = await UserService.getUserById(data.StoryUserId, ['fcmToken', 'userName', 'profileThumbnail']);
        const message = {
            to: userData.fcmToken,
            notification: {
                title: 'Datesafe',
                body: `${userData.userName} has Liked your Story`,
                sound: "ping.aiff",
                imageUrl: userData.profileThumbnail,
                delivery_receipt_requested: true,
                storyId: data.StoryId
            },
            data: {
                message: `${userData.userName} has Liked your Story`,
            }
        };
        fcm.send(message, (d) => {
            Logging.logMessage('Story Liked Successfully.')
        });
    } catch (err: any) {
        throw err;
    }
};

const addCommentReaction = async (data: any) => {
    try {
        Reaction.create(data);
        const fcm = new FCM(config.dev.fcm.server_key);
        let userData = await UserService.getUserById(data.CommentUserId, ['fcmToken', 'userName', 'profileThumbnail']);
        const message = {
            to: userData.fcmToken,
            notification: {
                title: 'Datesafe',
                body: `${userData.userName} has Liked your Comment`,
                sound: "ping.aiff",
                imageUrl: userData.profileThumbnail,
                delivery_receipt_requested: true,
                commentId: data.CommentId
            },
            data: {
                message: `${userData.userName} has Liked your Comment`,
            }
        };
        fcm.send(message, (d) => {
            Logging.logMessage('Comment Liked Successfully.')
        });
    } catch (err: any) {
        throw err;
    }
};

const removeReaction = async (data: any) => {
    try {
        Reaction.destroy({
            where: data
        });
    } catch (err: any) {
        throw err;
    }
};

const getStoryReactionsWithUserId = async (storyId: string, userId: string) => {
    try {
        return await Reaction.findOne({
            where: {
                UserId: userId,
                StoryId: storyId
            }
        })
    } catch (err: any) {
        throw err
    }
};

export default {
    addPostReaction,
    addStoryReaction,
    addCommentReaction,
    removeReaction,
    getStoryReactionsWithUserId
};