import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const getReportList = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            limit: Joi.string().allow('').default('20').description('limit').optional(),
            offset: Joi.string().allow('').default('0').description('offset').optional()
        })
        const validation = schema.validate(req.query);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.query = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const getSearchedReportList = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            userName: Joi.string().allow('').default('').description('userName').optional(),
            action: Joi.string().allow('').default('').description('action').optional(),
            fromDate: Joi.string().allow('').default('').description('fromDate').optional(),
            toDate: Joi.string().allow('').default('').description('toDate').optional(),
            limit: Joi.string().allow('').default('20').description('limit').optional(),
            offset: Joi.string().allow('').default('0').description('offset').optional()
        })
        const validation = schema.validate(req.query);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.query = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const createReport = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            comment: Joi.string().allow('').description('comment').required(),
            reportReason: Joi.string().allow('').description('reportReason').required(),
            toUserId: Joi.string().allow('').description('toUserId').required()
        })
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    getReportList,
    getSearchedReportList,
    createReport
}