import db from "../../../models";
const Story = db.Story;
import sequelize, { Op } from "sequelize";
import FCM from 'fcm-node';
import { config } from "../../../config";
import UserService from "../users/user.service";
import { Logging } from "../../commonServices/logging.service";
import StoryViewsService from "../storyViews/storyViews.service";
import moment from "moment";
import { CommonAnswers, DatePageActions, FolderNames, Questions } from "../../../utills/enums";
import ReactionService from "../reaction/reaction.service";
import storyPrivacyService from "../storyPrivacy/storyPrivacy.service";
import { Constants } from "../../../utills/constants";
import userCharacteristicService from "../userCharacteristic/userCharacteristic.service";
const Reaction = db.Reaction;
const DatePageAction = db.DatePageAction;

const createStory = async (data: any) => {
    try {
        return await Story.create(data);
    } catch (err: any) {
        throw err
    }
};

const addStoryReaction = async (data: any) => {
    try {
        Reaction.create(data);
        const fcm = new FCM(config.dev.fcm.server_key);
        let userData = await UserService.getUserById(data.StoryUserId, ['fcmToken', 'userName', 'profileThumbnail']);
        const message = {
            to: userData.fcmToken,
            notification: {
                title: 'Datesafe',
                body: `${userData.userName} has Liked your Story`,
                sound: "ping.aiff",
                imageUrl: userData.profileThumbnail,
                delivery_receipt_requested: true,
                storyId: data.StoryId
            },
            data: {
                message: `${userData.userName} has Liked your Story`,
            }
        };
        fcm.send(message, (d) => {
            Logging.logMessage('Story Liked Successfully.')
        });
    } catch (err: any) {
        throw err;
    }
};
const getStoriesByFollowingIdUserId = async (followingIds: any, userId: string, limit = 20, offset = 0) => {
    try {
        const condition = {
            include: [
                {
                    model: db.User,
                    as: "User",
                    attributes: [
                        "userName",
                        "profilePicture"
                    ]
                },
            ],
            where: {
                UserId: {
                    [Op.in]: followingIds
                },
                expire: {
                    [Op.gt]: moment().unix()
                }
            },
            attributes: [
                [sequelize.col('User.userName'), 'userName'],
                [sequelize.col('User.profilePicture'), 'profilePicture'],
                [sequelize.fn('COUNT', sequelize.col('UserId')), 'mediaCount'],
                ["UserId", "userId"],
            ],
            group: ["UserId", "User.userName", "User.profilePicture", "User.id"],
            offset: offset,
            limit: limit,
            raw: true
        }
        let stories = await Story.findAll(condition);

        for (let i = 0; i < stories.length; i++) {
            let story = stories[i];
            const isReqesterIsHiddenUser = await storyPrivacyService.getStoryPrivacyByHideByAndHideForUserId(story.userId, parseInt(userId))
            if (isReqesterIsHiddenUser) {
                stories.splice(stories[i], 1);
            };
        }

        for (let i = 0; i < stories.length; i++) {
            let story = stories[i];
            delete story['User.userName'];
            delete story['User.profilePicture'];
            story['profilePicture'] = story['profilePicture'] || '';

            let storyList = await getStoriesByUserId(story.userId);
            for (let j = 0; j < storyList.length; j++) {
                storyList[j]['url'] = '/static/users/' + storyList[j]['userId'] + '/' + FolderNames.STORIES + '/' + storyList[j]['url'];
                storyList[j]['thumbnail'] = '/static/users/' + storyList[j]['userId'] + '/' + FolderNames.STORIES + '/' + storyList[j]['thumbnail'];
                if (storyList[j]['userId'] !== userId) {
                    delete storyList[j]['totalViewsCount'];
                }
                const isViewed = await StoryViewsService.getStoryViewByUserIdAndStoryId(userId, storyList[j].id);
                storyList[j]['isViewed'] = !!isViewed;

                const isLiked = await ReactionService.getStoryReactionsWithUserId(storyList[j].id, userId);
                storyList[j]['isLiked'] = !!isLiked;
            }

            story['isViewed'] = storyList.filter((st) => st.isViewed == false).length == 0;

            story['storyMedia'] = storyList;
        }

        return stories;
    } catch (err: any) {
        throw err
    }
};

const getStories = async (userId: string, limit, offset) => {
    try {
        if(!limit) {
            limit = 50;
        } else {
            limit = parseInt(limit);
        }
        if(!offset) {
            offset = 0;
        } else {
            offset = parseInt(offset);
        }
        const likedAccounts = await DatePageAction.findAll({
            where: {
                userId,
                actionType: DatePageActions.LIKE
            }
        });
        const idsForStories = likedAccounts.map((like) => like.actionUserId );
        const condition = {
            where: {
                userId: idsForStories,
                expire: {
                    [Op.gt]: moment().unix()
                }
            },
            attributes: [
                [sequelize.fn('COUNT', sequelize.col('UserId')), 'mediaCount'],
                ["UserId", "userId"],
            ],
            group: ["UserId"],
            offset: offset,
            limit: limit,
            raw: true
        }
        let stories = await Story.findAll(condition);

        for (let i = 0; i < stories.length; i++) {
            let story = stories[i];
            const isReqesterIsHiddenUser = await storyPrivacyService.getStoryPrivacyByHideByAndHideForUserId(story.userId, parseInt(userId))
            if (isReqesterIsHiddenUser) {
                stories.splice(stories[i], 1);
            };
        }

        for (let i = 0; i < stories.length; i++) {
            let story = stories[i];
            const userData = await UserService.getUserById(story.userId);
            const userBasicInfo = {
                userName: userData.userName,
                firstName: userData.firstName,
                lastName: userData.lastName,
                profilePicture: userData['profilePicture'] ? '/static/users/' + story.userId + '/' + FolderNames.PROFILE_PICTURE + '/' + userData['profilePicture'] : "",
                profileThumbnail: userData['profileThumbnail'] ? '/static/users/' + story.userId + '/' + FolderNames.PROFILE_PICTURE + '/' + userData['profileThumbnail'] : "",
                age: userData['DOB'] == null ? Constants.DEFAULT_AGE : moment().diff(userData['DOB'], 'years'),
                works: null,
                educationLevel: null,
                university: null,
                hometown: null
            }

            const workQuestion = await userCharacteristicService.getCharacteristicsByUserIdQuestionId(story.userId, Questions.WORKS);
            userBasicInfo.works = workQuestion[0] ? workQuestion[0].customAns : null;

            const eductaionLevelQuestion = await userCharacteristicService.getCharacteristicsByUserIdQuestionId(story.userId, Questions.EDUCATION_LEVEL);
            const universityQuestion = await userCharacteristicService.getCharacteristicsByUserIdQuestionId(story.userId, Questions.COLLEGE_UNIVERSITY);
            if (eductaionLevelQuestion.length > 0 && universityQuestion.length > 0) {
                if (eductaionLevelQuestion[0].answers[0].answerText != CommonAnswers.PREFER_NOT_TO_SAY) {
                    userBasicInfo.educationLevel = eductaionLevelQuestion[0].answers[0].answerText;
                    userBasicInfo.university = universityQuestion[0].customAns;
                }
            }

            const hometownQuestion = await userCharacteristicService.getCharacteristicsByUserIdQuestionId(story.userId, Questions.HOMETOWN);
            userBasicInfo.hometown = hometownQuestion[0] ? hometownQuestion[0].customAns : null;
            story['userBasicInfo'] = userBasicInfo;
            let storyList = await getStoriesByUserId(story.userId);
            for (let j = 0; j < storyList.length; j++) {
                storyList[j]['url'] = '/static/users/' + storyList[j]['userId'] + '/' + FolderNames.STORIES + '/' + storyList[j]['url'];
                storyList[j]['thumbnail'] = '/static/users/' + storyList[j]['userId'] + '/' + FolderNames.STORIES + '/' + storyList[j]['thumbnail'];
                if (storyList[j]['userId'] !== userId) {
                    delete storyList[j]['totalViewsCount'];
                }
                const isViewed = await StoryViewsService.getStoryViewByUserIdAndStoryId(userId, storyList[j].id);
                storyList[j]['isViewed'] = !!isViewed;

                const isLiked = await ReactionService.getStoryReactionsWithUserId(storyList[j].id, userId);
                storyList[j]['isLiked'] = !!isLiked;
            }

            story['isViewed'] = storyList.filter((st) => st.isViewed == false).length == 0;

            story['storyMedia'] = storyList;
        }

        return stories;
    } catch (err: any) {
        throw err
    }
};

const getStoriesByCondition = async (condition: any) => {
    try {
        return await Story.findAll(condition)
    } catch (err: any) {
        throw err
    }
};

const getStoriesByUserId = async (userId: string) => {
    try {
        return await Story.findAll({
            where: {
                UserId: userId,
                expire: {
                    [Op.gt]: moment().unix()
                }
            },
            attributes: [
                "id",
                "type",
                "typeText",
                "thumbnail",
                ["mediaPath", "url"],
                "timestamp",
                "totalViewsCount",
                "caption",
                ["UserId", "userId"],
            ],
            order: [["timestamp", "ASC"]],
            raw: true
        });
    } catch (err: any) {
        throw err
    }
};

const getStoriesByStoryIdUserId = async (storyId: string, userId: string) => {
    try {
        return await Story.findAll({
            where: {
                UserId: userId,
                id: storyId,
                expire: {
                    [Op.gt]: moment().unix()
                }
            }
        })
    } catch (err: any) {
        throw err
    }
};

const getStoriesByStoryIdNotWithUserId = async (storyId: string, userId: string) => {
    try {
        return await Story.findAll({
            where: {
                id: storyId,
                UserId: {
                    [Op.notIn]: userId
                },
                expire: {
                    [Op.gt]: moment().unix()
                }
            }
        })
    } catch (err: any) {
        throw err
    }
};

const deleteStoriesByStoryIdUserId = async (storyId: string, userId: string) => {
    try {
        return await Story.destroy({
            where: {
                UserId: userId,
                id: storyId
            }
        });
    } catch (err: any) {
        throw err
    }
};

const makeViewIncrementWithColNameAndId = async (colName: any, id: any) => {
    try {
        return await Story.increment({ [colName]: 1 }, {
            where: {
                id: parseInt(id)
            }
        });
    } catch (err: any) {
        throw err
    }
};
const removeReaction = async (data: any) => {
    try {
        Reaction.destroy({
            where: data
        });
    } catch (err: any) {
        throw err;
    }
};

export default {
    createStory,
    getStoriesByFollowingIdUserId,
    getStories,
    getStoriesByCondition,
    getStoriesByUserId,
    getStoriesByStoryIdNotWithUserId,
    getStoriesByStoryIdUserId,
    deleteStoriesByStoryIdUserId,
    makeViewIncrementWithColNameAndId,
    addStoryReaction,
    removeReaction
};