import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addStoryPayloadValidate = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Story Validator  -> Add New Story Validator")
    try {
        const schema = Joi.object().keys({
            mediaPath: Joi.any().allow('').description('mediaPath').error(new Error('mediaPath is missing')),
            caption: Joi.string().allow('').description('caption').error(new Error('caption is missing'))
        })
        req.body.mediaPath = req['files'];
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const storyByUserId = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Story Validator  -> Get List Stories Validator")
    try {
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const deleteStoryValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Story Validator  -> DELETE Stories Validator")
    try {
        const schema = Joi.object().keys({
            storyId: Joi.number().required().description('storyId: 1').error(new Error('storyId missing or Invalid')),
        });

        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const getStoriesWithOffsetLimit = (req: Request, res: Response, next: NextFunction) => {
    try {
        const schema = Joi.object().keys({
            limit: Joi.string().allow('').default('20').description('limit').optional(),
            offset: Joi.string().allow('').default('0').description('offset').optional()
        });
        const validation = schema.validate(req.query);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        console.log(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addStoryPayloadValidate,
    storyByUserId,
    deleteStoryValidator,
    getStoriesWithOffsetLimit
}
