import { NextFunction, Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import { Logging } from "../../commonServices/logging.service";
import getErrorMessage from "../../../utills/errorStructure";
import StoryPrivacyService from "./storyPrivacy.service";
import UserService from "../users/user.service";
import { CustomError } from "../../../utills/types";

const addNewStoryPrivacy = async (req: Request, res: Response) => {
    try {
        const user = req['user'];
        let userData = await UserService.getUserById(user.id);
        if (!userData) {
            return res.status(HTTPStatusCode.NOTFOUND).json({
                message: StaticMessages.getNotExistMessage('User'),
                status: Constants.ERROR
            });
        }
        const userPrivacyData = [];
        await StoryPrivacyService.deleteStoryPrivacyByUserId(user.id);

        const hideUserIds = req.body.userIds;
        hideUserIds.map((id) => {
            userPrivacyData.push({
                hideByUserId: user.id,
                hideForUserId: id
            })
        });

        if (userPrivacyData.length > 0) {
            await StoryPrivacyService.createStoryPrivacy(userPrivacyData);
        }

        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getCreatedMessage('Story Privacy')
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        return res.status(HTTPStatusCode.SUCCESS).json({
            error: getErrorMessage(error),
            status: Constants.ERROR,
            message: StaticMessages.STORY_CREATE_FAILED
        });
    }
};

const getStoryPrivacy = async (req: Request, res: Response) => {
    try {
        const user = req['user'];
        let userData = await UserService.getUserById(user.id);
        if (!userData) {
            throw new CustomError(StaticMessages.getNotExistMessage('User'), HTTPStatusCode.NOTFOUND);
        }
        const userPrivacy = await StoryPrivacyService.getStoryPrivacyByHideByUserId(user.id);
        if (Object.keys(userPrivacy).length == 0) {
            throw new CustomError(StaticMessages.getNotExistMessage('Story Privacy'), HTTPStatusCode.NOTFOUND);
        }
        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getListReturnedMessage('Story Privacy'),
            userPrivacyData: userPrivacy
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        return res.status(HTTPStatusCode.SUCCESS).json({
            error: getErrorMessage(error),
            status: Constants.ERROR,
            message: getErrorMessage(error).message
        });
    }
};

export default {
    addNewStoryPrivacy,
    getStoryPrivacy
}