import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addStoryPrivacyPayloadValidate = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Story Privacy Validator  -> Add Privacy");
    try {
        const schema = Joi.object().keys({
            userIds: Joi.array().items(Joi.number().optional()).empty(Joi.array().length(0)).required().description('userIds: integer ids of user').error(new Error('userIds is missing or incorrect or must be in array'))
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addStoryPrivacyPayloadValidate
}