import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import StoryViewsService from "./storyViews.service";
import StoryService from "../story/story.service";
import UserService from "../users/user.service";
import moment from "moment";

const addStoryViewsForStory = async (req: Request, res: Response) => {
    const authUser = req['user'];

    const getStoryData = () => new Promise<void>((resolve, reject) => {
        return StoryService.getStoriesByStoryIdNotWithUserId(req.body.storyId, authUser.id)
            .then((data) => {
                if (data.length > 0) resolve()
                reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('Stories') })
            }).catch((err) => reject(err))
    });
    const checkUniqueViews = () => new Promise<void>((resolve, reject) => {
        return StoryViewsService.getStoryViewsByStoryIdUserId(req.body.storyId, authUser.id)
            .then((data) => {
                if (data.length > 0) {
                    return res.status(HTTPStatusCode.SUCCESS).json({
                        status: Constants.SUCCESS,
                        message: StaticMessages.STORY_VIEW_SYNCED
                    });
                }
                resolve();
            }).catch((err) => reject(err))
    });
    const getUserInfo = () => new Promise((resolve, reject) => {
        return UserService.getUserById(authUser.id)
            .then((data) => {
                if (!data) {
                    reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('User') })
                }
                resolve(data);
            }).catch(err => reject(err));
    });
    const addStoryViews = (userData) => new Promise<void>((resolve, reject) => {
        const storyViewsInsertData = {
            viewedByUserId: parseInt(authUser.id),
            viewedByUserName: userData.userName,
            viewedByFirstName: userData.firstName,
            viewedByLastName: userData.lastName,
            profileThumbnail: userData.profilePicture || '',
            viewTimeStamp: moment().unix(),
            createdAt: new Date(),
            updatedAt: new Date(),
            StoryId: parseInt(req.body.storyId)
        }
        return StoryViewsService.createStoryViews(storyViewsInsertData)
            .then(() => resolve())
            .catch(err => reject(err));
    });
    const makeViewIncrement = () => new Promise<void>((resolve, reject) => {
        return StoryService.makeViewIncrementWithColNameAndId("totalViewsCount", parseInt(req.body.storyId))
            .then(() => resolve())
            .catch(err => reject(err));
    });
    return getStoryData()
        .then(checkUniqueViews)
        .then(getUserInfo)
        .then(addStoryViews)
        .then(makeViewIncrement)
        .then(() => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.STORY_VIEW_SYNCED
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        });
};

const getStoryViews = async (req: Request, res: Response) => {
    const authUser = req['user'];

    const getStoryData = () => new Promise<void>((resolve, reject) => {
        return StoryService.getStoriesByStoryIdUserId(String(req.query.storyId), authUser.id)
            .then((data) => {
                if (data.length > 0) resolve();
                reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('Stories') })
            }).catch((err) => reject(err));
    });
    const getStoryViewsData = () => new Promise((resolve, reject) => {
        return StoryViewsService.getStoryViewsByStoryId(String(req.query.storyId), parseInt(String(req.query.limit)), parseInt(String(req.query.offset)))
            .then((data) => {
                if (data.length > 0) {
                    resolve(data);
                }
                reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.STORY_VIEW_NOTFOUND })
            }).catch((err) => reject(err));
    });
    return getStoryData()
        .then(getStoryViewsData)
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.STORY_VIEW_SYNCED,
                storyViews: data
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        });
};

export default {
    addStoryViewsForStory,
    getStoryViews
}