import { Request, Response, NextFunction } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const storyViewPostValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Story View Validator  -> Stories Views POST Payload Validator")
    try {
        const schema = Joi.object().keys({
            storyId: Joi.number().required().description('storyId: 1').error(new Error('storyId missing or Invalid'))
        })
        req.body = Object.keys(req.body).length > 0 ? req.body : req['fields'];
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const storyViewGetValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Story View Validator  -> Stories Views GET Payload Validator")
    try {
        const schema = Joi.object().keys({
            storyId: Joi.number().required().description('storyId: 1').error(new Error('storyId missing or Invalid')),
            limit: Joi.string().allow('').default('20').description('limit').optional(),
            offset: Joi.string().allow('').default('0').description('offset').optional()
        })
        const validation = schema.validate(req.query);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    storyViewPostValidator,
    storyViewGetValidator
}