import { Request, Response, NextFunction } from "express";
import { Logging } from "../../commonServices/logging.service";
import { HTTPStatusCode, Users } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import getErrorMessage from "../../../utills/errorStructure";
import subscribedUserService from "./subscribedUser.service";
import { StaticMessages } from "../../../utills/staticMessage";
import UserService from "../users/user.service";

//Get the list of all users who buys subscription plan
const getSubscribedUserList = async (req: Request, res: Response, next: NextFunction) => {
    try {
        if (req.query.type) {
            let data = await subscribedUserService.getSubscribedUserList(req.query.type);
            res.status(HTTPStatusCode.CREATED).json({
                status: Constants.SUCCESS,
                data: data
            })
        } else {
            let data = await subscribedUserService.getSubscribedUserList(Users.ALLUSERS);
            res.status(HTTPStatusCode.CREATED).json({
                status: Constants.SUCCESS,
                data: data
            })
        }
    } catch (error) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(error.statusCode).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.NO_RECORD_FOUND
        });
    }
};

//Get subscribed user by ID
const getSubscribedUserById = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let data = await subscribedUserService.getSubscribedUserById(req.params.id);
        if (data) {
            res.status(HTTPStatusCode.CREATED).json({
                status: Constants.SUCCESS,
                data: data
            });
        }
    } catch (error) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(error.statusCode).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.NO_RECORD_FOUND
        });
    }
};

//Update user By ID
const updateSubscribedUser = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let data = await subscribedUserService.updateSubsUser(req.params.id, req.body);
        if (data) {
            res.status(HTTPStatusCode.CREATED).json({
                status: Constants.SUCCESS,
                data: data
            });
        }
    } catch (error) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(error.statusCode).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.NO_RECORD_FOUND
        });
    }
};

//Delete User By ID
const deleteSubscribedUser = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let data = await subscribedUserService.deleteSubsUser(req.params.id);
        if (data) {
            res.status(HTTPStatusCode.CREATED).json({
                status: Constants.SUCCESS,
                data: data
            });
        }
    } catch (error) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(error.statusCode).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.NO_RECORD_FOUND
        });
    }
};

export default {
    getSubscribedUserList,
    getSubscribedUserById,
    updateSubscribedUser,
    deleteSubscribedUser
};
