import db from "../../../models";
import { Users, UserType } from "../../../utills/enums";
import { Op } from "sequelize";
import moment from "moment";
const SubscribedUser = db.SubscribedUser;
const User = db.User;

const addSubscribedUser = async (data) => {
    try {
        return await SubscribedUser.create(data);
    } catch (error) {
        return error;
    }
};

const getSubscribedUserList = async (type) => {
    try {
        if (type == Users.ACTIVE) {
            let user = await User.findAll({
                where: {
                    isActive: 1
                }
            });
            return user;
        }
        if (type == Users.DISABLE) {
            let user = await User.findAll({
                where: {
                    isActive: 0
                }
            });
            return user;
        }
        if (type == Users.ALLUSERS) {
            let user = await User.findAll({
                attributes: [
                    ['id', 'userId'],
                    'id',
                    'firstName',
                    'lastName',
                    'userName',
                    'phone',
                    'email',
                    'profilePicture',
                    'profileThumbnail',
                    'isActive'
                ]
            });
            return user;
        }
        if (type == Users.SUBSCRIBEDUSERS) {
            let user = await SubscribedUser.findAll();
            return user;
        }
    } catch (error) {
        return error;
    }
};

const getSubscribedUserById = async (id: string) => {
    try {
        let user = await SubscribedUser.findAll({
            where: { id: id }
        });
        return user;
    } catch (error) {
        return error;
    }
};

const updateSubsUser = async (id: string, data: any) => {
    try {
        let user = await SubscribedUser.update(data, {
            where: { id: id }
        });
        return user;
    } catch (error) {
        return error;
    }
};

const deleteSubsUser = async (id: string) => {
    try {
        let user = await SubscribedUser.destroy({
            where: { id: id }
        });
        return user;
    } catch (error) {
        return error;
    }
};

const getUserListBySearch = async (userName, email, userType, isActive) => {
    try {
        let subscribedUser = await SubscribedUser.findAll({
            where: {
                expiresOn: {
                    [Op.gt]: moment().unix()
                }
            },
            raw: true
        });

        let sUserIds = subscribedUser.map((sUser) => sUser.UserId) //subscribed user

        let user = [];
        if (userName != "" && email != "") {
            const condition = {
                where: {
                    userType: UserType.USER,
                    [Op.and]: [
                        { userName: { [Op.regexp]: `.*${userName}.*` } },
                        { email: { [Op.regexp]: `.*${email}.*` } },
                    ]
                },
                attributes: [
                    ['id', 'userId'],
                    'id',
                    'firstName',
                    'lastName',
                    'userName',
                    'phone',
                    'email',
                    'profilePicture',
                    'profileThumbnail',
                    'isActive'
                ],
                raw: true
            }
            if (parseInt(userType) == Users.SUBSCRIBEDUSERS) {
                condition.where["id"] = {
                    [Op.in]: sUserIds
                }
            }
            if (isActive > -1) {
                condition.where['isActive'] = parseInt(isActive);
            }
            user = await User.findAll(condition);
        } else if (userName || email || userType || isActive) {
            const condition = {
                where: {
                    userType: UserType.USER
                },
                attributes: [
                    ['id', 'userId'],
                    'id',
                    'firstName',
                    'lastName',
                    'userName',
                    'phone',
                    'email',
                    'profilePicture',
                    'profileThumbnail',
                    'isActive'
                ],
                raw: true
            }
            if (userName != "") {
                condition.where["userName"] = { [Op.regexp]: `.*${userName}.*` }
            }
            if (email != "") {
                condition.where["email"] = { [Op.regexp]: `.*${email}.*` }
            }
            if (parseInt(userType) == Users.SUBSCRIBEDUSERS) {
                condition.where["id"] = {
                    [Op.in]: sUserIds
                }
            }
            if (isActive > -1) {
                condition.where['isActive'] = parseInt(isActive);
            }
            user = await User.findAll(condition);
        }
        return user;
    } catch (error: any) {
        throw error;
    }
};

const getActiveSubscriptionByUserId = async (userId: any) => {
    try {
        let user = await SubscribedUser.findAll({
            where: {
                expiresOn: {
                    [Op.gt]: moment().unix()
                },
                userId: userId
            },
            order: [['id', 'DESC']],
            raw: true
        });
        return user;
    } catch (error) {
        return error;
    }
};


export default {
    addSubscribedUser,
    getSubscribedUserList,
    getSubscribedUserById,
    updateSubsUser,
    deleteSubsUser,
    getUserListBySearch,
    getActiveSubscriptionByUserId
}
