import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const getSearchedUsers = (req: Request, res: Response, next: NextFunction) => {
    try {
        const schema = Joi.object({
            userName: Joi.string().allow('').optional(),
            email: Joi.string().allow('').optional(),
            userType: Joi.string().allow('').optional(),
            isActive: Joi.string().allow('').optional()
        });
        const validation = schema.validate(req.query);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.query = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    getSearchedUsers
}