import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const getSubscriptionList = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            limit: Joi.string().allow('').default('20').description('limit').optional(),
            offset: Joi.string().allow('').default('0').description('offset').optional()
        })
        const validation = schema.validate(req.query);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.query = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const planPurchaseValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("SubscriptionPlan Validator  -> Purchase Validator");
    try {
        const schema = Joi.object({
            subscriptionPlanId: Joi.number().description('subscriptionPlanId').required().error(new Error('subscriptionPlanId is missing or incoreect')),
            paymentProfileId: Joi.string().description('paymentProfileId').required().error(new Error('paymentProfileId is missing or incoreect')),
            cvv: Joi.string().description('paymentProfileId').required().error(new Error('paymentProfileId is missing or incoreect'))
        })
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const createUpdateSubscriptionPlan = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            requestHeader: Joi.object().optional().description('requestHeader').error(new Error('Request Header is missing')),
            planName: Joi.string().required().description('planName').error(new Error('planName is missing.')),
            planDetails: Joi.string().allow('').description('planDetails').required(),
            planValidityInDays: Joi.string().allow('').description('planValidityInDays').required(),
            price: Joi.string().allow('').description('price').required(),
            discount: Joi.string().allow('').description('discount').required()
        })
        
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    getSubscriptionList,
    planPurchaseValidator,
    createUpdateSubscriptionPlan
}