import db from "../../../models";
import sequelize, { Op } from "sequelize";

const UserCharacteristics = db.UserCharacteristics;

interface ansArray {
    userId: number,
    customAns: string,
    questionId: number,
    answer: number
};

const addCharacteristics = async (data: any) => {
    try {
        return await UserCharacteristics.create(data);
    } catch (err: any) {
        throw err;
    }
};

const addMultipleCharacteristicsAtOnce = async (data: Array<ansArray>) => {
    try {
        return await UserCharacteristics.bulkCreate(data);
    } catch (err: any) {
        throw err;
    }
};

const getCharacteristicsByUserId = async (userId: any) => {
    try {
        const condition = {
            where: {
                userId: String(userId)
            },
            include: [
                {
                    model: db.Question,
                    as: "Question",
                    attributes: [
                        "name",
                        "quesCategoryId"
                    ],
                    include: {
                        model: db.QuestionCategory,
                        as: "QuestionCategory",
                        attributes: ["name"],
                    }
                },
                {
                    model: db.Choice,
                    as: "Choice",
                    attributes: ["name"]
                },
            ],
            attributes: [
                ["id", "characteristicId"],
                "userId",
                "questionId",
                [sequelize.col('Question.name'), 'question'],
                [sequelize.col('Question.quesCategoryId'), 'quesCategoryId'],
                [sequelize.col('Question.QuestionCategory.name'), 'quesCategory'],
                ["answer", "answerId"],
                [sequelize.col('Choice.name'), 'answer'],
                "customAns"
            ],
            order: ['questionId'],
            raw: true
        }
        let characteristics = await UserCharacteristics.findAll(condition);
        const resp = [];
        for (let char of characteristics) {
            if (resp.filter((v) => v.questionId == char.questionId).length > 0) {
                resp[resp.findIndex((fArr) => fArr.questionId == char.questionId)].answers.push({
                    answerId: char.answerId,
                    answerText: char['Choice.name']
                })
            } else {
                resp.push({
                    userId: char.userId,
                    questionId: char.questionId,
                    question: char['Question.name'],
                    quesCategoryId: char['Question.quesCategoryId'],
                    quesCategory: char['Question.QuestionCategory.name'],
                    customAns: char.customAns,
                    answers: [
                        {
                            answerId: char.answerId,
                            answerText: char['Choice.name']
                        }
                    ]
                })
            }
        }
        return resp;
    } catch (err: any) {
        throw err
    }
};

const getCharacteristicsByUserIdQuestionId = async (userId: any, questionId: any) => {
    try {
        const condition = {
            where: {
                userId: String(userId),
                questionId: String(questionId)
            },
            include: [
                {
                    model: db.Question,
                    as: "Question",
                    attributes: [
                        "name",
                        "quesCategoryId"
                    ],
                    include: {
                        model: db.QuestionCategory,
                        as: "QuestionCategory",
                        attributes: ["name"],
                    }
                },
                {
                    model: db.Choice,
                    as: "Choice",
                    attributes: ["name"]
                },
            ],
            attributes: [
                ["id", "characteristicId"],
                "userId",
                "questionId",
                [sequelize.col('Question.name'), 'question'],
                [sequelize.col('Question.quesCategoryId'), 'quesCategoryId'],
                [sequelize.col('Question.QuestionCategory.name'), 'quesCategory'],
                ["answer", "answerId"],
                [sequelize.col('Choice.name'), 'answer'],
                "customAns"
            ],
            order: ['questionId'],
            raw: true
        }
        let characteristics = await UserCharacteristics.findAll(condition);
        const resp = [];
        for (let char of characteristics) {
            if (resp.filter((v) => v.questionId == char.questionId).length > 0) {
                resp[resp.findIndex((fArr) => fArr.questionId == char.questionId)].answers.push({
                    answerId: char.answerId,
                    answerText: char['Choice.name']
                })
            } else {
                resp.push({
                    userId: char.userId,
                    questionId: char.questionId,
                    question: char['Question.name'],
                    quesCategoryId: char['Question.quesCategoryId'],
                    quesCategory: char['Question.QuestionCategory.name'],
                    customAns: char.customAns,
                    answers: [
                        {
                            answerId: char.answerId,
                            answerText: char['Choice.name']
                        }
                    ]
                })
            }
        }
        return resp;
    } catch (err: any) {
        throw err
    }
};

const getHomeEducationWorkDetailsOfUser = async (userId: any, questionIds: any) => {
    try {
        const condition = {
            where: {
                userId: String(userId),
                questionId: questionIds
            },
            include: [
                {
                    model: db.Question,
                    as: "Question",
                    attributes: [
                        "name",
                        "quesCategoryId"
                    ],
                    include: {
                        model: db.QuestionCategory,
                        as: "QuestionCategory",
                        attributes: ["name"],
                    }
                },
                {
                    model: db.Choice,
                    as: "Choice",
                    attributes: ["name"]
                },
            ],
            attributes: [
                ["id", "characteristicId"],
                "userId",
                "questionId",
                [sequelize.col('Question.name'), 'question'],
                [sequelize.col('Question.quesCategoryId'), 'quesCategoryId'],
                [sequelize.col('Question.QuestionCategory.name'), 'quesCategory'],
                ["answer", "answerId"],
                [sequelize.col('Choice.name'), 'answer'],
                "customAns"
            ],
            order: [['questionId','DESC']],
            raw: true
        }
        let characteristics = await UserCharacteristics.findAll(condition);
        const resp = [];
        for (let char of characteristics) {
            if (resp.filter((v) => v.questionId == char.questionId).length > 0) {
                resp[resp.findIndex((fArr) => fArr.questionId == char.questionId)].answers.push({
                    answerId: char.answerId,
                    answerText: char['Choice.name']
                })
            } else {
                resp.push({
                    userId: char.userId,
                    questionId: char.questionId,
                    question: char['Question.name'],
                    quesCategoryId: char['Question.quesCategoryId'],
                    quesCategory: char['Question.QuestionCategory.name'],
                    customAns: char.customAns,
                    answers: [
                        {
                            answerId: char.answerId,
                            answerText: char['Choice.name']
                        }
                    ]
                })
            }
        }
        return resp;
    } catch (err: any) {
        throw err
    }
};

const getCharacteristicsByUserIdInShort = async (userId: any) => {
    try {
        return await UserCharacteristics.findAll({
            where: {
                userId: String(userId)
            }
        });
    } catch (err: any) {
        throw err
    }
};

const getPreferencedUsersByUserQnA = async (userId: any, questionIds: any, answers: any, ignoreUserIds: any) => {
    try {
        return await UserCharacteristics.findAll({
            include: [
                {
                    model: db.User,
                    as: "User",
                    where: {
                        isActive: 1,
                        isDocumentVerified: 3
                    },
                    attributes: []
                },
                {
                    model: db.Question,
                    as: "Question",
                    where: {
                        isSearchable: 1
                    },
                    attributes: []
                }
            ],
            where: {
                userId: {
                    [Op.notIn]: [userId, ...ignoreUserIds]
                },
                questionId: {
                    [Op.in]: questionIds
                },
                answer: {
                    [Op.in]: answers
                }
            },
            attributes: [
                "userId"
            ],
            group: ["userId"],
            raw: true
        });
    } catch (err: any) {
        throw err
    }
};

const deleteCharacteristicsByCondition = async (condition: any) => {
    try {
        return await UserCharacteristics.destroy({
            where: condition
        });
    } catch (err: any) {
        throw err
    }
};

export default {
    addCharacteristics,
    addMultipleCharacteristicsAtOnce,
    getCharacteristicsByUserId,
    getCharacteristicsByUserIdQuestionId,
    getHomeEducationWorkDetailsOfUser,
    getCharacteristicsByUserIdInShort,
    getPreferencedUsersByUserQnA,
    deleteCharacteristicsByCondition
}