import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import UserHearAboutUsFromService from "./userHearAboutUsFrom.service";
import UserService from "../users/user.service";
import HearAboutUsFromService from "../hearAboutUsFrom/hearAboutUsFrom.service";

const addNewHearLocation = (req: Request, res: Response) => {
    const authUser = req['user'];
    const checkUserIsExistOrNot = () => new Promise<void>((resolve, reject) => {
        return UserService.getUserById(String(authUser.id))
            .then((data) => {
                if (!data) {
                    reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('User') })
                }
                resolve();
            }).catch(err => reject(err))
    });

    const checkAboutUsChoiceExistOrNot = () => new Promise((resolve, reject) => {
        return HearAboutUsFromService.getHearLocationById(req.body.hearAboutUsFromId)
            .then((data) => {
                if (!data) {
                    reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('Hear About Us Choice') })
                }
                resolve(data);
            }).catch(err => reject(err))
    });

    const addHearing = () => new Promise((resolve, reject) => {
        return UserHearAboutUsFromService.addHearLocation({
            userId: parseInt(authUser.id),
            hearAboutUsFromId: parseInt(req.body.hearAboutUsFromId)
        })
            .then((data) => resolve(data))
            .catch(err => reject(err))
    });

    return checkUserIsExistOrNot()
        .then(checkAboutUsChoiceExistOrNot)
        .then(addHearing)
        .then(() => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getCreatedMessage('Hear About Us')
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

const getMyHearLocation = (req: Request, res: Response) => {
    const authUser = req['user'];
    const checkUserIsExistOrNot = () => new Promise<void>((resolve, reject) => {
        return UserService.getUserById(String(authUser.id))
            .then((data) => {
                if (!data) {
                    reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('User') });
                }
                resolve();
            }).catch(err => reject(err))
    });

    const getHearingLocations = () => new Promise((resolve, reject) => {
        return UserHearAboutUsFromService.getHearLocationByUserId(authUser.id)
            .then((data) => {
                if (data) {
                        delete data['HearAboutUsFrom.sourceName'];
                    resolve(data);
                }
                reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('User Hear About Us') });
            })
            .catch(err => reject(err))
    });

    return checkUserIsExistOrNot()
        .then(getHearingLocations)
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Hear About us'),
                userHearAboutUsFrom: data
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

export default {
    addNewHearLocation,
    getMyHearLocation
}