import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addUserHearAboutUsFromValidation = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Hear AboutUs Validator  -> Add New Hear About Us Validator")
    try {
        const schema = Joi.object().keys({
            hearAboutUsFromId: Joi.number().required().description('hearAboutUsFromId: integer id from hearAboutUsFromId API').error(new Error('hearAboutUsFromId is missing or incoreect'))
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addUserHearAboutUsFromValidation
}