import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import UserOnlineLogsService from "./userOnlineLogs.service";
import getErrorObject from "../../../utills/errorStructure";
import { CustomError } from "../../../utills/types";

const addOnlineLog = async (req: Request, res: Response) => {
    try {
        await UserOnlineLogsService.addOnlineLogsByUserId(req['user'].id)

        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getUpdatedMessage('Log')
        });
    } catch (err) {
        if (err instanceof Error) {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: StaticMessages.INTERNAL_SERVER_ERROR
            });
        } else {
            err = getErrorObject(err)
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: err.message
            });
        }
    }
};

export default {
    addOnlineLog
}