import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addPreferencePayloadValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Story Validator  -> Add New Story Validator")
    try {
        const schema = Joi.object().keys({
            questionId: Joi.number().required().description('questionId: integer id from Question API').error(new Error('questionId is missing or incoreect')),
            answers : Joi.array().items(Joi.number()).required().description('answers: integer id from choice API').error(new Error('answers is missing or incorrect or must be in array'))
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addPreferencePayloadValidator
}