import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import UserPromptsService from "./userPrompts.service";
import UserService from "../users/user.service";
import PromptsQuestionsService from "../promptsQuestions/promptsQuestions.service";
import getErrorObject from "../../../utills/errorStructure";
import { CustomError } from "../../../utills/types";

const addNewUserPrompts = (req: Request, res: Response) => {
    const authUser = req['user'];
    const checkUserIsExistOrNot = () => new Promise<void>((resolve, reject) => {
        return UserService.getUserById(String(authUser.id))
            .then((data) => {
                if (!data) {
                    reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('User') })
                }
                resolve();
            }).catch(err => reject(err))
    });

    const checkQuestionExistOrNot = () => new Promise(async (resolve, reject) => {
        const promptsArray = req.body.prompts;
        const promptsArrayMap = await promptsArray.map(async (prompt) => {
            let promptsData = await PromptsQuestionsService.getPromptsQuestionByQuestionId(prompt.promptsQuestionId);
            if (!promptsData) throw new CustomError(StaticMessages.getNotExistMessage('Question'), HTTPStatusCode.NOTFOUND);
            return prompt;
        });

        return Promise.all(promptsArrayMap)
            .then(() => resolve(promptsArray))
            .catch(err => reject(err))
    });

    const removeAllOldPromptsWithSameQuestionId = (promptsArray) => new Promise<void>((resolve, reject) => {
        return UserPromptsService.deleteUserPromptsByCondition({
            userId: authUser.id
        })
            .then(() => resolve(promptsArray))
            .catch(err => reject(err))
    });

    const addUserPrompt = (promptsArray) => new Promise((resolve, reject) => {
        const insertDataArr = [];
        promptsArray.map((prompt) => {
            insertDataArr.push({
                userId: parseInt(authUser.id),
                promptsQuestionId: parseInt(prompt.promptsQuestionId),
                answer: prompt.answer
            })
        })
        return UserPromptsService.addMultiplePromptsAtOnce(insertDataArr)
            .then((data) => resolve(data))
            .catch(err => reject(err))
    });

    return checkUserIsExistOrNot()
        .then(checkQuestionExistOrNot)
        .then(removeAllOldPromptsWithSameQuestionId)
        .then(addUserPrompt)
        .then(() => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getCreatedMessage('User Prompt')
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                err = getErrorObject(err)
                return res.status(HTTPStatusCode.SUCCESS).json({
                    error: err,
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

const getMyPrompts = (req: Request, res: Response) => {
    const authUser = req['user'];
    const checkUserIsExistOrNot = () => new Promise<void>((resolve, reject) => {
        return UserService.getUserById(String(authUser.id))
            .then((data) => {
                if (!data) {
                    reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('User') });
                }
                resolve();
            }).catch(err => reject(err))
    });

    const getPrompts = () => new Promise((resolve, reject) => {
        return UserPromptsService.getUserPromptsByUserId(authUser.id)
            .then((data) => {
                if (data.length > 0) {
                    resolve(data);
                }
                reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('User Prompt') });
            })
            .catch(err => reject(err))
    });

    return checkUserIsExistOrNot()
        .then(getPrompts)
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('User Prompt'),
                userPrompts: data
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

export default {
    addNewUserPrompts,
    getMyPrompts
}