import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const addUserPromptsPayloadValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("User Prompts Validator  -> Add New User Prompts Validator")
    try {
        const schema = Joi.object().keys({
            prompts: Joi.array().items(Joi.object({
                promptsQuestionId: Joi.number().required().description('promptsQuestionId: integer id from Question API').error(new Error('promptsQuestionId is missing or incoreect')),
                answer: Joi.string().allow('').default('').optional()
            }))
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addUserPromptsPayloadValidator
}