import { Constants } from "../../../utills/constants";
import { Utils } from "../../../utills/helperMethods";
import { sendMail } from "../../../utills/mailer";
import { IUser } from "../../../models/user";
import db from "../../../models";
import { FolderNames, HTTPStatusCode, MediaSources, UserType } from "../../../utills/enums";
import { StaticMessages } from "../../../utills/staticMessage";
import { jwtSign } from "../../../utills/token";
import { CustomError } from "../../../utills/types";
import { Users } from "../../../utills/enums";
import { handleFileUpload, handleImageVideoThumb } from "../../commonServices/fsFileUpload";
import { Op } from "sequelize";
import sequelize from 'sequelize';
import moment from "moment";
import FollowingFollowerService from "../followingFollower/followingFollower.service";
const User = db.User;
const OTP = db.Otp;
const AgeRanges = db.AgeRanges;
const BlockUser = db.BlockUser;
const DatePageAction = db.DatePageAction;
const DatingPageMedia = db.DatingPageMedia;
const DistanceMatch = db.DistanceMatch;
const DocumentVerificationLogs = db.DocumentVerificationLogs;
const EmergencyContact = db.EmergencyContact;
const Follower = db.Follower;
const NotificationSettings = db.NotificationSettings;
const PaymentGatewayLogs = db.PaymentGatewayLogs;
const ReportedUser = db.ReportedUser;
const SignUpCheckList = db.SignUpCheckList;
const StoryPrivacy = db.StoryPrivacy;
const SubscribedUser = db.SubscribedUser;
const IdenfyLogs = db.IdenfyLogs;
const SpotifyPlaylists = db.SpotifyPlaylists;
const SpotifyTracks = db.SpotifyTracks;

const createUser = async (data: IUser) => {
    try {
        let res = await User.create(data);
        const otp = Utils.randomDigits(4);
        await OTP.create({
            signUpOTP: otp,
            resetPasswordOTP: null,
            email: data.email
        });
        await sendMail(
            Constants.FROM_EMAIL,
            data.email,
            StaticMessages.CONFIRM_ACCOUNT,
            Utils.getOptMailHtml(otp));
        return res;
    } catch (err: any) {
        throw err
    }
};

const deleteUser = async (id: number) => {
    try {
        await AgeRanges.destroy({
            where: {
                userId: id
            }
        });
        await BlockUser.destroy({
            where: {
                [Op.or] :[
                    { blockByUserId: id },{ blockForUserId: id }
                ]
            }
        });
        await DatePageAction.destroy({
            where: {
                [Op.or] : [
                    { actionUserId: id }, { userId: id }
                ]
            }
        });
        await Follower.destroy({
            where: {
                [Op.or] : [
                    { toUser: id }, { fromUser: id }
                ]
            }
        });
        await DatingPageMedia.destroy({
            where: {
                userId: id
            }
        });
        await DistanceMatch.destroy({
            where: {
                userId: id
            }
        });
        await DocumentVerificationLogs.destroy({
            where: {
                userId: id
            }
        });
        await EmergencyContact.destroy({
            where: {
                userId: id
            }
        });
        await NotificationSettings.destroy({
            where: {
                userId: id
            }
        });
        await PaymentGatewayLogs.destroy({
            where: {
                userId: id
            }
        });
        await ReportedUser.destroy({
            where: {
                [Op.or] : [
                    { userId: id }, { toUserId: id }
                ]
            }
        });
        await SignUpCheckList.destroy({
            where: {
                userId: id
            }
        });
        await StoryPrivacy.destroy({
            where: {
                [Op.or] : [
                    { hideByUserId: id }, { hideForUserId: id }
                ]
            }
        });
        await IdenfyLogs.destroy({
            where: {
                userId: id
            }
        });
        const userDeleted = await User.destroy({
            where: {
                id
            }
        });
        return userDeleted;
    } catch (err: any) {
        throw err
    }
};

const selectAllUserIds = async () => {
    try {
        const allIds = [];
        let res = await User.findAll({
            where: {
                isActive: 1,
                isDocumentVerified: 3
            },
            attributes: [
                "id"
            ]
            , raw: true
        });
        res.map((resUser) => allIds.push(resUser.id));
        return allIds;
    } catch (err: any) {
        throw err
    }
}

const selectRandomUserIds = async (ignoreUserIds: any, limit: number) => {
    try {
        const res = await User.findAll({
            where: {
                isActive: 1,
                isDocumentVerified: 3,
                id: {
                    [Op.notIn]: ignoreUserIds
                }
            },
            limit,
            order: sequelize.literal('RAND()'),
            attributes: [
                "id",
                "userName",
                "firstName",
                "lastName",
                "profilePicture",
                "DOB",
                "gender",
                "height",
                "heightVerified",
                "instagramToken",
                "tiktokToken",
                "spotifyToken",
                "appleMusicToken"
            ]
        });
        return res;
    } catch (err: any) {
        throw err
    }
}

const updatePassword = async (data: any, id: number) => {
    try {
        let where = null;
        where = { id: id};
        let res = await User.update({
            password:Utils.encryptedAES(data.password)
        }, {
            where: where
        });
        return res;
    } catch (err: any) {
        throw err
    }
};

const getUserById = async (id: String, attrs: string[] = []) => {
    try {
        if (attrs.length === 0) {
            attrs = [
                "id",
                "firstName",
                "lastName",
                "userName",
                "userType",
                "email",
                "height",
                "heightVerified",
                "DOB",
                "gender",
                "countryCode",
                "phone",
                "isVerified",
                "isDocumentVerified",
                "isVideoVerified",
                "profilePicture",
                "profileThumbnail",
                "defaultNotification",
                "profileBio",
                "emailNotification",
                "datePageViewCount",
                "smsNotification",
                "isLastSeenEnabled",
                "themeColor",
                "instagramToken",
                "spotifyToken",
                "tiktokToken",
                "appleMusicToken"
            ]
        }

        return await User.findByPk(id, { 
            attributes: attrs,
            include: [{
                model: db.SpotifyPlaylists,
                as: 'SpotifyPlaylists',
                include: [{
                    model: db.SpotifyTracks,
                    as: 'SpotifyTracks'
                }]
            }]
        });
    } catch (error: any) {
        throw error;
    }
};

const getUserDetails = async (id: String, attrs: string[] = []) => {
    try {
        if (attrs.length === 0) {
            attrs = [
                "id",
                "firstName",
                "lastName",
                "userName",
                "userType",
                "email",
                "height",
                "heightVerified",
                "DOB",
                "gender",
                "countryCode",
                "phone",
                "isVerified",
                "isDocumentVerified",
                "isVideoVerified",
                "profilePicture",
                "profileThumbnail",
                "defaultNotification",
                "profileBio",
                "emailNotification",
                "datePageViewCount",
                "smsNotification",
                "isLastSeenEnabled",
                "themeColor",
                "instagramToken",
                "spotifyToken",
                "tiktokToken",
                "appleMusicToken"
            ]
        }

        return await User.findByPk(id, { 
            attributes: attrs
        });
    } catch (error: any) {
        throw error;
    }
};

const getUserList = async (type: any, limit = 20, offset = 0) => {
    try {
        if (type == Users.ACTIVE) {
            let activeUsers = await User.findAll({
                where: {
                    "isActive": 1,
                    userType: UserType.USER
                },
                attributes: [
                    ['id', 'userId'],
                    'firstName',
                    'lastName',
                    'userName',
                    'profilePicture',
                    'profileThumbnail'
                ],
                offset: offset,
                limit: limit,
                raw: true
            });

            activeUsers.map((user) => {
                user.profileThumbnail = user.profileThumbnail ? '/static/users/' + user.userId + '/' + FolderNames.PROFILE_PICTURE + '/' + user.profileThumbnail : "";
                user.profilePicture = user.profilePicture ? '/static/users/' + user.userId + '/' + FolderNames.PROFILE_PICTURE + '/' + user.profilePicture : "";
            });
            return activeUsers;
        };
        if (type == Users.DISABLE) {
            let disableUsers = await User.findAll({
                where: {
                    "isActive": 0,
                    userType: UserType.USER
                },
                offset: offset,
                limit: limit,
                raw: true
            });

            disableUsers.map((user) => {
                user.profileThumbnail = user.profileThumbnail ? '/static/users/' + user.id + '/' + FolderNames.PROFILE_PICTURE + '/' + user.profileThumbnail : "";
                user.profilePicture = user.profilePicture ? '/static/users/' + user.id + '/' + FolderNames.PROFILE_PICTURE + '/' + user.profilePicture : "";
            });
            return disableUsers;
        };
        if (type == Users.ALLUSERS) {
            let allUsers = await User.findAll({
                where: { userType: UserType.USER },
                offset: offset,
                limit: limit,
                raw: true
            });

            allUsers.map((user) => {
                user.profileThumbnail = user.profileThumbnail ? '/static/users/' + user.id + '/' + FolderNames.PROFILE_PICTURE + '/' + user.profileThumbnail : "";
                user.profilePicture = user.profilePicture ? '/static/users/' + user.id + '/' + FolderNames.PROFILE_PICTURE + '/' + user.profilePicture : "";
            });
            return allUsers;
        };
        if (type == Users.SUBSCRIBEDUSERS) {
            let subscribedUsers = await SubscribedUser.findAll({
                offset: offset,
                limit: limit,
                raw: true
            });
            return subscribedUsers
        }
    } catch (error: any) {
        throw error;
    }
};

const getUserListBySearch = async (userId: string, search: string, limit = 20, offset = 0) => {
    try {
        const userFollowings = await FollowingFollowerService.getUserFollowingIds(userId);
        const userList = await User.findAll({
            where: {
                [Op.or]: [
                    { firstName: { [Op.regexp]: `.*${search}.*` } },
                    { lastName: { [Op.regexp]: `.*${search}.*` } },
                    { userName: { [Op.regexp]: `.*${search}.*` } },
                ]
            },
            attributes: [
                ['id', 'userId'],
                'id',
                'firstName',
                'lastName',
                'userName',
                'profilePicture',
                'profileThumbnail'
            ],
            offset: offset,
            limit: limit,
            raw: true
        });

        userList.map((user) => {
            user.profileThumbnail = user.profileThumbnail ? '/static/users/' + user.userId + '/' + FolderNames.PROFILE_PICTURE + '/' + user.profileThumbnail : "";
            user.profilePicture = user.profilePicture ? '/static/users/' + user.userId + '/' + FolderNames.PROFILE_PICTURE + '/' + user.profilePicture : "";
            user.isFriend = userFollowings.includes(user.userId);
        });

        return userList;
    } catch (error: any) {
        throw error;
    }
};

const userLogin = async (userName: string, password: string, email: string) => {
    try {
        if (!userName && !email) {
            throw new CustomError(StaticMessages.EMAIL_USERNAME_NOT_FOUND, HTTPStatusCode.BAD_REQUEST);
        }
        if (!password) {
            throw new CustomError(StaticMessages.PASSWORD_MISSING, HTTPStatusCode.BAD_REQUEST);
        }
        let WHERE = userName ? { userName: userName, password: password } : { email: email, password: password };
        const user = await User.findOne({ where: WHERE });
        if (user) {
            if (user.isVerified) {
                return jwtSign({ name: user.userName, email: user.email, id: user.id, userType: user.userType, issueTimeStamp: moment().unix() });
            } else {
                throw new CustomError('Email is Not Verified.', HTTPStatusCode.BAD_REQUEST);
            }
        } else {
            throw new CustomError(StaticMessages.INVALID_CREDS, HTTPStatusCode.UNAUTHORIZED);
        }
    } catch (error: any) {
        throw error;
    }
};

const getUserByEmail = async (email: string) => {
    try {
        return await User.findOne({
            where: {
                email: email
            },
            raw: true
        });
    } catch (error: any) {
        throw error;
    }
};

const getVerifiedUserByEmail = async (email: string) => {
    try {
        return await User.findOne({
            where: {
                email: email,
                isVerified: 1
            },
            raw: true
        });
    } catch (error: any) {
        throw error;
    }
};

const forgotPassword = async (email: string) => {
    try {
        let { count, rows } = await OTP.findAndCountAll({ where: { email: email } });
        if (count) {
            const otp = Utils.randomDigits(4);
            await sendMail(Constants.FROM_EMAIL, email, "Reset Password datesafe Account", Utils.getOptMailHtml(otp));
            await OTP.update({ resetPasswordOTP: otp, }, { where: { email: email } });
            return true;
        } else {
            throw new CustomError(StaticMessages.EMAIL_NOT_FOUND, HTTPStatusCode.UNAUTHORIZED);
        }
    } catch (error: any) {
        throw error;
    }
};

const updateUser = async (id: string, data: any) => {
    try {
        const updateData = {};

        if (data.firstName !== '' && typeof data.firstName !== 'undefined') {
            updateData['firstName'] = data.firstName;
        }

        if (data.lastName !== '' && typeof data.lastName !== 'undefined') {
            updateData['lastName'] = data.lastName;
        }

        if (data.gender !== '' && typeof data.gender !== 'undefined') {
            updateData['gender'] = data.gender;
        }

        if (data.heightUnit !== '' && typeof data.heightUnit !== 'undefined') {
            updateData['heightUnit'] = data.heightUnit;
        } 

        if (data.distanceUnit !== '' && typeof data.distanceUnit !== 'undefined') {
            updateData['distanceUnit'] = data.distanceUnit;
        } 

        if (data.height !== '' && typeof data.height !== 'undefined') {
            updateData['height'] = data.height;
        } 

        if (data.heightVerified !== '' && typeof data.heightVerified !== 'undefined') {
            updateData['heightVerified'] = data.heightVerified;
        } 

        if (data.userName !== '' && typeof data.userName !== 'undefined') {
            updateData['userName'] = data.userName;
        }

        if (data.email !== '' && typeof data.email !== 'undefined') {
            updateData['email'] = data.email;
        }

        if (data.DOB !== '' && typeof data.DOB !== 'undefined') {
            updateData['DOB'] = data.DOB;
        }

        if (data.pgProfileId !== '' && typeof data.pgProfileId !== 'undefined') {
            updateData['pgProfileId'] = data.pgProfileId;
        }

        if ((data.phone !== '' && typeof data.phone !== 'undefined') && (data.countryCode !== '' && typeof data.countryCode !== 'undefined')) {
            updateData['countryCode'] = data.countryCode;
            updateData['phone'] = data.phone;
        }

        if (data.profileBio !== '' && typeof data.profileBio !== 'undefined') {
            updateData['profileBio'] = data.profileBio;
        }

        if (data.defaultNotification !== '' && typeof data.defaultNotification !== 'undefined') {
            updateData['defaultNotification'] = data.defaultNotification;
        }

        if (data.emailNotification !== '' && typeof data.emailNotification !== 'undefined') {
            updateData['emailNotification'] = data.emailNotification;
        }

        if (data.smsNotification !== '' && typeof data.smsNotification !== 'undefined') {
            updateData['smsNotification'] = data.smsNotification;
        }

        if (data.fcmToken !== '' && typeof data.fcmToken !== 'undefined') {
            updateData['fcmToken'] = data.fcmToken;
        }

        if (data.alertSystemConsent !== '' && typeof data.alertSystemConsent !== 'undefined') {
            updateData['alertSystemConsent'] = data.alertSystemConsent;
        }

        if (data.isLastSeenEnabled !== '' && typeof data.isLastSeenEnabled !== 'undefined') {
            updateData['isLastSeenEnabled'] = data.isLastSeenEnabled;
        }

        if (data.themeColor !== '' && typeof data.themeColor !== 'undefined') {
            updateData['themeColor'] = data.themeColor;
        }

        if (data.isDocumentVerified !== '' && typeof data.isDocumentVerified !== 'undefined') {
            updateData['isDocumentVerified'] = data.isDocumentVerified;
        }

        if (data.isVideoVerified !== '' && typeof data.isVideoVerified !== 'undefined') {
            updateData['isVideoVerified'] = data.isVideoVerified;
        }

        if (data['files'] && data['files'].length > 0) {
            let filePaths: any = await handleFileUpload({
                "type": MediaSources.PROFILE_PICTURE,
                "files": data.files,
                "itemId": null,
                "userId": id
            });
            delete data['files'];
            const thumbnailsPath = await handleImageVideoThumb({
                type: MediaSources.PROFILE_PICTURE,
                "files": filePaths,
                "itemId": null,
                "userId": id
            });
            updateData['profileThumbnail'] = thumbnailsPath[0];
            updateData['profilePicture'] = filePaths[0];
        }

        return await User.update(updateData, {
            where: { id: id }
        });
    } catch (err: any) {
        throw err;
    }
};

const setIsOnDate = async (id: string, data: any) => {
    try {
        return await User.update({
            isOnDate: data.isOnDate,
            onDateLong: data.longitude,
            onDateLat: data.latitude,
            onDateTimeStamp: moment().unix()
        }, {
            where: { id: id }
        }
        );
    } catch (err: any) {
        throw err;
    }
};

const resetPassword = async (email: string, password: string) => {
    try {
        let { count, rows } = await User.findAndCountAll({ where: { email: email } });
        if (await exists({
            "email": email
        })) {
            await User.update({ password }, { where: { email: email } });
            return true;
        } else {
            throw new CustomError(StaticMessages.EMAIL_NOT_FOUND, HTTPStatusCode.UNAUTHORIZED);
        }
    } catch (error: any) {
        throw error;
    }
}

const exists = async (where: any) => {
    try {
        let user = await User.findOne({ where: where });
        return user !== null;
    } catch (error: any) {
        throw error;
    }
};

const updateDocumentVerification = async (id: string) => {
    try {
        return await User.update({ isDocumentVerified: 1 }, {
            where: { id: id }
        });
    } catch (err: any) {
        throw err;
    }
};

const enableDisableUser = async (id: string, isActive: string) => {
    try {
        return await User.update({ isActive: isActive }, {
            where: { id: id }
        });
    } catch (err: any) {
        throw err;
    }
};

const getCountOfAllUsers = async () => {
    try {
        return await User.count({
            where: {
                userType: UserType.USER
            }
        });
    } catch (err: any) {
        throw err;
    }
};

const getCountOfAllUserTypes = async () => {
    try {
        let allUsers = await User.count({
            where: {
                userType: UserType.USER
            }
        });
        let subscribedUsers = await SubscribedUser.count();
        let activeUsers = await User.count({
            where: {
                userType: UserType.USER,
                isActive: Users.ACTIVE //1
            }
        })
        let disableUsers = await User.count({
            where: {
                userType: UserType.USER,
                isActive: Users.INACTIVE //0
            }
        })
        return { allUsers, subscribedUsers, activeUsers, disableUsers }
    } catch (err: any) {
        throw err;
    }
};

const updateProfilePicture = async (userId: string, profilePicture: string) => {
    try {
        return await User.update({ profilePicture: profilePicture, profileThumbnail: profilePicture }, {
            where: { id: userId }
        });
    } catch (err: any) {
        throw err;
    }
};
const deleteProfilePicture = async (userId: string) => {
    try {
        return await User.update({ profilePicture: null, profileThumbnail: null }, {
            where: { id: userId }
        });
    } catch (err: any) {
        throw err;
    }
};

const incrementDatePageViewCount = async (id: any) => {
    try {
        return await User.increment({ datePageViewCount: 1 }, {
            where: {
                id: parseInt(id)
            }
        });
    } catch (err: any) {
        throw err
    }
};

const addSocialMediaDetails = async (id: any, data: any) => {
    try {
        let detailsToUpdate: any = {}
        if(data.instagramToken || data.instagramToken === '' ) {
            detailsToUpdate['instagramToken'] = data.instagramToken;
        }
        if(data.tiktokToken || data.tiktokToken === '' ) {
            detailsToUpdate['tiktokToken'] = data.tiktokToken;
        }
        if(data.spotifyToken || data.spotifyToken === '') {
            detailsToUpdate['spotifyToken'] = data.spotifyToken;
        }
        if(data.appleMusicToken || data.appleMusicToken === '') {
            detailsToUpdate['appleMusicToken'] = data.appleMusicToken;
        }
        const user = await User.update(
            detailsToUpdate, {
            where: {
                id: id
            }
        });
        return user;
    } catch (err) {
        throw err;
    }
}

const addSpotifyTrack = async (id: any, playlistId: any, data: any) => {
    try {
        for(const track of data) {
            await SpotifyTracks.create({
                playlistId: playlistId,
                name: track.name,
                image: track.image,
                url: track.url,
                uri: track.uri
            });
        }
        return true;
    } catch (err) {
        throw err;
    }
}

const addSpotifyPlaylist = async (id: any, data: any) => {
    try {
        await SpotifyPlaylists.destroy({
            where: {
                userId: id
            }
        });
        const createdPlaylist = await SpotifyPlaylists.create({
            userId: id,
            name: data.name,
            image: data.image
        });
        for(const track of data.tracks) {
            await SpotifyTracks.create({
                playlistId: createdPlaylist.id,
                name: track.name,
                image: track.image,
                url: track.url,
                uri: track.uri
            });
        }
        return true;
    } catch (err) {
        throw err;
    }
}

const deleteSpotifyPlaylist = async (id: any) => {
    try {
        await SpotifyPlaylists.destroy({
            where: {
                userId: id
            }
        });
        return true;
    } catch (err) {
        throw err;
    }
}

export default {
    createUser,
    getUserById,
    getUserList,
    getUserListBySearch,
    userLogin,
    getUserByEmail,
    getVerifiedUserByEmail,
    forgotPassword,
    updateUser,
    updatePassword,
    resetPassword,
    exists,
    setIsOnDate,
    updateDocumentVerification,
    enableDisableUser,
    getCountOfAllUsers,
    getCountOfAllUserTypes,
    deleteProfilePicture,
    updateProfilePicture,
    selectAllUserIds,
    incrementDatePageViewCount,
    deleteUser,
    addSocialMediaDetails,
    addSpotifyPlaylist,
    addSpotifyTrack,
    getUserDetails,
    deleteSpotifyPlaylist,
    selectRandomUserIds
};
