import { Request, Response, NextFunction } from "express";
import UserService from "./user.service";
import { CommonAnswers, FolderNames, FollowStatus, FollowStatusText, HTTPStatusCode, MediaSources, Questions, UserLastStatus, Users } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import { Logging } from "../../commonServices/logging.service";
import getErrorObject from "../../../utills/errorStructure";
import LocationService from "../locations/location.service";
import FollowingFollowerService from "../followingFollower/followingFollower.service";
import DatingPageMediaService from "../datingPageMedia/datingPageMedia.service";
import UserPromptsService from "../userPrompts/userPrompts.service";
import { handleFileDelete } from "../../commonServices/fsFileUpload";
import moment from "moment";
import UserCharacteristicService from "../userCharacteristic/userCharacteristic.service";
import SubscribedUserService from "../subscribedUser/subscribedUser.service";
import UserOnlineLogsService from "../userOnlineLogs/userOnlineLogs.service";

const getUserById = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let authUser = req['user'];
        let user = await UserService.getUserById(req.params.id);
        const finalResp = {
            userId: req.params.id,
            userBasicInfo: {},
            datingPageMedia: [],
            prompts: [],
            interests: [],
            socialMedia: [],
            location: {}
        }
        if (user) {
            const locationInfo = await LocationService.getLocationByUserId(req.params.id, ["longitude", "latitude", "address"]);
            finalResp.userBasicInfo['firstName'] = user['firstName'];
            finalResp.userBasicInfo['lastName'] = user['lastName'];
            finalResp.userBasicInfo['userName'] = user['userName'];
            finalResp.userBasicInfo['email'] = user['email'];
            finalResp.userBasicInfo['DOB'] = user['DOB'];
            finalResp.userBasicInfo['gender'] = user['gender'];
            finalResp.userBasicInfo['height'] = user['height'];
            finalResp.userBasicInfo['heightVerified'] = user['heightVerified'];
            finalResp.userBasicInfo['countryCode'] = user['countryCode'];
            finalResp.userBasicInfo['phone'] = user['phone'];
            finalResp.userBasicInfo['themeColor'] = user['themeColor'];
            finalResp.userBasicInfo['isVerified'] = user['isVerified'];
            finalResp.userBasicInfo['profilePicture'] = user['profilePicture'] ? '/static/users/' + req.params.id + '/' + FolderNames.PROFILE_PICTURE + '/' + user['profilePicture'] : "";
            finalResp.userBasicInfo['profileThumbnail'] = user['profileThumbnail'] ? '/static/users/' + req.params.id + '/' + FolderNames.PROFILE_PICTURE + '/' + user['profileThumbnail'] : "";
            finalResp.userBasicInfo['defaultNotification'] = user['defaultNotification'];
            finalResp.userBasicInfo['profileBio'] = user['profileBio'] || "";
            finalResp.userBasicInfo['datePageViewCount'] = user['datePageViewCount'] || 0;
            finalResp.userBasicInfo['emailNotification'] = user['emailNotification'] || 0;
            finalResp.userBasicInfo['smsNotification'] = user['smsNotification'] || 0;
            finalResp.userBasicInfo['isLastSeenEnabled'] = user['isLastSeenEnabled'] || 0;
            finalResp.userBasicInfo['instagramToken'] = user['instagramToken'];
            finalResp.userBasicInfo['spotifyToken'] = user['spotifyToken'];
            finalResp.userBasicInfo['tiktokToken'] = user['tiktokToken'];
            finalResp.userBasicInfo['appleMusicToken'] = user['appleMusicToken'];
            finalResp.userBasicInfo['age'] = user['DOB'] == null ? Constants.DEFAULT_AGE : moment().diff(user['DOB'], 'years');
            if (user['SpotifyPlaylists']) {
                finalResp.userBasicInfo['spotifyPlaylists'] = user['SpotifyPlaylists'];
            } else {
                finalResp.userBasicInfo['spotifyPlaylists'] = [];
            }
 
            const workQuestion = await UserCharacteristicService.getCharacteristicsByUserIdQuestionId(req.params.id, Questions.WORKS);
            finalResp.userBasicInfo['works'] = workQuestion[0] ? workQuestion[0].customAns : null;

            const jobTitleQuestion = await UserCharacteristicService.getCharacteristicsByUserIdQuestionId(req.params.id, Questions.JOB_TITLE);
            finalResp.userBasicInfo['jobTitle'] = jobTitleQuestion[0] ? jobTitleQuestion[0].customAns : null;

            const eductaionLevelQuestion = await UserCharacteristicService.getCharacteristicsByUserIdQuestionId(req.params.id, Questions.EDUCATION_LEVEL);
            const universityQuestion = await UserCharacteristicService.getCharacteristicsByUserIdQuestionId(req.params.id, Questions.COLLEGE_UNIVERSITY);
            if (eductaionLevelQuestion.length > 0 && universityQuestion.length > 0) {
                if (eductaionLevelQuestion[0].answers[0].answerText != CommonAnswers.PREFER_NOT_TO_SAY) {
                    finalResp.userBasicInfo['educationLevel'] = eductaionLevelQuestion[0].answers[0].answerText;
                    finalResp.userBasicInfo['university'] = universityQuestion[0].customAns;
                }
            } else {
                finalResp.userBasicInfo['educationLevel'] = null;
                finalResp.userBasicInfo['university'] = null;
            }

            const hometownQuestion = await UserCharacteristicService.getCharacteristicsByUserIdQuestionId(req.params.id, Questions.HOMETOWN);
            finalResp.userBasicInfo['hometown'] = hometownQuestion[0] ? hometownQuestion[0].customAns : null;

            // commented out this code as currently we are considering all users to be premium

            // const userSubscription = await SubscribedUserService.getActiveSubscriptionByUserId(req.params.id);
            // if (userSubscription.length > 0) {
                finalResp.userBasicInfo['isPaidUser'] = 1;
            // } else {
                // finalResp.userBasicInfo['isPaidUser'] = 0;
            // }

            const userOnlineLog = await UserOnlineLogsService.getOnlineLogByUserId(req.params.id);
            if ((!userOnlineLog) || (userOnlineLog && userOnlineLog.offTimestamp && userOnlineLog.offTimestamp !== 0)) {
                finalResp.userBasicInfo['onlineStatus'] = UserLastStatus.OFFLINE;
                finalResp.userBasicInfo['lastSeenTimestamp'] = userOnlineLog && userOnlineLog.offTimestamp || moment().subtract(5, 'minutes').unix();
            } else if (userOnlineLog && userOnlineLog.offTimestamp == 0) {
                finalResp.userBasicInfo['onlineStatus'] = UserLastStatus.ONLINE;
                finalResp.userBasicInfo['lastSeenTimestamp'] = null;
            }

            if (authUser.id == req.params.id) {
                finalResp.userBasicInfo['onlineStatus'] = UserLastStatus.ONLINE;
                finalResp.userBasicInfo['lastSeenTimestamp'] = null;
            }

            if (authUser.id != req.params.id) {
                delete finalResp.userBasicInfo['email'];
                delete finalResp.userBasicInfo['DOB'];
                delete finalResp.userBasicInfo['countryCode'];
                delete finalResp.userBasicInfo['phone'];
                delete finalResp.userBasicInfo['isVerified'];
                delete finalResp.userBasicInfo['defaultNotification'];
                delete finalResp.userBasicInfo['emailNotification'];
                delete finalResp.userBasicInfo['datePageViewCount']
                delete finalResp.userBasicInfo['smsNotification'];
                delete finalResp.userBasicInfo['isLastSeenEnabled'];
                delete finalResp.userBasicInfo['themeColor'];

                if (!user['isLastSeenEnabled']) {
                    finalResp.userBasicInfo['onlineStatus'] = null;
                    finalResp.userBasicInfo['lastSeenTimestamp'] = null;
                }

                finalResp.userBasicInfo['followStatus'] = FollowStatus.FOLLOW;
                finalResp.userBasicInfo['followStatusText'] = FollowStatusText.FOLLOW;

                const followData = await FollowingFollowerService.getUserFollowFollowingData(authUser.id, req.params.id)
                if (followData) {
                    if (followData.status == FollowStatus.FOLLOW_REQUEST_SENT) {
                        finalResp.userBasicInfo['followStatus'] = FollowStatus.FOLLOW_REQUEST_SENT;
                        finalResp.userBasicInfo['followStatusText'] = FollowStatusText.FOLLOW_REQUEST_SENT;
                    }

                    if (followData.status == FollowStatus.FOLLOW_REQUEST_ACCEPTED) {
                        finalResp.userBasicInfo['followStatus'] = FollowStatus.FOLLOW_REQUEST_ACCEPTED;
                        finalResp.userBasicInfo['followStatusText'] = FollowStatusText.FOLLOW_REQUEST_ACCEPTED;
                    }
                }

                const followingData = await FollowingFollowerService.getUserFollowFollowingData(req.params.id, authUser.id)
                if (followingData && followingData.status && followingData.status == FollowStatus.FOLLOW_REQUEST_ACCEPTED) {
                    finalResp.userBasicInfo['followStatus'] = FollowStatus.FOLLOW_BACK;
                    finalResp.userBasicInfo['followStatusText'] = FollowStatusText.FOLLOW_BACK;
                }
            }

            let datingMedia = await DatingPageMediaService.getMediaByUserId(req.params.id);
            finalResp.datingPageMedia = datingMedia.map((media) => {
                media.mediaUrl = '/static/users/' + media.userId + '/' + FolderNames.DATING_PAGE_MEDIA + '/' + media.mediaUrl;
                delete media.userId;
                return media;
            });

            const prefUserPrompts = await UserPromptsService.getUserPromptsByUserId(req.params.id);
            finalResp.prompts = prefUserPrompts.map((prompt) => {
                delete prompt.userId;
                return prompt;
            })

            finalResp.interests = ["Netflix"];
            finalResp.socialMedia = [
                {
                    mediaType: "FACEBOOK",
                    mediaId: "example"
                }
            ];

            finalResp.location = locationInfo || {};

            res.status(HTTPStatusCode.SUCCESS).json({
                user: finalResp,
                message: StaticMessages.getUserMessage('User'),
                status: Constants.SUCCESS
            });
        } else {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getNotExistMessage('User'),
                status: Constants.ERROR
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: "",
            status: Constants.ERROR
        });
    }
};

const getUserList = async (req: Request, res: Response, next: NextFunction) => {
    try {
        const userCount = await UserService.getCountOfAllUsers()
        if (req.query.type !== "") {
            let users = await UserService.getUserList(parseInt(String(req.query.type)), parseInt(String(req.query.limit)), parseInt(String(req.query.offset)));
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getListReturnedMessage("User"),
                status: Constants.SUCCESS,
                userList: { users, userCount }
            });
        } else {
            let users = await UserService.getUserList(Users.ALLUSERS, parseInt(String(req.query.limit)), parseInt(String(req.query.offset)));
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getListReturnedMessage("User"),
                status: Constants.SUCCESS,
                userList: { users, userCount }
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error);
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({ message: error });
    }
};

const getSearchedUsers = async (req: Request, res: Response, next: NextFunction) => {
    try {
        const authUser = req['user'];
        let users = await UserService.getUserListBySearch(authUser.id, String(req.query.search), parseInt(String(req.query.limit)), parseInt(String(req.query.offset)));

        if (users.length == 0) {
            return res.status(HTTPStatusCode.NOTFOUND).json({
                message: StaticMessages.getNotExistMessage("Users"),
                status: Constants.ERROR,
                error: {
                    statusCode: HTTPStatusCode.NOTFOUND,
                    message: StaticMessages.getNotExistMessage("Users"),
                }
            });
        }

        return res.status(HTTPStatusCode.SUCCESS).json({
            users: users,
            message: StaticMessages.getListReturnedMessage("User"),
            status: Constants.SUCCESS
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({ message: error });
    }
};

const updateUserById = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let updateResponse = await UserService.updateUser(req['user'].id, req.body);
        if (updateResponse && updateResponse.length && updateResponse[0] !== '') {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getUpdatedMessage('Profile'),
                status: Constants.SUCCESS
            });
        } else {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getNotExistMessage('User'),
                status: Constants.ERROR
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('User'),
            status: Constants.ERROR
        });
    }
};

const updatePassword = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let updateResponse = await UserService.updatePassword(req.body, req['user'].id);
        if (updateResponse && updateResponse.length && updateResponse[0] !== '' && updateResponse[0] !== 0) {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getUpdatedMessage('Password'),
                status: Constants.SUCCESS,
                updateResponse: updateResponse
            });
        } else {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getNotExistMessage('Password'),
                status: Constants.ERROR
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('Password'),
            status: Constants.ERROR
        });
    }
};

const enableDisableUser = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let updateResponse = await UserService.enableDisableUser(req.body.id, req.body.isActive);
        if (updateResponse && updateResponse.length && updateResponse[0] !== '' && updateResponse[0] !== 0) {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getUpdatedMessage('Status'),
                status: Constants.SUCCESS,
                updateResponse: updateResponse
            });
        } else {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getNotExistMessage('Status'),
                status: Constants.ERROR
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('Status'),
            status: Constants.ERROR
        });
    }
};

const getAllCount = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let count = await UserService.getCountOfAllUserTypes();
        if (count && count[0] !== '' && count[0] !== 0) {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getListReturnedMessage('User Count'),
                status: Constants.SUCCESS,
                count: count
            });
        } else {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getNotExistMessage('User Count'),
                status: Constants.ERROR
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('User Count'),
            status: Constants.ERROR
        });
    }
};

const deleteProfilePicture = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let userData = await UserService.getUserById(req['user'].id);
        if (userData.profilePicture) {
            const profilePictureDeleteHandler = {
                type: MediaSources.PROFILE_PICTURE,
                userId: req['user'].id,
                itemId: '0',
                filePath: userData.profilePicture
            }

            await handleFileDelete(profilePictureDeleteHandler);
        }

        if (userData.profileThumbnail) {
            const profileThumbnailDeleteHandler = {
                type: MediaSources.PROFILE_PICTURE,
                userId: req['user'].id,
                itemId: '0',
                filePath: userData.profileThumbnail
            }

            await handleFileDelete(profileThumbnailDeleteHandler);
        }

        await UserService.deleteProfilePicture(req['user'].id)

        res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getUpdatedMessage('Profile'),
            status: Constants.SUCCESS
        });

    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('User'),
            status: Constants.ERROR
        });
    }
};

const deleteUser = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let userData = await UserService.getUserById(req['user'].id);
        if (userData.profilePicture) {
            const profilePictureDeleteHandler = {
                type: MediaSources.PROFILE_PICTURE,
                userId: req['user'].id,
                itemId: '0',
                filePath: userData.profilePicture
            }

            await handleFileDelete(profilePictureDeleteHandler);
        }

        if (userData.profileThumbnail) {
            const profileThumbnailDeleteHandler = {
                type: MediaSources.PROFILE_PICTURE,
                userId: req['user'].id,
                itemId: '0',
                filePath: userData.profileThumbnail
            }

            await handleFileDelete(profileThumbnailDeleteHandler);
        }

        await UserService.deleteUser(req['user'].id)

        res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getDeletedMessage('User'),
            status: Constants.SUCCESS
        });

    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('User'),
            status: Constants.ERROR
        });
    }
};

const addSocialMediaDetails = async (req: Request, res: Response, next: NextFunction) => {
    try {
        await UserService.addSocialMediaDetails(req['user'].id, req.body);
        res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getUpdatedMessage('Social Media Details'),
            status: Constants.SUCCESS
        });

    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('User'),
            status: Constants.ERROR
        });
    }
};

const addSpotifyPlaylist = async (req: Request, res: Response, next: NextFunction) => {
    try {
        await UserService.addSpotifyPlaylist(req['user'].id, req.body);
        res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getCreatedMessage('Spotify Playlist'),
            status: Constants.SUCCESS
        });

    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('User'),
            status: Constants.ERROR
        });
    }
};

const deleteSpotifyPlaylist = async (req: Request, res: Response, next: NextFunction) => {
    try {
        await UserService.deleteSpotifyPlaylist(req['user'].id);
        res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getDeletedMessage('Spotify Playlist'),
            status: Constants.SUCCESS
        });

    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('User'),
            status: Constants.ERROR
        });
    }
};

const updateHeight = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let updateResponse = await UserService.updateUser(req['user'].id, req.body);
        res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getUpdatedMessage('User Height'),
            status: Constants.SUCCESS,
            updated: updateResponse
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('User'),
            status: Constants.ERROR
        });
    }
};

const getUserHeight = async (req: Request, res: Response, next: NextFunction) => {
    try {
        const heightDetails = await UserService.getUserDetails(req['user'].id, ['height', 'heightVerified']);
        res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getItemMessage('User Height'),
            status: Constants.SUCCESS,
            heightDetails
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('User'),
            status: Constants.ERROR
        });
    }
};

const updateUnits = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let updateResponse = await UserService.updateUser(req['user'].id, req.body);
        res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getUpdatedMessage('Units'),
            status: Constants.SUCCESS,
            updated: updateResponse
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('User'),
            status: Constants.ERROR
        });
    }
};

const getUnits = async (req: Request, res: Response, next: NextFunction) => {
    try {
        const heightDetails = await UserService.getUserDetails(req['user'].id, ['heightUnit', 'distanceUnit']);
        res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getItemMessage('Units'),
            status: Constants.SUCCESS,
            heightDetails
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('User'),
            status: Constants.ERROR
        });
    }
};

const addSpotifyTrack = async (req: Request, res: Response, next: NextFunction) => {
    try {
        await UserService.addSpotifyTrack(req['user'].id, req.params.playlistId, req.body);
        res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getCreatedMessage('Spotify Track'),
            status: Constants.SUCCESS
        });

    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('User'),
            status: Constants.ERROR
        });
    }
};

export default {
    getUserById,
    getUserList,
    getSearchedUsers,
    updateUserById,
    updatePassword,
    enableDisableUser,
    getAllCount,
    deleteProfilePicture,
    deleteUser,
    addSocialMediaDetails,
    addSpotifyPlaylist,
    addSpotifyTrack,
    updateHeight,
    getUserHeight,
    updateUnits,
    getUnits,
    deleteSpotifyPlaylist
};
