import express from 'express';
import usersController from './users.controller';
import { tokenValidation } from '../../middlewares/validateToken';
import userValidator from './user.validator';
const router = express.Router();

router.get('/users', tokenValidation,  userValidator.getUserList, usersController.getUserList);
router.get('/users/profile/:id', tokenValidation, usersController.getUserById);
router.get('/users/search', tokenValidation, userValidator.getProfileSearchValidator, usersController.getSearchedUsers);
router.put('/users/profileUpdate', tokenValidation, userValidator.updateUserById , usersController.updateUserById);
router.put('/users/updatePassword', tokenValidation, userValidator.updatePassword, usersController.updatePassword);
router.put('/users', tokenValidation, usersController.enableDisableUser);
router.put('/user/height', tokenValidation, usersController.updateHeight);
router.get('/user/height', tokenValidation, usersController.getUserHeight);
router.put('/user/units', tokenValidation, usersController.updateUnits);
router.get('/user/units', tokenValidation, usersController.getUnits);
router.get('/users/count', tokenValidation, usersController.getAllCount);
router.delete('/users/profilePicture', tokenValidation, usersController.deleteProfilePicture);
router.delete('/user/account', tokenValidation, usersController.deleteUser);
router.post('/user/socialMedia', tokenValidation, usersController.addSocialMediaDetails);
router.post('/user/spotify/playlist', tokenValidation, usersController.addSpotifyPlaylist);
router.delete('/user/spotify/playlist', tokenValidation, usersController.deleteSpotifyPlaylist);
router.post('/user/spotify/track/:playlistId', tokenValidation, usersController.addSpotifyTrack); 

export default router;